/** Copyright 2012
*
* Author: Ismael PONCE GORDILLO
*
* Licensed under the EUPL, Version 1.1 or – as soon they will be
* approved by the European Commission – subsequent versions of the
* EUPL (the "Licence"); you may not use this work except in
* compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* https://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the Licence is distributed on an "AS
* IS" BASIS, * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
* express or implied.
* See the Licence for the specific language governing permissions
* and limitations under the Licence.
*/
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
//
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.FileDialog;

public class BBDD {
	public static final String apendBBDD="_bbdd";
	public static final String apendTemporal="_Temporal";
	public String dirBBDD;
	public static final String dirViejoBBDD="pCuaderno_bbdd";
	public static final String nombreBBDD="bbdd.hsqldb";
	public String archivo;
	public String archivoLCK;
	public String archivoLOG;
	public static final String extSCRIPT=".script";
	public String archivoSCRIPT;
	public static final String extPROPERTIES=".properties";
	public String archivoPROPERTIES;
	public String archivoDUMP;
	private String cadenaConexion=null;
	private Connection conexion=null;
	public static final int idTipoEvaluacion=1;
	public static final int idTipoClaustro=2;
	public static final int idTipoConsejoEscolar=3;
	public static final int idTipoCitaConTutores=4;
	public static final int idTipoETCP=5;
	public static final int idTipoOtroTipo=6;
	public static final String expTipoTodoss=PCuaderno.MiPCuaderno.mensajes.getString("AllM");
	public static final int idTipoTodos=0;
	public static final String verdadero="TRUE";
	public static final String expOrdinario=PCuaderno.MiPCuaderno.mensajes.getString("Ordinary");
	public static final String expExtraordinario=PCuaderno.MiPCuaderno.mensajes.getString("Extraordinary");
	public static final String sqlDemoradoINFO="<<INFO>>";
	public static final String expSinGrupoEvaluacion=PCuaderno.MiPCuaderno.mensajes.getString("NoGroup");
	public static final String expNuevoGrupoEvaluacion=PCuaderno.MiPCuaderno.mensajes.getString("NewGroup");
	public static final String expNoMoverGrupo="";
	public static final String expNo=PCuaderno.MiPCuaderno.mensajes.getString("No");
	public static final String expDiaCompleto=PCuaderno.MiPCuaderno.mensajes.getString("FullDay");
	public static final String idDiaCompleto="#DiaCompleto";
	public static final String expTodos=PCuaderno.MiPCuaderno.mensajes.getString("AllM");
	public static final String expTodas=PCuaderno.MiPCuaderno.mensajes.getString("AllF");
	public static final String idTodos="#Todos";
	public static final String conjuntoCaracteresEstudiantes="ISO-8859-1";//ISO-8859-1,UTF-8,windows-1250
	public static final String conjuntoCaracteresHSQLDB="Unicode";
	public static final int bufferArchivo=2048;
	public BBDD(String dir){
		this.dirBBDD=dir;
		this.archivo=this.dirBBDD+File.separatorChar+BBDD.nombreBBDD;
		this.archivoLCK=this.archivo+".lck";
		this.archivoLOG=this.archivo+".log";
		this.archivoSCRIPT=this.archivo+extSCRIPT;
		this.archivoPROPERTIES=this.archivo+extPROPERTIES;
		this.archivoDUMP=this.archivo+".dump";
	}
	public void conexionFijaCadenaAbrir(String cadena){
		conexionFijaCadenaPlana();
		if (!conexionComprobar())
			conexionFijaCadenaCifrada(cadena);
	}
	public void conexionFijaCadenaCrear(String cadena){
		if (cadena.equals("")) 
			conexionFijaCadenaPlana();
		else
			conexionFijaCadenaCifrada(cadena);
	}
	public void conexionFijaCadenaPlana(){
		this.cadenaConexion="jdbc:hsqldb:file:"+this.archivo+";hsqldb.write_delay=false;shutdown=true";
	}
	public void conexionFijaCadenaCifrada(String cadena){
		this.cadenaConexion="jdbc:hsqldb:file:"+this.archivo+";crypt_key="+cadena+";crypt_type=AES;hsqldb.write_delay=false;shutdown=true";
	}
	public boolean conexionDriverComprobar(){
		boolean resultado=false;
		try {Class.forName("org.hsqldb.jdbc.JDBCDriver");resultado=true;
		} catch (ClassNotFoundException e) {
			System.out.println("this.compruebaDriverConexion - Can't found driver.");
			e.printStackTrace();}
		if (resultado) resultado=conexionComprobar();
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() result "+resultado);
		return resultado;
	}
	public boolean conexionComprobar(){
		System.out.println("Check connection BBDD in forlder "+this.archivo);
		String cadena=this.cadenaConexion;
		boolean resultado=true;
		Connection tConexion = null;
		try {tConexion = DriverManager.getConnection(cadena);tConexion.close();
		} catch (SQLException e) {
			//System.out.println("Cadena de conexión: "+this.cadenaConexion);
			System.out.println("this.compruebaConexion - Can't connect to database.");
			//System.out.println(e.toString());
			resultado=false;
			}
		return resultado;
	}
	public void borrarParcial(String idParcial){
		int ejecucion=-1;
		if (idParcial!=null) {
			if (!idParcial.equals("")) {
				String sql="DELETE FROM PARCIALES WHERE PARCIALES.ID="+idParcial+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idParcial+") deleted "+ejecucion);
	}
	public String insertarParcial(String idMateria,Calendar inicio, Calendar fin){
		String id=null;
		if (idMateria!=null && inicio!=null && fin!=null) if (!idMateria.equals("")){
			String sql="INSERT INTO PARCIALES (MATERIA,INICIO,FIN)"+
					" VALUES ("+idMateria+",'"+Fecha.bbddSoloDia(inicio)+"','"+Fecha.bbddSoloDia(fin)+"');";
			id = this.sqlInsertarId("PARCIALES",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+Fecha.formaFechaDiaHora(inicio)+","+Fecha.formaFechaDiaHora(fin)+") id inserted "+id);
		return id;
	}
	public void modificarParcial(String idParcial, Calendar inicio, Calendar fin){
		int ejecucion=-1;
		if (idParcial!=null && inicio!=null && fin!=null) {
			if (!idParcial.equals("")) {
				String sql="UPDATE PARCIALES SET PARCIALES.INICIO='"+Fecha.bbddSoloDia(inicio)+
						"',PARCIALES.FIN='"+Fecha.bbddSoloDia(fin)+"'"+
				" WHERE PARCIALES.ID="+idParcial+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idParcial+","+Fecha.formaFechaDiaHora(inicio)+") modified "+ejecucion);
	}
	public Vector<Vector<String>> consultarParcial(String idMateria,String idParcial){
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null) if (!idMateria.equals("")) {
			String sql="SELECT ID,MATERIA,DAYOFMONTH(INICIO),MONTH(INICIO),YEAR(INICIO)" +
					",DAYOFMONTH(FIN),MONTH(FIN),YEAR(FIN) FROM PARCIALES"+
					" WHERE PARCIALES.MATERIA="+idMateria+
					((idParcial!=null)?" AND PARCIALES.ID="+idParcial:"")+
					" ORDER BY INICIO ASC;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+idParcial+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get teaching (can be null, get all teaching)
	 * @return teaching info <br>
	 * 0 - ENSENANZAS.ID<br>
	 * 1 - ENSENANZAS.REFERENCIA<br>
	 * 2 - ENSENANZAS.DESCRIPCION<br>
	 */
	public Vector<Vector<String>> consultarEnsenanza(String idEnsenanza){
		String sql="SELECT ENSENANZAS.ID,ENSENANZAS.REFERENCIA,ENSENANZAS.DESCRIPCION FROM ENSENANZAS"+
				((idEnsenanza!=null)?" WHERE ENSENANZAS.ID="+idEnsenanza:"")+
				" ORDER BY REFERENCIA;";
		Vector<Vector<String>> tabla = this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public String insertarEnsenanza(String referencia, String ensenanza){
		String id=null;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+referencia+","+ensenanza+") "+id);
		if (referencia!=null && ensenanza!=null) 
			if (!referencia.equals("")) {
				String sql="INSERT INTO ENSENANZAS (REFERENCIA,DESCRIPCION)"+
			" VALUES ('"+referencia+"','"+ensenanza+"');";
			id = this.sqlInsertarId("ENSENANZAS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+referencia+","+ensenanza+") id inserted = "+id);
		return id;
	}
	public void borrarEnsenanza(String idEnsenanza){
		int ejecucion=-1;
		if (idEnsenanza!=null) {
			if (!idEnsenanza.equals("")) {
				Vector<Vector<String>> titulos = this.consultarTitulo(idEnsenanza,null);
				if (titulos!=null) if (titulos.size()==0) {
					String sql="DELETE FROM ENSENANZAS WHERE ENSENANZAS.ID="+idEnsenanza+";";
					ejecucion=this.sqlEjecuta(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+") deleted "+ejecucion);
	}
	/** 
	 * Get title (can be null, get all title)
	 * @return teaching info <br>
	 * 0 - TITULOS.ID<br>
	 * 1 - TITULOS.ENSENANZA<br>
	 * 2 - TITULOS.REFERENCIA<br>
	 * 3 - TITULOS.DESCRIPCION<br>
	 */
	public Vector<Vector<String>> consultarTitulo(String idEnsenanza,String idTitulo){
		String sql="SELECT TITULOS.ID,TITULOS.ENSENANZA,TITULOS.REFERENCIA,TITULOS.DESCRIPCION FROM TITULOS"+
				((idEnsenanza!=null || idTitulo!=null)?" WHERE":"")+
				((idEnsenanza!=null)?" TITULOS.ENSENANZA="+idEnsenanza:"")+
				((idEnsenanza!=null && idTitulo!=null)?" AND":"")+
				((idTitulo!=null)?" TITULOS.ID="+idTitulo:"")+
				" ORDER BY REFERENCIA;";
		Vector<Vector<String>> tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public String insertarTitulo(String idEnsenanza,String referencia,String descripcion){
		String id=null;
		if (idEnsenanza!=null && referencia!=null && descripcion!=null) 
			if (!idEnsenanza.equals("") && !referencia.equals("")) {
				String sql="INSERT INTO TITULOS (ENSENANZA,REFERENCIA,DESCRIPCION)"+
			" VALUES ('"+idEnsenanza+"','"+referencia+"','"+descripcion+"');";
			id = this.sqlInsertarId("TITULOS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+","+referencia+","+descripcion+") id inserted = "+id);
		return id;
	}
	public void borrarTitulo(String idTitulo){
		int ejecucion=-1;
		if (idTitulo!=null) {
			if (!idTitulo.equals("")) {
				Vector<Vector<String>> titulos = this.consultarNivel(idTitulo,null);
				if (titulos!=null) if (titulos.size()==0) {
					String sql="DELETE FROM TITULOS WHERE TITULOS.ID="+idTitulo+";";
					ejecucion=this.sqlEjecuta(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTitulo+") deleted "+ejecucion);
	}
	/** 
	 * Get level (can be null, get all level)
	 * @return teaching info <br>
	 * 0 - NIVELES.ID<br>
	 * 1 - NIVELES.TITULO<br>
	 * 2 - NIVELES.REFERENCIA<br>
	 */
	public Vector<Vector<String>> consultarNivel(String idTitulo,String idNivel){
		String sql="SELECT NIVELES.ID,NIVELES.TITULO,NIVELES.REFERENCIA FROM NIVELES"+
				((idTitulo!=null || idNivel!=null)?" WHERE":"")+
				((idTitulo!=null)?" NIVELES.TITULO="+idTitulo:"")+
				((idTitulo!=null && idNivel!=null)?" AND":"")+
				((idNivel!=null)?" NIVELES.ID="+idNivel:"")+
				" ORDER BY NIVELES.REFERENCIA;";
		Vector<Vector<String>> tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTitulo+","+idNivel+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public String insertarNivel(String idTitulo,String referencia){
		String id=null;
		if (idTitulo!=null && referencia!=null) 
			if (!idTitulo.equals("") && !referencia.equals("")) {
				String sql="INSERT INTO NIVELES (TITULO,REFERENCIA)"+
			" VALUES ("+idTitulo+",'"+referencia+"');";
			id = this.sqlInsertarId("NIVELES",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTitulo+","+referencia+") id inserted = "+id);
		return id;
	}
	public void borrarNivel(String idNivel){
		int ejecucion=-1;
		if (idNivel!=null) {
			if (!idNivel.equals("")) {
				Vector<Vector<String>> clases = this.consultarClase(idNivel,null);
				if (clases!=null) if (clases.size()==0) {
					String sql="DELETE FROM NIVELES WHERE NIVELES.ID="+idNivel+";";
					ejecucion=this.sqlEjecuta(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+") deleted "+ejecucion);
	}
	public String insertarClase(String idNivel,String referencia){
		String id=null;
		int ejecucion=0;
		String tutor=null;
		Vector<Vector<String>> materias=null;
		Vector<Vector<String>> profesores=this.consultarProfesor(null);
		if (profesores!=null) if (profesores.size()>0) tutor=profesores.get(0).get(0);
		if (idNivel!=null && referencia!=null && tutor!=null) 
			if (!idNivel.equals("") && !referencia.equals("")) {
				String sql="INSERT INTO CLASES (NIVEL,REFERENCIA,TUTOR)" +
						" VALUES ("+idNivel+",'"+referencia+"',"+tutor+");";
				id = this.sqlInsertarId("CLASES",sql);
				sql="";
				materias = this.consultarMateria(idNivel, null);
				if (materias!=null) if (materias.size()>0) {
					for (int i=0;i<materias.size();i++) 
						sql+="INSERT INTO MATERIA_CLASE (MATERIA,CLASE) VALUES ("+materias.get(i).get(0)+","+id+");";
					ejecucion=this.sqlEjecuta(sql);
				}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+referencia+") tutor ("+tutor+") subjects ("+((materias!=null)?materias.size():"null")+") id inserted = "+id+" records inserted "+ejecucion);
		return id;
	}
	public void borrarClase(String idClase){
		int ejecucion=-1;
		Vector<Vector<String>> estudiantes=null;
		Vector<Vector<String>> uhs=null;
		Vector<Vector<String>> evaluaciones=null;
		if (idClase!=null) {
			if (!idClase.equals("")) {
				estudiantes = this.consultarEstudiantesClase(idClase,null);
				if (estudiantes==null || estudiantes.size()==0) {
					evaluaciones = this.sqlConsulta("SELECT EVALUACIONES.ID FROM EVALUACIONES WHERE EVALUACIONES.CLASE="+idClase+";");
					String sql="";
					if (evaluaciones!=null) if (evaluaciones.size()>0) {
						for (int i=0;i<evaluaciones.size();i++) sql+="DELETE FROM EVALUACIONES WHERE EVALUACIONES.ID="+evaluaciones.get(i).get(0)+";"+
								" DELETE FROM EVENTOS WHERE EVENTOS.ID="+evaluaciones.get(i).get(0)+";";
					}
					uhs = this.consultarUHMateriaClase(null, idClase);
					if (uhs!=null) if (uhs.size()>0) {
						for (int i=0;i<uhs.size();i++) sql+="DELETE FROM UH_SEMANA WHERE UH_SEMANA.ID="+uhs.get(i).get(0)+";";
					}
					sql+="DELETE FROM PROGRAMACION_TEMPORAL WHERE PROGRAMACION_TEMPORAL.CLASE="+idClase+";"+
							" DELETE FROM DIASNOLECTIVOS WHERE DIASNOLECTIVOS.CLASE="+idClase+";"+
							" DELETE FROM MATERIA_CLASE WHERE MATERIA_CLASE.CLASE="+idClase+";"+
							" DELETE FROM CLASES WHERE CLASES.ID="+idClase+";";
					ejecucion=this.sqlEjecuta(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") students ("+((estudiantes!=null)?estudiantes.size():"null")+") evaluations ("+((evaluaciones!=null)?evaluaciones.size():"null")+") classSchedule ("+((uhs!=null)?uhs.size():"null")+") total deleted "+ejecucion);
	}
	public void borrarMateria(String idMateria){
		int ejecucion=-1;
		Vector<Vector<String>> estudiantes=null;
		Vector<Vector<String>> tiposEvaluable=null;
		Vector<Vector<String>> uhs=null;
		Vector<Vector<String>> nodos=null;
		if (idMateria!=null) {
			if (!idMateria.equals("")) {
				estudiantes = this.consultarEstudiantesMateria(idMateria,null);
				tiposEvaluable = this.consultarTiposEvaluablesParaMateria(idMateria);
				if ((estudiantes==null||estudiantes.size()==0) && (tiposEvaluable==null ||tiposEvaluable.size()==0)) {
					String sql="DELETE FROM PARCIALES WHERE PARCIALES.MATERIA="+idMateria+";";
					nodos = this.sqlConsulta("SELECT ID FROM PROGRAMACION_NODO WHERE PROGRAMACION_NODO.MATERIA="+idMateria+";");
					for (int i=0;i<nodos.size();i++) {
						sql+="DELETE FROM PROGRAMACION_TEMPORAL WHERE PROGRAMACION_TEMPORAL.NODO="+nodos.get(i).get(0)+";";
					}
					sql+="DELETE FROM PROGRAMACION_NODO WHERE PROGRAMACION_NODO.MATERIA="+idMateria+";";
					uhs = this.consultarUHMateriaClase(idMateria,null);
					if (uhs.size()>0) {
						for (int i=0;i<uhs.size();i++) sql+="DELETE FROM UH_SEMANA WHERE UH_SEMANA.ID="+uhs.get(i).get(0)+";";
					}
					sql+="DELETE FROM MATERIA_CLASE WHERE MATERIA_CLASE.MATERIA="+idMateria+";" +
							"DELETE FROM MATERIAS WHERE MATERIAS.ID="+idMateria+";";
					ejecucion=this.sqlEjecuta(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+")" +" students ("+((estudiantes!=null)?estudiantes.size():"null")+")" +" tiposEvaluables ("+((tiposEvaluable!=null)?tiposEvaluable.size():"null")+")" +" nodos ("+((nodos!=null)?nodos.size():"null")+")" +" uhs ("+((uhs!=null)?uhs.size():"null")+") total deleted "+ejecucion);
	}
	/** 
	 * Get level (can be null, get all level)
	 * @return teaching info <br>
	 * 0 - MATERIAS.ID<br>
	 * 1 - MATERIAS.REFERENCIA<br>
	 * 2 - MATERIAS.DESCRIPCION<br>
	 * 3 - MATERIAS.NIVEL<br>
	 */
	public Vector<Vector<String>> consultarMateria(String idNivel,String idMateria){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT MATERIAS.ID,MATERIAS.REFERENCIA,MATERIAS.DESCRIPCION,MATERIAS.NIVEL FROM MATERIAS"+
				((idNivel!=null || idMateria!=null)?" WHERE":"")+
				((idNivel!=null)?" MATERIAS.NIVEL="+idNivel:"")+
				((idNivel!=null && idMateria!=null)?" AND":"")+
				((idMateria!=null)?" MATERIAS.ID="+idMateria:"")+
				" ORDER BY MATERIAS.REFERENCIA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public void modificarEvaluableRecupera(String idEvaluable,String idRecupera){
		int ejecucion=-1;
		if (idEvaluable!=null && idRecupera!=null) {
			if (!idEvaluable.equals("")) {
				String sql="UPDATE EVALUABLES SET EVALUABLES.RECUPERA="+((idRecupera.equals(""))?"NULL":idRecupera)+
				" WHERE EVALUABLES.ID="+idEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+","+idRecupera+") records modified "+ejecucion);
	}
	public void modificarEvaluablePeso(String idEvaluable,String peso){
		int ejecucion=-1;
		if (idEvaluable!=null && peso!=null) {
			if (!idEvaluable.equals("") && !peso.equals("")) {
				String sql="UPDATE EVALUABLES SET EVALUABLES.PESO="+peso+
				" WHERE EVALUABLES.ID="+idEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+","+peso+") records modified "+ejecucion);
	}
	public void modificarTipoDeEvaluable(String idEvaluable, String idTipo){
		int ejecucion=-1;
		if (idEvaluable!=null && idTipo!=null) {
			if (!idEvaluable.equals("") && !idTipo.equals("")) {
				String sql="UPDATE EVALUABLES SET EVALUABLES.TIPO="+idTipo+
				" WHERE EVALUABLES.ID="+idEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+","+idTipo+") records modified "+ejecucion);
	}
	public String insertarProgramacionNodo(String enunciado, String idPadre, Integer posicion, String idMateria){
		String id=null;
		if (idPadre!=null && posicion!=null && idMateria!=null) 
			if (!idPadre.equals("") && posicion>=0 && !idMateria.equals("")) {
				String sql="INSERT INTO PROGRAMACION_NODO (PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO)"+
						" VALUES ("+idPadre+","+posicion+","+idMateria+",'"+enunciado+"','');";
				id = this.sqlInsertarId("PROGRAMACION_NODO",sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+enunciado+","+idPadre+","+posicion+","+idMateria+") id inserted = "+id);
		return id;
	}
	public String insertarProgramacionNodoHermano(String idHermano){
		String id=null;
		Vector<Vector<String>> hermanosPosteriores=null;
		if (idHermano!=null) 
			if (!idHermano.equals("")) {
				Vector<Vector<String>> nodo = this.consultarProgramacionNodo(idHermano);
				String idPadre = nodo.get(0).get(1);
				Integer posNodo = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD insertarProgramcionNodoHermano (padre "+idPadre+", posNodo"+posNodo+")");
				hermanosPosteriores=null;
				String sql="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE"+((idPadre==null)?" IS NULL AND MATERIA="+nodo.get(0).get(3):"="+idPadre)+
						" AND POSICION>"+posNodo+
						" ORDER BY POSICION ASC;";
				hermanosPosteriores=this.sqlConsulta(sql);
				//System.out.println("BBDD insertarProgramcionNodoHermano (hermanos "+hermanosPosteriores.size()+")");
				sql="";
				if (hermanosPosteriores!=null) if (hermanosPosteriores.size()>0) {
					for (int i=0;i<hermanosPosteriores.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+posNodo+2)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPosteriores.get(i).get(0)+";";
					}
				}
				sql+="INSERT INTO PROGRAMACION_NODO (PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO)" +
						" VALUES ("+((idPadre==null)?"NULL":idPadre)+","+(posNodo+1)+","+nodo.get(0).get(3)+",'"+PCuaderno.MiPCuaderno.mensajes.getString("Programacion_BtnCrearNodo_Nuevo")+"','');";
				id = this.sqlInsertarId("PROGRAMACION_NODO",sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idHermano+") following brother ("+((hermanosPosteriores!=null)?hermanosPosteriores.size():"null")+")id inserted = "+id);
		return id;
	}
	public void insertarProgramacionDuracionNodo(String idNodo, String idClase, Integer duracion){
		int ejecucion=-1;
		if (idNodo!=null && duracion!=null && idClase!=null) 
			if (!idNodo.equals("") && duracion>=0 && !idClase.equals("")) {
				String sql="INSERT INTO PROGRAMACION_TEMPORAL (NODO,CLASE,DURACION)"+
						" VALUES ("+idNodo+","+idClase+","+duracion+");";
				ejecucion=this.sqlEjecuta(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+idClase+","+duracion+") inserted = "+ejecucion);
	}
	public void borrarProgramacionDuracionNodo(String idNodo, String idClase){
		int ejecucion=-1;
		if (idNodo!=null && idClase!=null) 
			if (!idNodo.equals("") && !idClase.equals("")) {
				String sql="DELETE FROM PROGRAMACION_TEMPORAL" +
						" WHERE PROGRAMACION_TEMPORAL.NODO="+idNodo+
						" AND PROGRAMACION_TEMPORAL.CLASE="+idClase+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+idClase+") inserted = "+ejecucion);
	}
	public Vector<Vector<String>> consultarProgramacionDuracionNodo(String idNodo,String idClase){
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null && idClase!=null) {
			if (!idNodo.equals("") && !idClase.equals("")) {
				String sql="SELECT NODO,CLASE,DURACION FROM PROGRAMACION_TEMPORAL" +
						" WHERE NODO="+idNodo+" AND CLASE="+idClase+";";
				tabla=this.sqlConsulta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+idClase+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public void borrarProgramacionNodoReOrdena(String idNodo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")");
		int ejecucion = -1;
		Vector<Vector<String>> hermanosPosteriores=null;
		if (idNodo!=null) {
			if (!idNodo.equals("")) {
				Vector<Vector<String>> nodo=this.consultarProgramacionNodo(idNodo);
				if (nodo!=null) {
					String idPadre = nodo.get(0).get(1);
					Integer posNodo = Integer.valueOf(nodo.get(0).get(2));
					borrarProgramacionNodo(idNodo);
					String sql="";
					String sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
					" WHERE PADRE"+((idPadre==null)?" IS NULL AND MATERIA="+nodo.get(0).get(3):"="+idPadre)+
					" AND POSICION>"+posNodo+
					" ORDER BY POSICION ASC;";
					hermanosPosteriores=this.sqlConsulta(sql2);
					if (hermanosPosteriores!=null) if (hermanosPosteriores.size()>0) {
						for (int i=0;i<hermanosPosteriores.size();i++){
							sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+posNodo)+
									" WHERE PROGRAMACION_NODO.ID="+hermanosPosteriores.get(i).get(0)+";";
						}
					}
					if (!sql.equals("")) ejecucion=this.sqlEjecuta(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")" +" following brother ("+((hermanosPosteriores!=null)?hermanosPosteriores.size():"null")+")" +" total records modified "+ejecucion);
	}
	public void borrarProgramacionNodo(String idNodo){
		int ejecucion=-1;
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null) {
			if (!idNodo.equals("")) {
				tabla = this.consultarProgramacionNodoHijos(idNodo,null);
				if (tabla!=null) if (tabla.size()>0) {
					for (int i=0;i<tabla.size();i++){
						borrarProgramacionNodoReOrdena(tabla.get(i).get(0));
					}
				}
				String sql="DELETE FROM PROGRAMACION_TEMPORAL WHERE PROGRAMACION_TEMPORAL.NODO="+idNodo+";";
				sql+="DELETE FROM PROGRAMACION_NODO WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+") sons ("+((tabla!=null)?tabla.size():"null")+") deleted "+ejecucion);
	}
	public void modificarProgramcionNodoBajarNodo(String idNodo, String idNuevoPadre,boolean primero){
		int ejecucion=-1;
		//System.out.println("BBDD modificarProgramcionNodoBajarNodo ("+idNodo+","+idNuevoPadre+","+primero+")");
		Vector<Vector<String>> hermanosPosteriores=null;
		Vector<Vector<String>> hermanosNuevos=null;
		if (idNodo!=null && idNuevoPadre!=null) {
			if (!idNodo.equals("") && !idNuevoPadre.equals("")) {
				Vector<Vector<String>> nodo = this.consultarProgramacionNodo(idNodo);
				String idPadre = nodo.get(0).get(1);
				Integer posNodo = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD modificarProgramcionNodoBajarNodo (padre "+idPadre+", posNodo"+posNodo+")");
				// adelantar los hermanos posteriores, si primero poner en 0 y correr los demas hermanos sino el último
				String sql="";
				String sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE"+((idPadre==null)?" IS NULL AND MATERIA="+nodo.get(0).get(3):"="+idPadre)+
						" AND POSICION>"+posNodo+
						" ORDER BY POSICION ASC;";
				hermanosPosteriores=this.sqlConsulta(sql2);
				//System.out.println("BBDD modificarProgramcionNodoBajarNodo (hermanos "+hermanosPosteriores.size()+")");
				if (hermanosPosteriores!=null) if (hermanosPosteriores.size()>0) {
					for (int i=0;i<hermanosPosteriores.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+posNodo)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPosteriores.get(i).get(0)+";";
					}
				}
				sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE="+idNuevoPadre+" ORDER BY POSICION ASC;";
				hermanosNuevos=this.sqlConsulta(sql2);
				//System.out.println("BBDD modificarProgramcionNodoBajarNodo (hermanosNuevos "+hermanosPosteriores.size()+")");
				if (primero){
					//System.out.println("BBDD modificarProgramcionNodoBajarNodo (primero y desplaza los demás)");
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION=1" +
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.PADRE="+idNuevoPadre+
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
					if (hermanosNuevos!=null) if (hermanosNuevos.size()>0) {
						for (int i=0;i<hermanosNuevos.size();i++){
							sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+2)+
									" WHERE PROGRAMACION_NODO.ID="+hermanosNuevos.get(i).get(0)+";";
						}
					}
				} else {
					//System.out.println("BBDD modificarProgramcionNodoBajarNodo (despues del último)");
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(hermanosNuevos.size()+1)+
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.PADRE="+idNuevoPadre+
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				}
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")" +" following brother ("+((hermanosPosteriores!=null)?hermanosPosteriores.size():"null")+")" +" new following brother ("+((hermanosNuevos!=null)?hermanosNuevos.size():"null")+")" +" total records modified "+ejecucion);
	}
	public void modificarProgramcionNodoSubirNodo(String idNodo){
		int ejecucion=-1;
		Vector<Vector<String>> hermanosPosterioresAntiguos=null;
		Vector<Vector<String>> hermanosPadrePosteriores=null;
		//System.out.println("BBDD modificarProgramcionNodoSubirNodo ("+idNodo+")");
		if (idNodo!=null) {
			if (!idNodo.equals("")) {
				Vector<Vector<String>> nodo = this.consultarProgramacionNodo(idNodo);
				String idPadre = nodo.get(0).get(1);
				Integer posNodo = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo nodo (padre "+idPadre+", posNodo"+posNodo+")");
				nodo = this.consultarProgramacionNodo(idPadre);
				String idAbuelo = nodo.get(0).get(1);
				Integer posPadre = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo padre (abuelo "+idAbuelo+", posPadre"+posPadre+")");
				if (idAbuelo==null) idAbuelo="NULL";
				String sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.PADRE="+idAbuelo+
						" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+posPadre+
						" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				String sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE="+idPadre+" AND POSICION>"+posNodo+
						" ORDER BY POSICION ASC;";
				hermanosPosterioresAntiguos=this.sqlConsulta(sql2);
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo (hermanos "+hermanosPosterioresAntiguos.size()+")");
				if (hermanosPosterioresAntiguos!=null) if (hermanosPosterioresAntiguos.size()>0) {
					for (int i=0;i<hermanosPosterioresAntiguos.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+posNodo)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPosterioresAntiguos.get(i).get(0)+";";
					}
				}
				sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(1+posPadre)+
				" WHERE PROGRAMACION_NODO.ID="+idPadre+";";
				sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE"+((idAbuelo.equals("NULL"))?" IS NULL AND MATERIA="+nodo.get(0).get(3):"="+idAbuelo)+
						" AND POSICION>"+posPadre+
						" ORDER BY POSICION ASC;";
				//System.out.println("sql (padre "+posPadre+") "+sql2);
				hermanosPadrePosteriores=this.sqlConsulta(sql2);
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo (hermanosPadre "+hermanosPadrePosteriores.size()+")");
				if (hermanosPadrePosteriores!=null) if (hermanosPadrePosteriores.size()>0) {
					for (int i=0;i<hermanosPadrePosteriores.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(2+i+posPadre)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPadrePosteriores.get(i).get(0)+";";
					}
				}
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")" +" old following brother ("+((hermanosPosterioresAntiguos!=null)?hermanosPosterioresAntiguos.size():"null")+")" +" following father's brother ("+((hermanosPadrePosteriores!=null)?hermanosPadrePosteriores.size():"null")+")" +" total records modified "+ejecucion);
	}
	public void modificarProgramcionNodoIntercambiaPos(String idNodoA, String idNodoB){
		int ejecucion=-1;
		if (idNodoA!=null && idNodoB!=null) {
			if (!idNodoA.equals("") && !idNodoB.equals("")) {
				Vector<Vector<String>> nodo = this.consultarProgramacionNodo(idNodoA);
				String posA = nodo.get(0).get(2); 
				nodo = this.consultarProgramacionNodo(idNodoB);
				String posB = nodo.get(0).get(2);
				String sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION='"+posB+"'" +
						" WHERE PROGRAMACION_NODO.ID="+idNodoA+";" +
						" UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION='"+posA+"'" +
						" WHERE PROGRAMACION_NODO.ID="+idNodoB+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodoA+","+idNodoB+") records modified "+ejecucion);
	}
	public void modificarProgramcionNodoEnunciado(String idNodo, String enunciado){
		int ejecucion=-1;
		if (idNodo!=null && enunciado!=null) {
			if (!idNodo.equals("") && !enunciado.equals("")) {
				String sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.ENUNCIADO='"+enunciado+"'"+
				" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+enunciado+") records modified "+ejecucion);
	}
	public void modificarProgramcionNodosDuraciones(String idClase, Vector<Vector<String>> datos){
		int ejecucion=-1;
		if (idClase!=null && datos!=null) {
			if (!idClase.equals("") && datos.size()>0) {
				Vector<String> sql = new Vector<String>();
				for (int i=0;i<datos.size();i++) sql.add("UPDATE PROGRAMACION_TEMPORAL SET DURACION='"+datos.get(i).get(1)+"'" +
						" WHERE CLASE="+idClase+" AND NODO="+datos.get(i).get(0)+";");
				ejecucion=this.sqlEjecutaVarias(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+((datos!=null)?datos.size():"null")+") records inserted "+ejecucion);
	}
	public void modificarProgramcionNodoDuracion(String idClase, String idNodo, String duracion){
		int ejecucion=-1;
		if (idClase!=null && idNodo!=null && duracion!=null) {
			if (!idClase.equals("") && !idNodo.equals("") && !duracion.equals("")) {
				String sql="UPDATE PROGRAMACION_TEMPORAL SET DURACION='"+duracion+"'"+
				" WHERE CLASE="+idClase+" AND NODO="+idNodo+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+idNodo+","+duracion+") records modified "+ejecucion);
	}
	public Vector<Vector<String>> consultarProgramacionNodo(String idNodo){
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null) {
			if (idNodo!="") {
				String sql="SELECT ID,PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO" +
						" FROM PROGRAMACION_NODO" +
						" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				tabla=this.sqlConsulta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarProgramacionNodoHijos(String idPadre,String idMateria){
		Vector<Vector<String>> tabla=null;
		if (idPadre!=null || idMateria!=null) {
			String sql="SELECT ID,PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO FROM PROGRAMACION_NODO" +
					" WHERE "+((idPadre!=null)?"PADRE="+idPadre:"PADRE IS NULL AND MATERIA="+idMateria)+
					" ORDER BY POSICION ASC;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idPadre+","+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/*public Vector<Vector<String>> consultarProgramacionNodosEnunciadoss(String idMateria, String idPadre){
		Vector<Vector<String>> tabla=consultarProgramacionNodoEnunciadosRecursivo(idMateria,idPadre, null);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarProgramacionNodoEnunciadosRecursivo(String idMateria, String idPadre, Vector<Vector<String>> anteriores){
		Vector<Vector<String>> salida=(anteriores==null)?new Vector<Vector<String>>():anteriores;
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null) {
			String sql="SELECT ID,ENUNCIADO FROM PROGRAMACION_NODO" +
					" WHERE "+((idPadre!=null)?"PADRE="+idPadre:"PADRE IS NULL AND MATERIA="+idMateria)+
					" ORDER BY POSICION ASC;";
			tabla=this.sqlConsulta(sql);
			for (int i=0;i<tabla.size();i++) {
				Vector<String> elemento = new Vector<String>();
				elemento.add(tabla.get(i).get(0));
				elemento.add(tabla.get(i).get(1));
				salida.add(elemento);
				salida=this.consultarProgramacionNodoEnunciadosRecursivo(idMateria,tabla.get(i).get(0),salida);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records");
		return salida;
	}*/
	public void modificarEvaluableNodo(Vector<String> idEvaluables, Vector<String> nodos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((idEvaluables!=null)?idEvaluables:"null")+" idevaluables,"+nodos+")");
		if (idEvaluables!=null && nodos!=null) if (idEvaluables.size()>0) {
			borrarEvaluableNodoParaEvaluable(idEvaluables);
			for (int i=0;i<idEvaluables.size();i++)
				for (int j=0;j<nodos.size();j++)
					insertarEvaluableNodo(idEvaluables.get(i),nodos.get(j));
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((idEvaluables!=null)?idEvaluables:"null")+" idevaluables,"+nodos+") end");
	}
	public Vector<String> consultarIdProgramacionNodoParaEvaluable(String idEvaluable) {
		Vector<String> registro=null;
		if (idEvaluable!=null) {
			String sql="SELECT NODO FROM EVALUABLES_NODOS" +
					" WHERE EVALUABLE="+idEvaluable +";";
			registro=this.sqlConsulta1Columna(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((registro!=null)?registro.size():"null")+" nodes");
		return registro;
	}
	public void borrarEvaluableNodoParaEvaluable(Vector<String> idEvaluables){
		int ejecucion=-1;
		if (idEvaluables!=null) {
			if (idEvaluables.size()>0) {
				String sql="";
				for (int i=0;i<idEvaluables.size();i++)
					if (!idEvaluables.get(i).equals(""))
						sql+="DELETE FROM EVALUABLES_NODOS WHERE EVALUABLE="+idEvaluables.get(i)+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((idEvaluables!=null)?idEvaluables:"null")+" idevaluables) deleted "+ejecucion);
	}
	public String insertarEvaluableNodo(String idEvaluable,String idNodo){
		String id=null;
		if (idEvaluable!=null && idNodo!=null) {
			if (!idEvaluable.equals("") && !idNodo.equals("")) {
				String sql="INSERT INTO EVALUABLES_NODOS (EVALUABLE,NODO) VALUES ("+idEvaluable+",'"+idNodo+"');";
				id=this.sqlInsertarId("EVALUABLES_NODOS", sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+","+idNodo+") id inserted "+id);
		return id;
	}
	public static void archivoBorra(String nomArchivo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+")");
		if (nomArchivo!=null) {
			File file = new File(nomArchivo);
			if (file.exists()) {
				if (file.isDirectory()) {
					for (File c : file.listFiles()) BBDD.archivoBorra(c.getAbsolutePath());
				}
				if (!file.delete())
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+") can't delete file.");
				else
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+") file deleted.");
			} else {
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+") file no exist.");
			}
		}
	}
	public String textoQuitaEscapes(String origen){
		String cadena="";
		String s=null;
		int iu=0;
		if (origen!=null) {
			for (byte b : origen.getBytes()){
				if (s!=null) {
					iu++;
					if (iu>2) s+=(char)b;
					if (iu==6) {
						cadena+=(char)Integer.parseInt( s, 16 );
						s=null;
						iu=0;
					}
				} else {
					if ((char)b=='\\') {
						iu=1;
						s="";
					} else cadena+=(char)b;
				}
				//System.out.println((char)b+":"+iu+"s"+s);
			}
			//System.out.println("origen "+origen+" => "+cadena);
		}
		return cadena;
	}
	public boolean bbddModificarClave(String acceso){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+acceso+")",false);
		boolean resultado=false;
		int ejecucion=0;
		if (acceso!=null) {
			this.bbddBackupHacer();
			this.sqlEjecuta("SCRIPT '"+this.archivoDUMP+"';");
			this.conexionCerrar();
			BBDD.archivoBorra(this.archivoLOG);
			BBDD.archivoBorra(this.archivoLCK);
			BBDD.archivoBorra(this.archivoSCRIPT);
			BBDD.archivoBorra(this.archivoPROPERTIES);
			this.conexionFijaCadenaCrear(acceso);
			if (this.conexionDriverComprobar()){
				acceso=null;
				this.bbddCrear();
		        File archivo = new File(this.archivoDUMP);
		        //File archivo = new File(this.archivoDUMP+".clase");
		        FileReader fr=null;
				try {fr = new FileReader(archivo);
				} catch (FileNotFoundException e1) {e1.printStackTrace();}
		        BufferedReader br = new BufferedReader(fr);
		        String linea=null;
		        Vector<String> sql = new Vector<String>();
		        int nlinea=1;
		        do {
		        	try {
		        		linea=br.readLine();
		        		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("line "+nlinea+":"+textoQuitaEscapes(linea)+" > ");
		        		if (linea!=null)
		        			if (linea.matches("^INSERT.*"))
		        				if (!linea.matches(".* BLOCKS .*") && !linea.matches(".* TIPOS_EVENTO .*")) {
		        					//System.out.print("ok");
		        					sql.add(textoQuitaEscapes(linea)+";");
		        					//System.out.println(quitaEscapes(linea)+"; = "+this.ejecutaSQL(quitaEscapes(linea)+";"));
		        				}
		        		//System.out.println(".");
		        		
					} catch (IOException e2) {e2.printStackTrace();}
		        	nlinea++;
		        } while (linea != null);
		        if (!sql.equals("")) ejecucion=this.sqlEjecutaVarias(sql);
				resultado=true;
			}
			BBDD.archivoBorra(this.archivoDUMP);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+acceso+") "+ejecucion+" sentences sql copied.",true);
		return resultado;
	}
	public void bbddLimpia(){
		String id = null;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("()");
		Vector<Vector<String>> tabla = this.consultarProfesor(this.consultarIdPropio());
		this.conexionCerrar();
		BBDD.archivoBorra(this.archivoLOG);
		BBDD.archivoBorra(this.archivoLCK);
		BBDD.archivoBorra(this.archivoSCRIPT);
		BBDD.archivoBorra(this.archivoPROPERTIES);
		this.bbddCrear();
		if (tabla!=null) if (tabla.size()>0) {
			id = this.insertarProfesor(tabla.get(0).get(1),tabla.get(0).get(2),tabla.get(0).get(3));
			if (id!=null) if (!id.equals("")) {
				this.insertarUsuario(id);
			}
		}		
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() userID: "+id+" end");
	}
	public static void bbddBorrarTemporal(){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Borrando base de datos temporal");
		if (PCuaderno.MiPCuaderno.bbdd_temporal.conexion!=null) PCuaderno.MiPCuaderno.bbdd_temporal.conexionCerrar(); 
		BBDD.archivoBorra(PCuaderno.MiPCuaderno.titulo+BBDD.apendTemporal);
		PCuaderno.MiPCuaderno.bbdd_temporal=null;
	}
	public static void bbddImportarNoLectivos(BBDD receptora, BBDD importada){
		if (receptora!=null && importada!=null) {
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion de configuracion => desplazo temporal...");
			// Averigurar el diferencia temporal entre inicio curso receptor e importado
			String sql="SELECT * FROM CONFIGURACION;";
			Vector<Vector<String>> confi = receptora.sqlConsulta(sql);
			if (confi.size()>0) {
				Integer MesIniRecep=Integer.valueOf(confi.get(0).get(1));
				//Integer MesesRecep=Integer.valueOf(confi.get(0).get(2));
				Integer AnnoIniRecep=Integer.valueOf(confi.get(0).get(3));
				confi=null;
				Calendar inicioRec=Calendar.getInstance();
				inicioRec.set(Calendar.YEAR, AnnoIniRecep);
				inicioRec.set(Calendar.MONTH, MesIniRecep);
				inicioRec.set(Calendar.DAY_OF_MONTH, 1);
				inicioRec=Fecha.fechaHoraCero(inicioRec);
				sql="SELECT * FROM CONFIGURACION;";
				confi = importada.sqlConsulta(sql);
				if (confi.size()>0) {
					Integer MesIniImpor=Integer.valueOf(confi.get(0).get(1));
					Integer MesesImpor=Integer.valueOf(confi.get(0).get(2));
					Integer AnnoIniImpor=Integer.valueOf(confi.get(0).get(3));
					confi=null;
					Calendar inicioImp=Calendar.getInstance();
					inicioImp.set(Calendar.YEAR, AnnoIniImpor);
					inicioImp.set(Calendar.MONTH, MesIniImpor);
					inicioImp.set(Calendar.DAY_OF_MONTH, 1);
					inicioImp=Fecha.fechaHoraCero(inicioImp);
					Calendar finImp=Calendar.getInstance();
					finImp.set(Calendar.YEAR, AnnoIniImpor);
					finImp.set(Calendar.MONTH, MesIniImpor);
					finImp.set(Calendar.DAY_OF_MONTH, 1);
					finImp=Fecha.fechaHoraCero(finImp);
					finImp.add(Calendar.MONTH, MesesImpor+1);
					finImp.add(Calendar.MINUTE, -1);
					Long desplazo = inicioRec.getTimeInMillis() - inicioImp.getTimeInMillis();
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Desplazo temporal "+Fecha.formaFechaDiaHora(inicioRec)+" - "+Fecha.formaFechaDiaHora(inicioImp)+" = "+desplazo+" milisegundos ...");
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion días no lectivos...");
					Vector<Vector<String>> noLectivos = importada.consultarDiasNoLectivos();
					for (int i=0;i<noLectivos.size();i++){
						Calendar fecha=Fecha.fecha(noLectivos.get(i).get(2),noLectivos.get(i).get(3),noLectivos.get(i).get(4));
						fecha.setTimeInMillis(desplazo+fecha.getTimeInMillis());
						if (noLectivos.get(i).get(1)==null)
							receptora.insertarDiaNoLectivo(fecha, null, noLectivos.get(i).get(5));
						else {
							String idClaseImpor = noLectivos.get(i).get(1);
							Vector<Vector<String>> clasesI = importada.consultarClase(null, idClaseImpor);
							if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Importando dia no lectivo para Clase "+idClaseImpor+": "+clasesI.get(0).get(4)+" "+clasesI.get(0).get(3)+" "+clasesI.get(0).get(1)+" "+clasesI.get(0).get(2)+" ...");
							Vector<Vector<String>> ensenanzasR = receptora.consultarEnsenanza(null);
							String idEnsenanzaRecep=null;
							String idTituloRecep=null;
							String idNivelRecep=null;
							String idClaseRecep=null;
							for (int j=0;j<ensenanzasR.size();j++)
								if (ensenanzasR.get(j).get(1).equalsIgnoreCase(clasesI.get(0).get(4))) {
									idEnsenanzaRecep=ensenanzasR.get(j).get(0);
									if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Enseñanza Receptora "+idEnsenanzaRecep+"...");
									break;
								}
							ensenanzasR=null;
							if (idEnsenanzaRecep!=null) {
								Vector<Vector<String>> titulosR = receptora.consultarTitulo(idEnsenanzaRecep,null);
								for (int j=0;j<titulosR.size();j++)
									if (titulosR.get(j).get(2).equalsIgnoreCase(clasesI.get(0).get(3))) {
										idTituloRecep=titulosR.get(j).get(0);
										if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Titulo Receptor "+idTituloRecep+"...");
										break;
									}
							}
							if (idTituloRecep!=null) {
								Vector<Vector<String>> nivelesR = receptora.consultarNivel(idTituloRecep,null);
								for (int j=0;j<nivelesR.size();j++)
									if (nivelesR.get(j).get(2).equalsIgnoreCase(clasesI.get(0).get(1))) {
										idNivelRecep=nivelesR.get(j).get(0);
										if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Nivel Receptor "+idNivelRecep+"...");
										break;
									}
							}
							if (idNivelRecep!=null) {
								Vector<Vector<String>> clasesR = receptora.consultarClase(idNivelRecep,null);
								for (int j=0;j<clasesR.size();j++)
									if (clasesR.get(j).get(2).equalsIgnoreCase(clasesI.get(0).get(2))) {
										idClaseRecep=clasesR.get(j).get(0);
										break;
									}
							}
							if (idClaseRecep!=null) {
								receptora.insertarDiaNoLectivo(fecha, idClaseRecep, noLectivos.get(i).get(5));
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Clase Receptora "+idClaseRecep+"...");
							} else {
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  No hayó clase receptora, no importado ...");
							}
						}
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("importando día no lectivo "+Fecha.formaFecha(fecha)+" "+noLectivos.get(i).get(5)+"...");
					}
				}
			}
		}
	}
	public static void bbddImportarEventos(BBDD receptora, BBDD importada){
		if (receptora!=null && importada!=null) {
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion de configuracion => desplazo temporal...");
			// Averigurar el diferencia temporal entre inicio curso receptor e importado
			String sql="SELECT * FROM CONFIGURACION;";
			Vector<Vector<String>> confi = receptora.sqlConsulta(sql);
			if (confi.size()>0) {
				Integer MesIniRecep=Integer.valueOf(confi.get(0).get(1));
				//Integer MesesRecep=Integer.valueOf(confi.get(0).get(2));
				Integer AnnoIniRecep=Integer.valueOf(confi.get(0).get(3));
				confi=null;
				Calendar inicioRec=Calendar.getInstance();
				inicioRec.set(Calendar.YEAR, AnnoIniRecep);
				inicioRec.set(Calendar.MONTH, MesIniRecep);
				inicioRec.set(Calendar.DAY_OF_MONTH, 1);
				inicioRec=Fecha.fechaHoraCero(inicioRec);
				sql="SELECT * FROM CONFIGURACION;";
				confi = importada.sqlConsulta(sql);
				if (confi.size()>0) {
					Integer MesIniImpor=Integer.valueOf(confi.get(0).get(1));
					Integer MesesImpor=Integer.valueOf(confi.get(0).get(2));
					Integer AnnoIniImpor=Integer.valueOf(confi.get(0).get(3));
					confi=null;
					Calendar inicioImp=Calendar.getInstance();
					inicioImp.set(Calendar.YEAR, AnnoIniImpor);
					inicioImp.set(Calendar.MONTH, MesIniImpor);
					inicioImp.set(Calendar.DAY_OF_MONTH, 1);
					inicioImp=Fecha.fechaHoraCero(inicioImp);
					Calendar finImp=Calendar.getInstance();
					finImp.set(Calendar.YEAR, AnnoIniImpor);
					finImp.set(Calendar.MONTH, MesIniImpor);
					finImp.set(Calendar.DAY_OF_MONTH, 1);
					finImp=Fecha.fechaHoraCero(finImp);
					finImp.add(Calendar.MONTH, MesesImpor+1);
					finImp.add(Calendar.MINUTE, -1);
					Long desplazo = inicioRec.getTimeInMillis() - inicioImp.getTimeInMillis();
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Desplazo temporal "+Fecha.formaFechaDiaHora(inicioRec)+" - "+Fecha.formaFechaDiaHora(inicioImp)+" = "+desplazo+" milisegundos ...");
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion de eventos...");
					Vector<Vector<String>> eventos = importada.consultarEvento(null);
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion de "+((eventos!=null)?eventos.size():"null")+" eventos...");
					for (int i=0;i<eventos.size();i++){
						sql="";
						int idTipo = Integer.valueOf(eventos.get(i).get(9));
						if (idTipo!=BBDD.idTipoCitaConTutores) {
							Calendar fecha=Fecha.fechaHora(eventos.get(i).get(1),eventos.get(i).get(2),eventos.get(i).get(3),eventos.get(i).get(4),eventos.get(i).get(5));
							fecha.setTimeInMillis(desplazo+fecha.getTimeInMillis());
							String idEventoR = receptora.insertarEvento(idTipo, fecha,eventos.get(i).get(6),eventos.get(i).get(7),eventos.get(i).get(8));
							if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("importando evento id "+idEventoR+" ...");
							Vector<Vector<String>> eventoTipo = importada.consultarEventoTipoDatos(eventos.get(i).get(0), idTipo);
							if (idTipo==BBDD.idTipoClaustro) {
								sql="UPDATE CLAUSTROS SET ORDINARIO="+eventoTipo.get(0).get(1)+" WHERE ID="+idEventoR+";";
							} else if (idTipo==BBDD.idTipoEvaluacion) {
								String idClaseImpor = eventoTipo.get(0).get(2);
								Vector<Vector<String>> clasesI = importada.consultarClase(null, idClaseImpor);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Importando Evaluacion Clase "+idEventoR+": "+clasesI.get(0).get(4)+" "+clasesI.get(0).get(3)+" "+clasesI.get(0).get(1)+" "+clasesI.get(0).get(2)+" ...");
								Vector<Vector<String>> ensenanzasR = receptora.consultarEnsenanza(null);
								String idEnsenanzaRecep=null;
								String idTituloRecep=null;
								String idNivelRecep=null;
								String idClaseRecep=null;
								for (int j=0;j<ensenanzasR.size();j++)
									if (ensenanzasR.get(j).get(1).equalsIgnoreCase(clasesI.get(0).get(4))) {
										idEnsenanzaRecep=ensenanzasR.get(j).get(0);
										if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Enseñanza Receptora "+idEnsenanzaRecep+"...");
										break;
									}
								ensenanzasR=null;
								if (idEnsenanzaRecep!=null) {
									Vector<Vector<String>> titulosR = receptora.consultarTitulo(idEnsenanzaRecep,null);
									for (int j=0;j<titulosR.size();j++)
										if (titulosR.get(j).get(2).equalsIgnoreCase(clasesI.get(0).get(3))) {
											idTituloRecep=titulosR.get(j).get(0);
											if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Titulo Receptor "+idTituloRecep+"...");
											break;
										}
								}
								if (idTituloRecep!=null) {
									Vector<Vector<String>> nivelesR = receptora.consultarNivel(idTituloRecep,null);
									for (int j=0;j<nivelesR.size();j++)
										if (nivelesR.get(j).get(2).equalsIgnoreCase(clasesI.get(0).get(1))) {
											idNivelRecep=nivelesR.get(j).get(0);
											if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Nivel Receptor "+idNivelRecep+"...");
											break;
										}
								}
								if (idNivelRecep!=null) {
									Vector<Vector<String>> clasesR = receptora.consultarClase(idNivelRecep,null);
									for (int j=0;j<clasesR.size();j++)
										if (clasesR.get(j).get(2).equalsIgnoreCase(clasesI.get(0).get(2))) {
											idClaseRecep=clasesR.get(j).get(0);
											break;
										}
								}
								if (idClaseRecep!=null) {
									sql="UPDATE EVALUACIONES SET DESCRIPCION='"+eventoTipo.get(0).get(1)+"' WHERE ID="+idEventoR+";";
									sql+="UPDATE EVALUACIONES SET CLASE="+idClaseRecep+" WHERE ID="+idEventoR+";";
									if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  Clase Receptora "+idClaseRecep+"...");
								} else {
									receptora.borrarEvento(idEventoR);
									if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("  No hayó clase receptora borrando evento "+idEventoR+"...");
								}
							} else if (idTipo==BBDD.idTipoConsejoEscolar) {
								sql="UPDATE C_ESCOLARES SET ORDINARIO="+eventoTipo.get(0).get(1)+" WHERE ID="+idEventoR+";";
							/*} else if (idTipo==BBDD.idTipoCitaConTutores) {
								sql="INSERT INTO CITAS_TUTORES (ID,RELACION) VALUES ("+id+",'');";*/
							} else if (idTipo==BBDD.idTipoETCP) {
							} else if (idTipo==BBDD.idTipoOtroTipo) {
								sql="UPDATE OTROS_EVENTOS SET DESCRIPCION='"+eventoTipo.get(0).get(1)+"' WHERE ID="+idEventoR+";";
							}
							if (!sql.equals("")){
								receptora.sqlEjecuta(sql);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Insertando tipo evento:"+sql);							
							}
						}
					}
				}
			}
		}
	}					
	public static String bbddImportarMateria(BBDD receptora, BBDD importada, String idMateriaImpor){
		String idMateriaRecep=null;
		if (receptora!=null && importada!=null && idMateriaImpor!=null)
			if (idMateriaImpor!="") { // importa una Materia (con sus parciales, y programación con duracion de nodo para clases (si no las encuentra las crea))
				Vector<Vector<String>> materiasImpor = importada.consultarMateria(idMateriaImpor);
				String refMateriaImpor = materiasImpor.get(0).get(1);
				String desMateriaImpor = materiasImpor.get(0).get(2);
				String idNivelImpor = materiasImpor.get(0).get(3);
				String refNivelImpor = materiasImpor.get(0).get(4);
				String refTituloImpor = materiasImpor.get(0).get(5);
				String refEnsenanzaImpor = materiasImpor.get(0).get(6);
				String idEnsenanzaRecep=null;
				String idTituloRecep=null;
				String idNivelRecep=null;
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Buscando enseñanza receptora ...");
				Vector<Vector<String>> ensenanzasR = receptora.consultarEnsenanza(null);
				if (ensenanzasR!=null) if(ensenanzasR.size()!=0) {
					for (int i=0;i<ensenanzasR.size();i++) 
						if (ensenanzasR.get(i).get(1).equalsIgnoreCase(refEnsenanzaImpor)) {
							idEnsenanzaRecep=ensenanzasR.get(i).get(0);
							if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Encontrada coincidencia enseñanza receptora ...");
							break;
						}
				}
				if (idEnsenanzaRecep==null) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("No hay coincidencia en enseñanza receptora , creando Enseñanza,Título,Nivel,Materia ...");
					Vector<Vector<String>> nivelesI = importada.consultarNivel(null, idNivelImpor);
					String idTituloImpor=nivelesI.get(0).get(1);
					nivelesI=null;
					Vector<Vector<String>> titulosI = importada.consultarTitulo(null, idTituloImpor);
					String idEnsenanzaImpor=titulosI.get(0).get(1);
					String desTituloImpor=titulosI.get(0).get(3);
					titulosI=null;
					Vector<Vector<String>> ensenanzasI = importada.consultarEnsenanza(idEnsenanzaImpor);
					String desEnsenanzaImpor=ensenanzasI.get(0).get(2);
					ensenanzasR=null;
					idEnsenanzaRecep=receptora.insertarEnsenanza(refEnsenanzaImpor, desEnsenanzaImpor);
					idTituloRecep=receptora.insertarTitulo(idEnsenanzaRecep,refTituloImpor,desTituloImpor);
					idNivelRecep=receptora.insertarNivel(idTituloRecep,refNivelImpor);
					idMateriaRecep=receptora.insertarMateria(idNivelRecep,refMateriaImpor,desMateriaImpor);
				}
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Relacionando enseñanza "+refEnsenanzaImpor+" => "+idEnsenanzaRecep+" ...");
				if (idTituloRecep==null){
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Buscando título receptor ...");
					Vector<Vector<String>> titulosR = receptora.consultarTitulo(idEnsenanzaRecep,null);
					if (titulosR!=null) if(titulosR.size()!=0) {
						for (int i=0;i<titulosR.size();i++) 
							if (titulosR.get(i).get(2).equalsIgnoreCase(refTituloImpor)) {
								idTituloRecep=titulosR.get(i).get(0);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Encontrada coincidencia título receptor ...");
								break;
							}
					}
					
				}
				if (idTituloRecep==null) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("No hay coincidencia en título receptor, creando Título,Nivel,Materia ...");
					Vector<Vector<String>> nivelesI = importada.consultarNivel(null, idNivelImpor);
					String idTituloImpor=nivelesI.get(0).get(1);
					nivelesI=null;
					Vector<Vector<String>> titulosI = importada.consultarTitulo(null, idTituloImpor);
					String desTituloImpor=titulosI.get(0).get(3);
					titulosI=null;
					idTituloRecep=receptora.insertarTitulo(idEnsenanzaRecep,refTituloImpor,desTituloImpor);
					idNivelRecep=receptora.insertarNivel(idTituloRecep,refNivelImpor);
					idMateriaRecep=receptora.insertarMateria(idNivelRecep,refMateriaImpor,desMateriaImpor);
				}
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Relacionando titúlo "+refTituloImpor+" => "+idTituloRecep+" ...");
				if (idNivelRecep==null){
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Buscando nivel receptor ...");
					Vector<Vector<String>> nivelesR = receptora.consultarNivel(idTituloRecep,null);
					if (nivelesR!=null) if(nivelesR.size()!=0) {
						for (int i=0;i<nivelesR.size();i++)
							if (nivelesR.get(i).get(2).equalsIgnoreCase(refNivelImpor)) {
								idNivelRecep=nivelesR.get(i).get(0);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Encontrada coincidencia nivel receptor ...");
								break;
							}
					}
					
				}
				if (idNivelRecep==null) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("No hay coincidencia en nivel receptor, creando Nivel,Materia ...");
					idNivelRecep=receptora.insertarNivel(idTituloRecep,refNivelImpor);
					idMateriaRecep=receptora.insertarMateria(idNivelRecep,refMateriaImpor,desMateriaImpor);
				}
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Relacionando nivel "+refNivelImpor+" => "+idNivelRecep+" ...");
				if (idMateriaRecep==null){
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Buscando materia receptora ...");
					Vector<Vector<String>> materiasR = receptora.consultarMateria(idNivelRecep,null);
					if (materiasR!=null) if(materiasR.size()!=0) {
						for (int i=0;i<materiasR.size();i++) 
							if (materiasR.get(i).get(1).equalsIgnoreCase(refMateriaImpor)) {
								idMateriaRecep=materiasR.get(i).get(0);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Encontrada coincidencia materia receptora ...");
								break;
							}
					}
					
				}
				if (idMateriaRecep==null) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("No hay coincidencia en materia receptora, creando Materia ...");
					idMateriaRecep=receptora.insertarMateria(idNivelRecep,refMateriaImpor,desMateriaImpor);
				}
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Relacionando materia "+refMateriaImpor+" => "+idMateriaRecep+" ...");

				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion de configuracion => desplazo temporal...");
				// Averigurar el diferencia temporal entre inicio curso receptor e importado
				String sql="SELECT * FROM CONFIGURACION;";
				Vector<Vector<String>> confi = receptora.sqlConsulta(sql);
				if (confi.size()>0) {
					Integer MesIniRecep=Integer.valueOf(confi.get(0).get(1));
					//Integer MesesRecep=Integer.valueOf(confi.get(0).get(2));
					Integer AnnoIniRecep=Integer.valueOf(confi.get(0).get(3));
					confi=null;
					Calendar inicioRec=Calendar.getInstance();
					inicioRec.set(Calendar.YEAR, AnnoIniRecep);
					inicioRec.set(Calendar.MONTH, MesIniRecep);
					inicioRec.set(Calendar.DAY_OF_MONTH, 1);
					inicioRec=Fecha.fechaHoraCero(inicioRec);
					sql="SELECT * FROM CONFIGURACION;";
					confi = importada.sqlConsulta(sql);
					if (confi.size()>0) {
						Integer MesIniImpor=Integer.valueOf(confi.get(0).get(1));
						Integer MesesImpor=Integer.valueOf(confi.get(0).get(2));
						Integer AnnoIniImpor=Integer.valueOf(confi.get(0).get(3));
						confi=null;
						Calendar inicioImp=Calendar.getInstance();
						inicioImp.set(Calendar.YEAR, AnnoIniImpor);
						inicioImp.set(Calendar.MONTH, MesIniImpor);
						inicioImp.set(Calendar.DAY_OF_MONTH, 1);
						inicioImp=Fecha.fechaHoraCero(inicioImp);
						Calendar finImp=Calendar.getInstance();
						finImp.set(Calendar.YEAR, AnnoIniImpor);
						finImp.set(Calendar.MONTH, MesIniImpor);
						finImp.set(Calendar.DAY_OF_MONTH, 1);
						finImp=Fecha.fechaHoraCero(finImp);
						finImp.add(Calendar.MONTH, MesesImpor+1);
						finImp.add(Calendar.MINUTE, -1);
						Long desplazo = inicioRec.getTimeInMillis() - inicioImp.getTimeInMillis();
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Desplazo temporal "+Fecha.formaFechaDiaHora(inicioRec)+" - "+Fecha.formaFechaDiaHora(inicioImp)+" = "+desplazo+" milisegundos ...");
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion parciales...");
						Vector<Vector<String>> parciales = importada.consultarParcial(idMateriaImpor, null);
						Vector<Calendar> ffParcialesImpor = new Vector<Calendar>();
						Vector<Calendar> ffParcialesRecep = new Vector<Calendar>();
						for (int i=0;i<parciales.size();i++){
							Calendar fInicio = Fecha.fecha(parciales.get(i).get(2), parciales.get(i).get(3), parciales.get(i).get(4));
							Calendar fFin = Fecha.fecha(parciales.get(i).get(5), parciales.get(i).get(6), parciales.get(i).get(7));
							if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Parcial: "+Fecha.formaFechaDiaHora(fInicio)+" - "+Fecha.formaFechaDiaHora(fFin)+" ...");
							fInicio.setTimeInMillis(desplazo+fInicio.getTimeInMillis());
							fInicio=Fecha.diaLaboralOSiguiente(fInicio,false);
							fInicio=Fecha.fechaHoraCero(fInicio);
							fFin=Fecha.fechaHoraCero(fFin);
							fFin.add(Calendar.DAY_OF_MONTH,1);
							fFin.add(Calendar.MINUTE, -1);
							ffParcialesImpor.add((Calendar)fFin.clone());
							fFin.setTimeInMillis(desplazo+fFin.getTimeInMillis());
							fFin=Fecha.diaLaboralOSiguiente(fFin,false);
							fFin=Fecha.fechaHoraCero(fFin);
							fFin.add(Calendar.DAY_OF_MONTH,1);
							fFin.add(Calendar.MINUTE, -1);
							ffParcialesRecep.add((Calendar)fFin.clone());
							if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Insertando parcial: "+Fecha.formaFechaDiaHora(fInicio)+" - "+Fecha.formaFechaDiaHora(fFin)+" ...");
							if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Fin parcial: "+Fecha.formaFechaDiaHora(ffParcialesImpor.lastElement())+" => "+Fecha.formaFechaDiaHora(ffParcialesRecep.lastElement())+" ...");
							receptora.insertarParcial(idMateriaRecep, fInicio, fFin);
						}			
						// importa programación la duración necesita a la clase solo para una materia
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion clases...");
						Vector<String> clasesImpor = new Vector<String>();
						Vector<String> materiaClasesImpor = new Vector<String>();
						Vector<String> clasesRecep = new Vector<String>();
						Vector<String> materiaClasesRecep = new Vector<String>();
						Vector<Vector<String>> clasesI = importada.consultarClasesParaMateria(idMateriaImpor);
						Vector<Vector<String>> clasesR = receptora.consultarClasesParaMateria(idMateriaRecep);
						for (int i=0;i<clasesI.size();i++) {
							boolean encontrado=false;
							clasesImpor.add(clasesI.get(i).get(0));
							Vector<String> materiaClaseI = importada.consultarIdMateriaClase(idMateriaImpor, clasesI.get(i).get(0));
							materiaClasesImpor.add(materiaClaseI.get(0));
							for (int j=0;j<clasesR.size();j++) {
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Clase ("+clasesI.get(i).get(0)+") importada "+clasesI.get(i).get(2)+" = Clase ("+clasesR.get(j).get(0)+") receptora "+clasesR.get(j).get(2)+" ...");
								if (clasesI.get(i).get(2).equalsIgnoreCase(clasesR.get(j).get(2))) {
									clasesRecep.add(clasesR.get(j).get(0));
									Vector<String> materiaClaseR = receptora.consultarIdMateriaClase(idMateriaRecep, clasesR.get(j).get(0));
									materiaClasesRecep.add(materiaClaseR.get(0));
									if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Relaciona Clases "+clasesI.get(i).get(0)+" => "+clasesR.get(j).get(0)+" MatClase "+materiaClaseI.get(0)+" => "+materiaClaseR.get(0));
									clasesR.remove(j);
									encontrado=true;
									break;
								}
							}
							if (!encontrado) {
								String idClaseR = receptora.insertarClase(idNivelRecep, clasesI.get(i).get(2));
								clasesRecep.add(idClaseR);
								Vector<String> materiaClaseR = receptora.consultarIdMateriaClase(idMateriaRecep, idClaseR);
								materiaClasesRecep.add(materiaClaseR.get(0));
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Nueva Clase "+clasesI.get(i).get(0)+" => "+idClaseR+" MatClase "+materiaClaseI.get(0)+" => "+materiaClaseR.get(0));
							}
						}
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Importando informacion programacion...",false);
						Vector<String> idNodosImpor = new Vector<String>();
						Vector<String> idNodosRecep = new Vector<String>();
						BBDD.bbddImportarNodo(receptora,importada,null,idMateriaImpor,idMateriaRecep,clasesImpor,clasesRecep,idNodosImpor,idNodosRecep);
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Nodos en importada "+idNodosImpor.size()+" en receptora "+idNodosRecep.size()+" ...",true);
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Importando informacion de tipos de evaluable...");
						Vector<String> tiposImpor = new Vector<String>();
						Vector<String> tiposRecep = new Vector<String>();
						Vector<Vector<String>> tiposI = importada.consultarTiposEvaluablesParaMateria(null);
						Vector<Vector<String>> tiposR = receptora.consultarTiposEvaluablesParaMateria(null);
						for (int i=0;i<tiposI.size();i++) {
							boolean encontrado=false;
							tiposImpor.add(tiposI.get(i).get(0));
							for (int j=0;j<tiposR.size();j++) {
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Tipo ("+tiposI.get(i).get(0)+") importada "+tiposI.get(i).get(1)+" = Tipo ("+tiposR.get(j).get(0)+") receptora "+tiposR.get(j).get(1)+" ...");
								if (tiposI.get(i).get(1).equalsIgnoreCase(tiposR.get(j).get(1))) {
									tiposRecep.add(tiposR.get(j).get(0));
									if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Relaciona Tipos "+tiposI.get(i).get(0)+" => "+tiposR.get(j).get(0));
									tiposR.remove(j);
									encontrado=true;
									break;
								}
							}
							if (!encontrado) {
								String idTipoR = receptora.insertarTipoEvaluable(
										null,tiposI.get(i).get(1),tiposI.get(i).get(2),
										Integer.valueOf(tiposI.get(i).get(3)),Numero.cadenaDecimal(Double.parseDouble(tiposI.get(i).get(4))));
								tiposRecep.add(idTipoR);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Nuevo Tipo "+tiposI.get(i).get(0)+" => "+idTipoR+": "+tiposI.get(i).get(1)+","+tiposI.get(i).get(2)+","+Integer.valueOf(tiposI.get(i).get(3))+","+tiposI.get(i).get(4)+"->"+Numero.cadenaDecimal(Double.parseDouble(tiposI.get(i).get(4))));
							}
						}
						tiposI = importada.consultarTiposEvaluablesParaMateria(idMateriaImpor);
						tiposR = receptora.consultarTiposEvaluablesParaMateria(idMateriaRecep);
						for (int i=0;i<tiposI.size();i++) {
							boolean encontrado=false;
							tiposImpor.add(tiposI.get(i).get(0));
							for (int j=0;j<tiposR.size();j++) {
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Tipo Exclusivo Materia ("+tiposI.get(i).get(0)+") importada "+tiposI.get(i).get(1)+" = Tipo ("+tiposR.get(j).get(0)+") receptora "+tiposR.get(j).get(1)+" ...");
								if (tiposI.get(i).get(1).equalsIgnoreCase(tiposR.get(j).get(1))) {
									tiposRecep.add(tiposR.get(j).get(0));
									if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Relaciona Tipos "+tiposI.get(i).get(0)+" => "+tiposR.get(j).get(0));
									tiposR.remove(j);
									encontrado=true;
									break;
								}
							}
							if (!encontrado) {
								String idTipoR = receptora.insertarTipoEvaluable(
										idMateriaRecep,tiposI.get(i).get(1),tiposI.get(i).get(2),
										Integer.valueOf(tiposI.get(i).get(3)),Numero.cadenaDecimal(Double.parseDouble(tiposI.get(i).get(4))));
								tiposRecep.add(idTipoR);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Nuevo Tipo Exclusivo Materia "+tiposI.get(i).get(0)+" => "+idTipoR+": "+tiposI.get(i).get(1)+","+tiposI.get(i).get(2)+","+Integer.valueOf(tiposI.get(i).get(3))+","+Numero.cadenaDecimal(Double.parseDouble(tiposI.get(i).get(4))));
							}
						}
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Recopilando informacion horario materia-clases ("+materiaClasesImpor.size()+" => "+materiaClasesRecep.size()+") y creando una uh receptora si es necesario...");
						Integer diaDeLaSemana=Calendar.WEDNESDAY;
						for (int i=0;i<materiaClasesRecep.size();i++) {
							String materiaClaseR=materiaClasesRecep.get(i);
							String materiaClaseI=materiaClasesImpor.get(i);
							String idUHR = null;
							Vector<String> evaluablesRecep = new Vector<String>();
							Vector<String> evaluablesImpor = new Vector<String>();
							if (receptora.consultarUHvacio(materiaClaseR)) {
								idUHR = receptora.insertarUHsiguiente(materiaClaseR, String.valueOf(diaDeLaSemana));
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Nueva UH receptora "+idUHR+" (MatCla "+materiaClaseR+" diaSemana "+diaDeLaSemana+") ...");
							} else {
								Vector<Vector<String>> uhR = receptora.consultarUHMateriaClase(materiaClaseR);
								idUHR = uhR.get(0).get(0);
								diaDeLaSemana=Integer.valueOf(uhR.get(0).get(1));
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("existente UH receptora "+idUHR+" ("+uhR.get(0).get(6)+" "+uhR.get(0).get(2)+":"+uhR.get(0).get(3)+"-"+uhR.get(0).get(4)+":"+uhR.get(0).get(5)+" ...");
							}
							if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("importando información evaluables materia-clase inicio curso impor "+Fecha.formaFechaDiaHora(inicioImp)+" ...");
							for (int p=1;p<=ffParcialesImpor.size();p++){
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Parcial "+p+" Hasta "+Fecha.formaFechaDiaHora(ffParcialesImpor.get(p-1))+" ...");
								Calendar fi=null;
								Calendar ff=null;
								if (p==1) fi=inicioImp; else {
									fi=ffParcialesImpor.get(p-2); // 1er Parcial = 1, 1er indice = 0
									fi.add(Calendar.DAY_OF_MONTH, 1);
								}
								fi=Fecha.fechaHoraCero(fi);
								if (p==ffParcialesImpor.size()) ff=finImp; else ff=ffParcialesImpor.get(p-1);
								ff=Fecha.fechaHoraCero(ff);
								ff.add(Calendar.DAY_OF_MONTH, 1);
								ff.add(Calendar.MINUTE, -1);
								Calendar fechaParaUHR=(Calendar)ffParcialesRecep.get(p-1).clone();
								fechaParaUHR=Fecha.fechaHoraCero(fechaParaUHR);
								for (int d=0;d<Fecha.NDiasSemana;d++) {
									if (diaDeLaSemana.equals(fechaParaUHR.get(Calendar.DAY_OF_WEEK)))
										break;
									else
										fechaParaUHR.add(Calendar.DAY_OF_MONTH, -1);
								}
								Vector<Vector<String>> evaluables = importada.consultarEvaluablesMateriaClaseFecha(materiaClaseI,fi,ff);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("De "+Fecha.formaFechaDiaHora(fi)+" a "+Fecha.formaFechaDiaHora(ff)+" "+((evaluables!=null)?evaluables.size():"0")+" a la fecha "+Fecha.formaFechaDia(fechaParaUHR)+" ...");
								if (evaluables!=null) {
									for (int j=0;j<evaluables.size();j++){
										Integer idRecup=null;
										if (evaluables.get(j).get(12)!=null) {
											int indice = evaluablesImpor.indexOf(evaluables.get(j).get(12));
											if (indice>=0) idRecup=Integer.valueOf(evaluablesRecep.get(indice));
										}
										String idEvaluableRecep=receptora.insertarEvaluable(fechaParaUHR,idUHR,evaluables.get(j).get(6),
												evaluables.get(j).get(7),Integer.valueOf(evaluables.get(j).get(8)),
												Integer.valueOf(tiposRecep.get(tiposImpor.indexOf(evaluables.get(j).get(9)))),
												idRecup);
										evaluablesRecep.add(idEvaluableRecep);
										evaluablesImpor.add(evaluables.get(j).get(0));
										Vector<Vector<String>> evaluablesNodosImpor = importada.consultarEvaluablesNodos(evaluables.get(j).get(0),null);
										if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Evaluable "+evaluables.get(j).get(0)+"->"+idEvaluableRecep+": "+evaluables.get(j).get(6)+","+evaluables.get(j).get(7)+","+Integer.valueOf(evaluables.get(j).get(8))+",RECU="+idRecup+" =>"+evaluablesNodosImpor.size()+" nodos");
										for (int k=0;k<evaluablesNodosImpor.size();k++){
											receptora.insertarEvaluableNodo(
												idEvaluableRecep,
												idNodosRecep.get(idNodosImpor.indexOf(evaluablesNodosImpor.get(k).get(1))));
										}
									}
								}
							}
						}
					}
				}
			}
		return idMateriaRecep;
	}
	private static void bbddImportarNodo(BBDD receptora,BBDD importada,
			String idNodo,String idMateriaImpor,String idMateriaRecep,
			Vector<String> clasesImpor,Vector<String> clasesRecep,
			Vector<String> idNodoImpor,Vector<String> idNodoRecep){
		Vector<Vector<String>> hijosImpor = importada.consultarProgramacionNodoHijos(
				idNodo,idMateriaImpor);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Nodo "+idNodo+" idmateriaImpor="+idMateriaImpor+" hijos="+hijosImpor.size());
		if (hijosImpor!=null){
			if (hijosImpor.size() > 0) {
				//if (PCuaderno.noti) for (int j = this.nivel(nodo); j > 0; j--) sangria += "  ";
				for (int i = 0; i < hijosImpor.size(); i++) {
					idNodoImpor.add(hijosImpor.get(i).get(0));
					Vector<Vector<String>> hijosDelPadreNodoRecep=receptora.consultarProgramacionNodoHijos((hijosImpor.get(i).get(1)==null)?null:idNodoRecep.elementAt(idNodoImpor.indexOf(hijosImpor.get(i).get(1))), idMateriaRecep);
					Integer posicion=((hijosDelPadreNodoRecep==null)?0:hijosDelPadreNodoRecep.size())+1;
					idNodoRecep.add(
						receptora.insertarProgramacionNodo(
							hijosImpor.get(i).get(4),
							(hijosImpor.get(i).get(1)==null)?"NULL":idNodoRecep.elementAt(idNodoImpor.indexOf(hijosImpor.get(i).get(1))),
							posicion,
							idMateriaRecep));
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(idNodoRecep.lastElement()+"< "+hijosImpor.get(i).get(2) + ": ("
							+ hijosImpor.get(i).get(0) + ") "
							+ hijosImpor.get(i).get(4));
					bbddImportarNodo(receptora,importada,hijosImpor.get(i).get(0),
							idMateriaImpor,idMateriaRecep,clasesImpor,clasesRecep,idNodoImpor,idNodoRecep);
				}
			} else {
				if (idNodo != null) {
					if (clasesImpor!=null) if (clasesImpor.size()>0){
						for (int i=0;i<clasesImpor.size();i++){
							Vector<Vector<String>> nodoInfo = importada
								.consultarProgramacionDuracionNodo(idNodo,clasesImpor.get(i));
							if (nodoInfo.size() > 0)
								receptora.insertarProgramacionDuracionNodo(idNodoRecep.elementAt(idNodoImpor.indexOf(idNodo)), clasesRecep.get(i), Integer.valueOf(nodoInfo.get(0).get(2)));
						}
					}
				}
			}
		}
	}
	public static boolean bbddPreparaTemporal(String nomArchivo){
		boolean bien=false;
		String dirTemporal=PCuaderno.MiPCuaderno.titulo+BBDD.apendTemporal;
	    File carpetaTemporal = new File(dirTemporal+File.separatorChar+PCuaderno.MiPCuaderno.titulo+BBDD.apendBBDD);
	    if (!carpetaTemporal.exists()) 
	    	carpetaTemporal.mkdirs();
		//String archivoTemporal=dirTemporal+File.separatorChar+this.dirBBDD+BBDD.nombreBBDD;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+")");
        if (nomArchivo!=null) {
			FileInputStream fis=null;
			try {fis = new FileInputStream(nomArchivo);
			} catch (FileNotFoundException e1) {e1.printStackTrace();}
			ZipInputStream zis = new ZipInputStream(new BufferedInputStream(fis));
			ZipEntry entry;
			BufferedOutputStream destino=null;
			try {
				boolean aProperties=false;
				boolean aScript=false;
				while((entry = zis.getNextEntry()) != null) {
					System.out.println("Extract: " +entry+" ("+nomArchivo+")");
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Archivos: ."+BBDD.extPROPERTIES+" ."+BBDD.extSCRIPT);
					if (entry.getName().matches(".*"+BBDD.nombreBBDD+BBDD.extPROPERTIES)
							|| entry.getName().matches(".*"+BBDD.nombreBBDD+BBDD.extSCRIPT)) {
						if (entry.getName().matches(".*"+BBDD.nombreBBDD+BBDD.extPROPERTIES)) aProperties=true; 
						if (entry.getName().matches(".*"+BBDD.nombreBBDD+BBDD.extSCRIPT)) aScript=true;
					    int cuenta;
					    byte datos[] = new byte[BBDD.bufferArchivo];
					    String nomDestino=entry.getName();
					    nomDestino=nomDestino.replaceFirst(BBDD.dirViejoBBDD, PCuaderno.MiPCuaderno.titulo+BBDD.apendBBDD);  //Compatibilidad version 0.1
					    nomDestino=dirTemporal+File.separatorChar+nomDestino;
					    if (carpetaTemporal.exists()) {
						    System.out.println("Restoring "+nomDestino);
						    FileOutputStream fos = new FileOutputStream(nomDestino);
						    destino = new BufferedOutputStream(fos, BBDD.bufferArchivo);
						    while ((cuenta = zis.read(datos, 0, BBDD.bufferArchivo))!=-1) {
						       destino.write(datos, 0, cuenta);
						    }
						    destino.flush();
						    destino.close();
						    if (aProperties	&& aScript) {
						    	PCuaderno.MiPCuaderno.bbdd_temporal=new BBDD(dirTemporal+File.separatorChar+PCuaderno.MiPCuaderno.titulo+BBDD.apendBBDD);
						    	bien=true;
						    }
					    } else {
					    	System.out.println("Folder "+dirTemporal+" can't be created.");
					    }
					} else {
						System.out.println("File "+nomArchivo+" is not backup.");
						break;
					}
				 }
			} catch (FileNotFoundException e) {e.printStackTrace();
			} catch (IOException e) {e.printStackTrace();}
			try {zis.close();} catch (IOException e) {e.printStackTrace();}
        }
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() end");
		return bien;
	}
	public boolean bbddBackupRestaurar(String nomArchivo){
		boolean bien=true;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+")");
        if (nomArchivo!=null) {
        	BBDD.archivoBorra(this.archivoLOG);
        	BBDD.archivoBorra(this.archivoLCK);
			FileInputStream fis=null;
			try {fis = new FileInputStream(nomArchivo);
			} catch (FileNotFoundException e1) {e1.printStackTrace();}
			ZipInputStream zis = new ZipInputStream(new BufferedInputStream(fis));
			ZipEntry entry;
			BufferedOutputStream destino=null;
			try {
				while((entry = zis.getNextEntry()) != null) {
					System.out.println("Extract: " +entry+" ("+nomArchivo+")");
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Archivos: "+this.archivoPROPERTIES+" "+this.archivoSCRIPT);
					if (entry.getName().matches(".*"+BBDD.nombreBBDD+BBDD.extPROPERTIES)
							|| entry.getName().matches(".*"+BBDD.nombreBBDD+BBDD.extSCRIPT)) {
					    int cuenta;
					    byte datos[] = new byte[BBDD.bufferArchivo];
					    String nomDestino=entry.getName();
					    nomDestino=nomDestino.replaceFirst(BBDD.dirViejoBBDD, PCuaderno.MiPCuaderno.titulo+BBDD.apendBBDD);  //Compatibilidad version 0.1
					    System.out.println("Restoring "+nomDestino);
					    FileOutputStream fos = new FileOutputStream(nomDestino);
					    destino = new BufferedOutputStream(fos, BBDD.bufferArchivo);
					    while ((cuenta = zis.read(datos, 0, BBDD.bufferArchivo))!=-1) {
					       destino.write(datos, 0, cuenta);
					    }
					    destino.flush();
					    destino.close();
					} else {
						bien=false;
						System.out.println("File "+nomArchivo+" is not backup.");
						break;
					}
				 }
			} catch (FileNotFoundException e) {e.printStackTrace();
			} catch (IOException e) {e.printStackTrace();}
			try {zis.close();} catch (IOException e) {e.printStackTrace();}
        }
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() end");
		return bien;
	}
	public boolean bbddBackupHacer(){
		boolean bien=true;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("()");
		FileOutputStream fout=null;
		Calendar fecha = Calendar.getInstance();
		String nombreCopia=PCuaderno.MiPCuaderno.titulo+"_backup_"+Fecha.nomArchivoFecha(fecha)+".zip";
		try {fout = new FileOutputStream(nombreCopia);
		} catch (FileNotFoundException e1) {bien=false;e1.printStackTrace();}
		ZipOutputStream archivoZIP = new ZipOutputStream(fout);
		archivoZIP.setLevel(9);
		ZipEntry zProperties = new ZipEntry(this.archivoPROPERTIES);
		FileInputStream aProperties=null;
		ZipEntry zScript = new ZipEntry(this.archivoSCRIPT);
		FileInputStream aScript=null;
		try {
			aProperties = new FileInputStream(this.archivoPROPERTIES);
			aScript = new FileInputStream(this.archivoSCRIPT);
		} catch (FileNotFoundException e1) {bien=false;e1.printStackTrace();}
		try {
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Comprimiendo "+this.archivoPROPERTIES+" ...");
			archivoZIP.putNextEntry(zProperties);
			for (int c=aProperties.read();c!=-1;c=aProperties.read()) {archivoZIP.write(c);}
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Comprimiendo "+this.archivoSCRIPT+" ...");
			archivoZIP.putNextEntry(zScript);
			for (int c=aScript.read();c!=-1;c=aScript.read()) {archivoZIP.write(c);}
		} catch (IOException x) {bien=false;x.printStackTrace();
		} finally {	
			try {aProperties.close();} catch (IOException e1) {e1.printStackTrace();}	
			try {aScript.close();} catch (IOException e1) {e1.printStackTrace();}
		}
		try {archivoZIP.close();
		} catch (IOException e1) {bien=false;e1.printStackTrace();}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() => "+nombreCopia);
		return bien;
	}
	public void importarEstudiantes(String nomArchivo,String idClase,boolean cabecera){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+","+idClase+","+cabecera+")");
    	int ejecucion=0;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") archivo "+nomArchivo+" ("+BBDD.conjuntoCaracteresEstudiantes+")");
        if (nomArchivo!=null) {
        	String sql="";
			String linea=null;
			BufferedReader bf=null;
			InputStreamReader isr=null;
			try {
				isr = new InputStreamReader(new FileInputStream(nomArchivo),Charset.forName(BBDD.conjuntoCaracteresEstudiantes));
			} catch (FileNotFoundException e) {e.printStackTrace();}
			bf = new BufferedReader (isr);
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("codificación "+isr.getEncoding());
			int nlinea=0;
			try {
				while ((linea = bf.readLine())!=null) {
					nlinea++;
					if (nlinea>1 || !cabecera) {
						linea=linea.replace("\"", "");
						linea=linea.replace("'", "");
						String[] campos=linea.split(",");
						if (campos.length<2 || campos.length>2) {
							System.out.println("Error: "+linea);
							System.out.println("Error al importar estudiante: linea con ningúna o más de una coma.");
							break;
						}
						campos[0]=campos[0].trim();
						campos[1]=campos[1].trim();
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(campos[1]+" "+campos[0]);
						sql+="INSERT INTO ESTUDIANTES (CLASE,NOMBRE,APELLIDOS) VALUES ("+idClase+",'"+campos[1]+"','"+campos[0]+"');";
					}
				}
				ejecucion=this.sqlEjecuta(sql);
			} catch (IOException e1) {e1.printStackTrace();} 
        }
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") "+ejecucion+" students imported.");
	}
	public void importarProgramacion(String idMateria){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+")");
        FileDialog fd = new FileDialog(PCuaderno.MiPCuaderno.shellPrincipal, SWT.OPEN);
        fd.setText("Importar índice desde");
        fd.setFilterPath(this.dirBBDD+File.separatorChar);
        String[] filterExt = { "*.txt", "*.*" };
        fd.setFilterExtensions(filterExt);
        String nomArchivo = fd.open();
        if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("BBDD importarProgramacion desde "+nomArchivo);
        int nlinea=0;
        if (nomArchivo!=null) {
	        File archivo = new File(nomArchivo);
	        FileReader fr=null;
			try {fr = new FileReader(archivo);
			} catch (FileNotFoundException e1) {e1.printStackTrace();}
	        BufferedReader br = new BufferedReader(fr);
	        String linea=null;
	        nlinea=1;
	        int nivel=0;
	        int nivelAnterior=0;
	        Vector<String> idPadreNivel = new Vector<String>();idPadreNivel.add("NULL");
	        Vector<Integer> posSigEnNivel = new Vector<Integer>();posSigEnNivel.add(1);
	        String idNodo=null;
	        if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Importing from "+nomArchivo+":");
	        Vector<Vector<String>> nodosHijo = null;
	        do {
	        	nodosHijo = this.consultarProgramacionNodoHijos(idNodo,MateriaClase.getIdMateriaDeTrabajo());
	        	if (nodosHijo!=null){
	        		if (nodosHijo.size() > 0) {
	        			if (idNodo!=null) {
	        				idPadreNivel.add(idNodo);
		        			posSigEnNivel.add(1);
	        			}
        				posSigEnNivel.set(posSigEnNivel.size()-1, Integer.valueOf(nodosHijo.lastElement().get(2)));
	        			idNodo=nodosHijo.lastElement().get(0);
	        		} else {nodosHijo=null;}
	        	}
	        } while (nodosHijo!=null);
	        nivelAnterior=posSigEnNivel.size()-1;
	        do {
	        	try {
	        		linea=br.readLine();
	        		if (linea!=null) {
	        			nivel=0;
	        			while (linea.charAt(nivel)=='\t') nivel++;
	        			if (nivel==nivelAnterior+1) {
	        				idPadreNivel.add(idNodo);
	        				posSigEnNivel.add(1);
	        			} else if (nivel==nivelAnterior) {
	        				Integer ultimo=posSigEnNivel.lastElement();
	        				posSigEnNivel.set(posSigEnNivel.size()-1, ultimo+1);
	        			} else if (nivel<nivelAnterior) {
	        				for (int i=0;i<nivelAnterior-nivel;i++) {
	        					idPadreNivel.remove(idPadreNivel.size()-1);
	        					posSigEnNivel.remove(posSigEnNivel.size()-1);
	        				}
            				Integer ultimo=posSigEnNivel.lastElement();
            				posSigEnNivel.set(posSigEnNivel.size()-1, ultimo+1);
	        			} else {
	        				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Error importing statements from "+nomArchivo+" in line "+nlinea+" more than one level down.");
	        				String m=PCuaderno.MiPCuaderno.mensajes.getString("ErrImportStatements1")+" '"+nomArchivo+"' "+PCuaderno.MiPCuaderno.mensajes.getString("ErrImportStatements2")+" "+nlinea+", "+PCuaderno.MiPCuaderno.mensajes.getString("ErrImportStatements3")+" "+nivelAnterior+", "+PCuaderno.MiPCuaderno.mensajes.getString("ErrImportStatements4")+" "+nivel+".";
	        				PCuaderno.MiPCuaderno.informacion(m);
	        				break;
	        			}
	        			String enunciado=linea.substring(nivel);
	        			if (enunciado.length()>256) enunciado=enunciado.substring(0,256);
	    				String sql="INSERT INTO PROGRAMACION_NODO (PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO)"+
	    						" VALUES ("+((idPadreNivel.size()<=0)?"NULL":idPadreNivel.lastElement())+
	    						","+posSigEnNivel.lastElement()+","+MateriaClase.getIdMateriaDeTrabajo()+
	    						",'"+enunciado+"','');";
	    				idNodo = this.sqlInsertarId("PROGRAMACION_NODO",sql);
	    				nivelAnterior=nivel;
	        			nlinea++;
	        	        //System.out.print(".");
	        		}
				} catch (IOException e2) {e2.printStackTrace();}
	        } while (linea != null);
	        //System.out.println("");
        }
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("from "+nomArchivo+" "+nlinea+" lines with records inserted.");
	}
	public void borrarEvento(String idEvento){
		int ejecucion=-1;
		if (idEvento!=null) {
			if (idEvento!="") {
				String sql=null;
				sql="SELECT EVENTOS.TIPO FROM EVENTOS WHERE EVENTOS.ID="+idEvento+";";
				Vector<Vector<String>> respuesta=this.sqlConsulta(sql);
				if (respuesta!=null) {
					if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoEvaluacion) {
						sql="DELETE FROM EVALUACIONES WHERE EVALUACIONES.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoClaustro) {
						sql="DELETE FROM CLAUSTROS WHERE CLAUSTROS.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoConsejoEscolar) {
						sql="DELETE FROM C_ESCOLARES WHERE C_ESCOLARES.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoCitaConTutores) {
						sql="DELETE FROM ESTUDIANTES_CITATUTORES WHERE ESTUDIANTES_CITATUTORES.EVENTO_CITATUTORES="+idEvento+";";
						this.sqlEjecuta(sql);
						sql="DELETE FROM CITAS_TUTORES WHERE CITAS_TUTORES.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoETCP) {
						sql="DELETE FROM ETCPS WHERE ETCPS.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoOtroTipo) {
						sql="DELETE FROM OTROS_EVENTOS WHERE OTROS_EVENTOS.ID="+idEvento+";";
					}
				}
				sql+="DELETE FROM EVENTOS WHERE EVENTOS.ID="+idEvento+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+") deleted "+ejecucion);
	}
	public void modificarEstudiantesCitaTutor(String idCita, Vector<String> nuevos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+","+nuevos+")");
		if (idCita!=null && nuevos!=null) if (!idCita.equals("")) {
			borrarEstudiantesCitaTutores(idCita);
			for (int i=0;i<nuevos.size();i++){
				insertarEstudianteCitaTutores(idCita,nuevos.get(i));
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+","+nuevos+") end");
	}
	public String insertarEstudiante(String idClase,String nombre,String apellidos){
		String id=null;
		if (idClase!=null && nombre!=null && apellidos!=null) {
			if (!idClase.equals("") && !nombre.equals("") && !apellidos.equals("")) {
				String sql="INSERT INTO ESTUDIANTES (CLASE,NOMBRE,APELLIDOS) VALUES ("+idClase+",'"+nombre+"','"+apellidos+"');";
				id=this.sqlInsertarId("ESTUDIANTES", sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+nombre+","+apellidos+") id inserted "+id);
		return id;
	}
	public void borrarEstudiantes(){
		int ejecucion=-1;
		String sql="DELETE FROM ESTUDIANTE_MATERIACLASE; DELETE FROM EE_ESTUDIANTES; DELETE FROM EVALUACIONES_EVALUABLES;" +
				"DELETE FROM ESTUDIANTES_CITATUTORES;";
		Vector<Vector<String>> citasTutores = this.sqlConsulta("SELECT ID FROM CITAS_TUTORES;");
		if (citasTutores!=null) if (citasTutores.size()>0) {
			for (int i=0;i<citasTutores.size();i++) {
				sql+="DELETE FROM CITAS_TUTORES;";
				sql+="DELETE FROM EVENTOS WHERE EVENTOS.ID="+citasTutores.get(i).get(0)+";";
			}
		}
		sql+="DELETE FROM ESTUDIANTES;";
		ejecucion = this.sqlEjecuta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("appointment with tutors ("+((citasTutores!=null)?citasTutores.size():"null")+") total deleted "+ejecucion);
	}
	public void borrarEstudiante(String idEstudiante){
		int ejecucion=-1;
		Vector<Vector<String>> citasTutores=null;
		Vector<Vector<String>> evalEvaluable=null;
		if (idEstudiante!=null) if (!idEstudiante.equals("")) {
			String sql="DELETE FROM ESTUDIANTES_CITATUTORES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE="+idEstudiante+";";
			citasTutores = this.sqlConsulta("SELECT EVENTO_CITATUTORES FROM ESTUDIANTES_CITATUTORES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE="+idEstudiante+";");
			if (citasTutores!=null) if (citasTutores.size()>0) {
				for (int i=0;i<citasTutores.size();i++) {
					Vector<Vector<String>> citas=this.consultarEstudiantesCitaTutores(citasTutores.get(i).get(0));
					if (citas!=null) if (citas.size()==1) {
						sql+="DELETE FROM CITAS_TUTORES WHERE CITAS_TUTORES.ID="+citasTutores.get(i).get(0)+";";
						sql+="DELETE FROM EVENTOS WHERE EVENTOS.ID="+citasTutores.get(i).get(0)+";";
					}
				}
			}
			evalEvaluable = this.sqlConsulta("SELECT EVALUACIONES_EVALUABLES.ID,EVALUACIONES_EVALUABLES.GRUPO" +
					" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
					" WHERE EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
					" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante+";");
			sql+="DELETE FROM EE_ESTUDIANTES WHERE EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante+";";
			if (evalEvaluable!=null) if (evalEvaluable.size()>0) {
				for (int i=0;i<evalEvaluable.size();i++) {
					if (evalEvaluable.get(i).get(1).equals("")) 
						sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
					else {
						Vector<Vector<String>> estudiantesGrupo=this.consultarEstudiantesEvalEvaluable(evalEvaluable.get(i).get(0));
						if (estudiantesGrupo!=null) if (estudiantesGrupo.size()==1)
							sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
					}
				}
			}
			sql+="DELETE FROM ESTUDIANTE_MATERIACLASE WHERE ESTUDIANTE_MATERIACLASE.ESTUDIANTE="+idEstudiante+";";
			sql+="DELETE FROM ESTUDIANTES WHERE ESTUDIANTES.ID="+idEstudiante+";";
			ejecucion=this.sqlEjecuta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+") evalEvaluable ("+((evalEvaluable!=null)?evalEvaluable.size():"null")+") appointment with tutors ("+((citasTutores!=null)?citasTutores.size():"null")+") total deleted "+ejecucion);
	}
	public void insertarEstudianteCitaTutores(String idCita,String idEstudiante){
		int ejecucion=-1;
		if (idCita!=null && idEstudiante!=null) {
			if (!idCita.equals("") && !idEstudiante.equals("")) {
				String sql="INSERT INTO ESTUDIANTES_CITATUTORES (ESTUDIANTE,EVENTO_CITATUTORES)"+
				" VALUES ("+idEstudiante+","+idCita+");";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+","+idEstudiante+") inserted "+ejecucion);
	}
	public void borrarEstudiantesCitaTutores(String idCita){
		int ejecucion=-1;
		if (idCita!=null) if (idCita!="") {
				String sql="DELETE FROM ESTUDIANTES_CITATUTORES WHERE EVENTO_CITATUTORES="+idCita+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+") deleted "+ejecucion);
	}
	public void modificarClaseEvaluacion(String idEvaluacion, String idClase){
		int ejecucion=-1;
		if (idEvaluacion!=null && idClase!=null) {
			if (!idEvaluacion.equals("") && !idClase.equals("")) {
				String sql="UPDATE EVALUACIONES SET EVALUACIONES.CLASE="+idClase+
				" WHERE EVALUACIONES.ID="+idEvaluacion+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluacion+","+idClase+") records modified "+ejecucion);
	}
	public void modificarClaseTutor(String idClase, String idTutor){
		int ejecucion=-1;
		if (idClase!=null && idTutor!=null) {
			if (!idClase.equals("") && !idTutor.equals("")) {
				String sql="UPDATE CLASES SET CLASES.TUTOR="+idTutor+
				" WHERE CLASES.ID="+idClase+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+idTutor+") records modified "+ejecucion);
	}
	public void modificarOrdinarioEvento(String idEvento, boolean Ordinario){
		int ejecucion=-1;
		if (idEvento!=null) {
			if (!idEvento.equals("")) {
				String sql=null;
				sql="SELECT EVENTOS.TIPO FROM EVENTOS WHERE EVENTOS.ID="+idEvento+";";
				Vector<Vector<String>> respuesta=this.sqlConsulta(sql);
				if (respuesta!=null) {
					if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoClaustro) {
						sql="UPDATE CLAUSTROS SET CLAUSTROS.ORDINARIO='"+String.valueOf(Ordinario).toUpperCase()+"'"+
						" WHERE CLAUSTROS.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoConsejoEscolar) {
						sql="UPDATE C_ESCOLARES SET C_ESCOLARES.ORDINARIO='"+String.valueOf(Ordinario).toUpperCase()+"'"+
						" WHERE C_ESCOLARES.ID="+idEvento+";";
					}
				}
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+","+Ordinario+") records modified "+ejecucion);
	}
	public void sqlEjecutaUpdateTextDemorado() {
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() text="+TextoDemorado.clase.ctrl.getText());
		String sql=null;
		int ejecucion=-1;
		if (TextoDemorado.clase!=null && TextoDemorado.sqlId1!=null) {
			if (PCuaderno.MiPCuaderno.evento.getTextAreaSumario()==TextoDemorado.clase.ctrl){
				sql="UPDATE EVENTOS SET EVENTOS.SUMARIO='"+BBDD.sqlDemoradoINFO+"' WHERE EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaPreparativos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVENTOS SET EVENTOS.PREPARATIVOS='"+BBDD.sqlDemoradoINFO+"' WHERE EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaConclusion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVENTOS SET EVENTOS.CONCLUSION='"+BBDD.sqlDemoradoINFO+"' WHERE EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextCitaTutoresRelacion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE CITAS_TUTORES SET CITAS_TUTORES.RELACION='"+BBDD.sqlDemoradoINFO+"' WHERE CITAS_TUTORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				PCuaderno.MiPCuaderno.calendario.actualizarEtiquetaDia(PCuaderno.MiPCuaderno.evento.getFechaComboFecha().fechaSeleccionada());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaEvaluacionDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUACIONES SET EVALUACIONES.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUACIONES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				PCuaderno.MiPCuaderno.calendario.actualizarEtiquetaDia(PCuaderno.MiPCuaderno.evento.getFechaComboFecha().fechaSeleccionada());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaOtroDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE OTROS_EVENTOS SET OTROS_EVENTOS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE OTROS_EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextReferencia()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUABLES SET EVALUABLES.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.evaluable.actualizarSoloListaEvaluables();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextPeso()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUABLES SET EVALUABLES.PESO='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextAreaEnunciado()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUABLES SET EVALUABLES.ENUNCIADO='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextAreaEvaluacionesObservaciones()==TextoDemorado.clase.ctrl){
				sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.OBSERVACIONES='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUACIONES_EVALUABLES.ID="+this.consultarIdEvalEstudianteEvaluable(TextoDemorado.sqlId1,TextoDemorado.sqlId2)+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextEvaluacionesNota()==TextoDemorado.clase.ctrl){
					String idEval=this.consultarIdEvalEstudianteEvaluable(TextoDemorado.sqlId1, TextoDemorado.sqlId2);
					Point posicion=TextoDemorado.clase.ctrl.getSelection();
					if (TextoDemorado.clase.ctrl.getText().equals("")){
						if (idEval!=null) {
							Vector<Vector<String>> evaluacion = this.consultarEval(idEval);
							//System.out.println("ejecutaSqlDemorado idEval = "+idEval+" grupo="+evaluacion.get(0).get(4));
							if (evaluacion.get(0).get(4).equals("")){ // Si no es grupo
								//System.out.println("borrando");
								this.borrarEvalEvaluable(idEval);
								PCuaderno.MiPCuaderno.evaluable.actualizarNotaObservaciones();
								TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(0);
							} else { // Si pertenece a un grupo 
								sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.NOTA='"+BBDD.sqlDemoradoINFO+
									"' WHERE EVALUACIONES_EVALUABLES.ID="+idEval+";";
								//System.out.println("Inactividad Nota: "+sql+" | "+TextoDemorado.clase.ctrl.getText());
								ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,"0");
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
								TextoDemorado.modificaSinActualizar(TextoDemorado.clase.ctrl,"0");
								TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(1);
							}
						}
					} else {
						//String nota=Numero.validarNumeroDecimal(TextoDemorado.clase.ctrl.getText(),Numero.notaMax,true);
						String nota=TextoDemorado.clase.ctrl.getText();
						if (nota!=null) {
							if (idEval==null) {
								sql="INSERT INTO EVALUACIONES_EVALUABLES (EVALUABLE,NOTA,OBSERVACIONES,GRUPO)" +
										" VALUES ("+TextoDemorado.sqlId2+","+BBDD.sqlDemoradoINFO+",'','');";
								sql=sql.replaceFirst(BBDD.sqlDemoradoINFO, String.valueOf(Numero.numeroDecimal(nota)));
								//System.out.println("Inactividad Nota1: "+sql+" | "+TextoDemorado.clase.ctrl.getText());
								String id=this.sqlInsertarId("EVALUACIONES_EVALUABLES", sql);					
								sql="INSERT INTO EE_ESTUDIANTES (E_EVALUABLE,ESTUDIANTE)" +
										" VALUES ("+BBDD.sqlDemoradoINFO+","+TextoDemorado.sqlId1+");";
								//System.out.println("Inactividad Nota2: "+sql+" | "+id);
								ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,id);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" () insertando evaluacion y evaluacion_estudiante. "+TextoDemorado.clase.ctrl.getText());
								PCuaderno.MiPCuaderno.evaluable.actualizarNotaObservaciones();
								TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(1);
							} else {
								sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.NOTA='"+BBDD.sqlDemoradoINFO+
										"' WHERE EVALUACIONES_EVALUABLES.ID="+idEval+";";
								//System.out.println("Inactividad Nota: "+sql+" | "+TextoDemorado.clase.ctrl.getText());
								ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,String.valueOf(Numero.numeroDecimal(nota)));
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
							}
						}
					}
					TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextEvaluacionGrupoNombre()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.GRUPO='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUACIONES_EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.evaluable.actualizarGrupoEvaluacion();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtDiaNoLectivoDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE DIASNOLECTIVOS SET DIASNOLECTIVOS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE DIASNOLECTIVOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Vector<String> noLectivo = this.consultarDiaNoLectivo(TextoDemorado.sqlId1);
				Calendar f = Fecha.fecha(noLectivo.get(2), noLectivo.get(3), noLectivo.get(4));
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.calendario.actualizarEtiquetaDia(f);
				PCuaderno.MiPCuaderno.mantenimiento.actualizarDiasNoLectivosLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesReferencia()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarTiposEvaluablesLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesPeso()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.PESO='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarTiposEvaluablesLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesMinimo()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.MINIMO='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,String.valueOf(Numero.numeroDecimal(TextoDemorado.clase.ctrl.getText())));
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.programacion.getTextSumario()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.SUMARIO='"+BBDD.sqlDemoradoINFO+"' WHERE PROGRAMACION_NODO.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.programacion.getTextDuracion()==TextoDemorado.clase.ctrl){
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				String duracion=null;
				Vector<Vector<String>> temporal = this.consultarProgramacionDuracionNodo(TextoDemorado.sqlId1, TextoDemorado.sqlId2);
				if (TextoDemorado.clase.ctrl.getText().equals("")){
					if (temporal!=null) if (temporal.size()>0){
						this.borrarProgramacionDuracionNodo(temporal.get(0).get(0), temporal.get(0).get(1));
					}
					duracion="0";
				} else {
					if (temporal!=null) if (temporal.size()>0){
						sql="UPDATE PROGRAMACION_TEMPORAL SET PROGRAMACION_TEMPORAL.DURACION='"+BBDD.sqlDemoradoINFO+"' WHERE" +
								" PROGRAMACION_TEMPORAL.NODO="+TextoDemorado.sqlId1+" AND PROGRAMACION_TEMPORAL.CLASE="+TextoDemorado.sqlId2+";";
					} else {					
						sql="INSERT INTO PROGRAMACION_TEMPORAL (NODO,CLASE,DURACION)" +
								" VALUES ("+TextoDemorado.sqlId1+","+TextoDemorado.sqlId2+","+BBDD.sqlDemoradoINFO+");";
					}
					duracion=Numero.validarNumeroEntero(TextoDemorado.clase.ctrl.getText(),null,false);
					ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,String.valueOf(Numero.numeroEntero(duracion)));
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				}
				PCuaderno.MiPCuaderno.programacion.getTreeConIdIndice().fijarMinutosRequeridos(
						PCuaderno.MiPCuaderno.programacion.getTreeConIdIndice().getNodo(TextoDemorado.sqlId1)
						,Numero.numeroEntero(duracion));
				PCuaderno.MiPCuaderno.programacion.actualizarTerminaNodo();
				PCuaderno.MiPCuaderno.programacion.actualizarRequeridoTotal();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEnsenanzaRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ENSENANZAS SET ENSENANZAS.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE ENSENANZAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarEnsenanzas(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEnsenanza()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ENSENANZAS SET ENSENANZAS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE ENSENANZAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtTituloRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TITULOS SET TITULOS.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE TITULOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarTitulos(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtTitulo()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TITULOS SET TITULOS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE TITULOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtNivelRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE NIVELES SET NIVELES.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE NIVELES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarNiveles(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtClaseRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE CLASES SET CLASES.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE CLASES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarClases(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtMateriaRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE MATERIAS SET MATERIAS.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE MATERIAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarMaterias(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtMateria()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE MATERIAS SET MATERIAS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE MATERIAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEstudianteNombre()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ESTUDIANTES SET ESTUDIANTES.NOMBRE='"+BBDD.sqlDemoradoINFO+"' WHERE ESTUDIANTES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarEstudiantes(null);
				PCuaderno.MiPCuaderno.matricula.actualizarMatriculados(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEstudianteApellidos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ESTUDIANTES SET ESTUDIANTES.APELLIDOS='"+BBDD.sqlDemoradoINFO+"' WHERE ESTUDIANTES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarEstudiantes(null);
				PCuaderno.MiPCuaderno.matricula.actualizarMatriculados(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEstudianteDatos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ESTUDIANTES SET ESTUDIANTES.DATOS='"+BBDD.sqlDemoradoINFO+"' WHERE ESTUDIANTES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtProfesorNombre()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROFESORES SET PROFESORES.NOMBRE='"+BBDD.sqlDemoradoINFO+"' WHERE PROFESORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarProfesoresLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtProfesorApellidos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROFESORES SET PROFESORES.APELLIDOS='"+BBDD.sqlDemoradoINFO+"' WHERE PROFESORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarProfesoresLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtProfesorDatos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROFESORES SET PROFESORES.DATOS='"+BBDD.sqlDemoradoINFO+"' WHERE PROFESORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=sqlEjecutaReemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() records modified="+ejecucion+" text="+TextoDemorado.clase.ctrl.getText());
			}
		}
	}
	public int sqlEjecutaReemplazando(String sql,String exp,String valor){
		if (TextoDemorado.clase.numero)
			sql=sql.replaceFirst(exp, valor.replace("'", ""));
		else
			sql=sql.replaceFirst(exp, TextoDemorado.quitaComillaSimple(valor));
		//System.out.println("Inactividad: "+sql);
		return this.sqlEjecuta(sql);
	}
	public void modificarHoraEvento(String idEvento, Calendar hora){
		int ejecucion=-1;
		if (idEvento!=null && hora!=null) {
			if (!idEvento.equals("")) {
				String sql="UPDATE EVENTOS SET EVENTOS.HORA='"+Fecha.bbddSoloHora(hora)+"'"+
				" WHERE EVENTOS.ID="+idEvento+";";
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+","+((hora!=null)?Fecha.formaFechaDiaHora(hora):"null")+") records modified "+ejecucion);
	}
	public void modificarFechaHoraEvaluables(Vector<String> idEvaluables, Calendar Ff, String idUhF){
		int ejecucion=-1;
		if (idEvaluables!=null && Ff!=null && idUhF!=null) {
			if (idEvaluables.size()>0 && !idUhF.equals("")) {
				String sql="";
				for (int i=0;i<idEvaluables.size();i++) {
					sql+="UPDATE EVALUABLES SET"+
					" EVALUABLES.DIA_FIN='"+Fecha.bbddSoloDia(Ff)+"'"+
					", EVALUABLES.UH_FIN="+idUhF+
					" WHERE EVALUABLES.ID="+idEvaluables.get(i)+";";
					//System.out.println("sql "+sql);
				}
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluables+","+Fecha.formaFecha(Ff)+","+idUhF+") records modified "+ejecucion);
	}
	public void modificarFechaEvento(String idEvento, Calendar fecha){
		int ejecucion=-1;
		if (idEvento!=null) {
			if (!idEvento.equals("")) {
				String sql="UPDATE EVENTOS SET EVENTOS.DIA="+((fecha!=null)?"'"+Fecha.bbddSoloDia(fecha)+"'":" NULL ")+
				" WHERE EVENTOS.ID="+idEvento+";";
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+","+Fecha.formaFecha(fecha)+") records modified "+ejecucion);
	}
	public String insertarEvento(Integer idTipo,Calendar fecha,String preparativos,String sumario,String conclusion){
		String id=null;
		if (idTipo!=null && fecha!=null) {
			if (idTipo>=0) {
				String sql="INSERT INTO EVENTOS (DIA,HORA,SUMARIO,PREPARATIVOS,CONCLUSION,TIPO)"+
				" VALUES ('"+Fecha.bbddSoloDia(fecha)+"','"+Fecha.bbddSoloHora(fecha)+"','"+((preparativos!=null)?sumario:"")+"','"+((preparativos!=null)?preparativos:"")+"','"+((conclusion!=null)?conclusion:"")+"',"+idTipo+");";
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Insertando Evento ("+idTipo+")");
				id = this.sqlInsertarId("EVENTOS",sql);
				if (id!=null) if (!id.equals("")) {
					if (idTipo==BBDD.idTipoEvaluacion) {
						String clase = MateriaClase.getIdClaseDeTrabajo();
						if (clase==null) clase=this.consultarClase(null, null).get(0).get(0);
						sql="INSERT INTO EVALUACIONES (ID,DESCRIPCION,CLASE) VALUES ("+id+",'',"+clase+");";
					} else if (idTipo==BBDD.idTipoClaustro) {
						sql="INSERT INTO CLAUSTROS (ID,ORDINARIO) VALUES ("+id+",true);";
					} else if (idTipo==BBDD.idTipoConsejoEscolar) {
						sql="INSERT INTO C_ESCOLARES (ID,ORDINARIO) VALUES ("+id+",true);";
					} else if (idTipo==BBDD.idTipoCitaConTutores) {
						sql="INSERT INTO CITAS_TUTORES (ID,RELACION) VALUES ("+id+",'');";
					} else if (idTipo==BBDD.idTipoETCP) {
						sql="INSERT INTO ETCPS (ID) VALUES ("+id+");";
					} else if (idTipo==BBDD.idTipoOtroTipo) {
						sql="INSERT INTO OTROS_EVENTOS (ID,DESCRIPCION) VALUES ("+id+",'');";
					}
					this.sqlEjecuta(sql);
				} else if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Can't insert event.");
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTipo+") id inserted = "+id);
		return id;
	}
	/** 
	 * Insert type of assessable (only idUH param can be null)
	 * @return identifier of assessable inserted<br>
	 */
	public String insertarEvaluable(Calendar fecha,String idUH,String referencia,String enunciado,Integer peso,Integer tipo,Integer recupera){
		String id=null;
		if (fecha!=null && referencia!=null && enunciado!=null && peso!=null && tipo!=null){
			String sql="INSERT INTO EVALUABLES (DIA_FIN,UH_FIN,REFERENCIA,ENUNCIADO,PESO,TIPO,RECUPERA)"+
			" VALUES ('"+Fecha.bbddSoloDia(fecha)+"',"+((idUH==null)?this.consultarIdUHoraria(fecha):idUH)+",'"+referencia+"','"+enunciado+"',"+peso+","+tipo+","+((recupera==null)?"NULL":recupera)+");";
			id = this.sqlInsertarId("EVALUABLES",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+","+idUH+","+referencia+","+enunciado+","+peso+","+tipo+") id inserted = "+id);
		return id;
	}
	public String insertarEvaluable(String tipo){
		int ejecucion=-1;
		String id=null;
		if (tipo!=null) if (!tipo.equals("")) {
			Calendar fecha = null;
			if (Parcial.getParcialEstricto(Fecha.fechaDeTrabajo)==-1 && Parcial.getParcial(Fecha.fechaDeTrabajo)!=-1) {
				fecha=Parcial.getFechaInicioParcial(Parcial.getParcial(Fecha.fechaDeTrabajo));
				fecha=Fecha.getFechaHoraLaboralOSiguiente(fecha, true);
			} else {
				fecha = (Calendar) Fecha.fechaDeTrabajo.clone();
				fecha=Fecha.getFechaHoraLaboralOSiguiente(fecha, true);
				if (Parcial.getParcialEstricto(fecha)==-1 && Parcial.getParcial(fecha)!=-1) {
					fecha=Parcial.getFechaInicioParcial(Parcial.getParcial(Fecha.fechaDeTrabajo));
					fecha=Fecha.getFechaHoraLaboralOSiguiente(fecha, true);
				}
			}
			String sql="INSERT INTO EVALUABLES (DIA_FIN,UH_FIN,REFERENCIA,ENUNCIADO,PESO,TIPO)"+
			" VALUES ('"+Fecha.bbddSoloDia(fecha)+"',"+this.consultarIdUHoraria(fecha)+",'','',1,"+tipo+");";
			id = this.sqlInsertarId("EVALUABLES",sql);
			String idNodo=PCuaderno.MiPCuaderno.arbolDeTrabajo.idNodoParaMinutosRequeridos(MateriaClase.minutosHastaFechaEnMateriaClaseTrabajo(fecha));
			if (idNodo!=null) {
				sql="INSERT INTO EVALUABLES_NODOS (EVALUABLE,NODO) VALUES ("+id+","+idNodo+")";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+tipo+") id inserted = "+id+" and records inserted in EVALUABLES_NODOS "+ejecucion);
		return id;
	}
	public Vector<Vector<String>> consultarEstudiantesTutoriaProfesor (String idProfesor){
		Vector<Vector<String>> tabla=null;
		if (idProfesor!=null) if (!idProfesor.equals("")) {
			String sql="SELECT DISTINCT ESTUDIANTES.ID,ESTUDIANTES.NOMBRE,ESTUDIANTES.APELLIDOS" +
					" FROM ESTUDIANTES,CLASES" +
					" WHERE ESTUDIANTES.CLASE=CLASES.ID" +
					" AND CLASES.TUTOR="+idProfesor + 
					" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idProfesor+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public String consultarIdClaseTutoria (String idProfesor){
		String resultado=null;
		Vector<Vector<String>> tabla=null;
		String sql="SELECT CLASES.ID FROM CLASES" +
				" WHERE CLASES.TUTOR="+idProfesor+";";
		tabla=this.sqlConsulta(sql);
		if (tabla!=null) if (tabla.size()>0) resultado=tabla.get(0).get(0);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+resultado);
		return resultado;
	}
	public String consultarIdPropio (){
		String id=null;
		Vector<Vector<String>> usuario=null;
		String sql="SELECT * FROM USUARIO;";
		usuario=this.sqlConsulta(sql);
		if (usuario!=null) if (usuario.size()>0) id=usuario.get(0).get(0);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+id);
		return id;
	}
	public void insertarUsuario(String id){
		int ejecucion=0;
		if (id!=null) 
			if (!id.equals(""))
				if (this.consultarIdPropio()==null){
					String sql="INSERT INTO USUARIO (PROFESOR)"+
						" VALUES ("+id+");";
					ejecucion = this.sqlEjecuta(sql);
				}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+ejecucion+" records inserted");
	}
	public String insertarProfesor(String nombre, String apellidos, String datos){
		String id=null;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nombre+","+apellidos+")");
		if (nombre!=null && apellidos!=null && apellidos!=null) 
			if (!nombre.equals("")) {
				String sql="INSERT INTO PROFESORES (NOMBRE,APELLIDOS,DATOS)"+
			" VALUES ('"+nombre+"','"+apellidos+"','"+datos+"');";
			id = this.sqlInsertarId("PROFESORES",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nombre+","+apellidos+") id inserted = "+id);
		return id;
	}
	public void borrarProfesor(String idProfesor){
		int ejecucion=-1;
		if (idProfesor!=null) {
			if (!idProfesor.equals("")) {
				String claseTutoria = this.consultarIdClaseTutoria(idProfesor);
				if (claseTutoria==null) {
					String sql="DELETE FROM PROFESORES WHERE PROFESORES.ID="+idProfesor+";";
					ejecucion=this.sqlEjecuta(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idProfesor+") deleted "+ejecucion);
	}
	public Vector<Vector<String>> consultarProfesor(String id){
		Vector<Vector<String>> tabla=null;
		String sql="SELECT PROFESORES.ID,PROFESORES.NOMBRE,PROFESORES.APELLIDOS,PROFESORES.DATOS,PROFESORES.ID=USUARIO.PROFESOR"+
				" FROM PROFESORES, USUARIO"+
				((id!=null)?" WHERE PROFESORES.ID="+id:"")+
				" ORDER BY PROFESORES.APELLIDOS,PROFESORES.NOMBRE;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEventosTipoDia(String tipo,Calendar fecha){
		Vector<Vector<String>> tabla; // si fecha=null => cualquier fecha y tipo=null => cualquier tipo 
		String sql="SELECT EVENTOS.ID,DAYOFMONTH(EVENTOS.DIA),MONTH(EVENTOS.DIA),YEAR(EVENTOS.DIA)," +
				" HOUR(EVENTOS.HORA),MINUTE(EVENTOS.HORA), EVENTOS.SUMARIO,EVENTOS.PREPARATIVOS," +
				" EVENTOS.CONCLUSION,TIPOS_EVENTO.ID,TIPOS_EVENTO.DESCRIPCION,TIPOS_EVENTO.COLOR " +
				" FROM EVENTOS, TIPOS_EVENTO" +
				" WHERE EVENTOS.TIPO=TIPOS_EVENTO.ID" +
				((tipo!=null)?((Integer.parseInt(tipo)!=0)?" AND EVENTOS.TIPO="+tipo:""):"")+
				" AND EVENTOS.DIA='"+((fecha!=null)?Fecha.bbddSoloDia(fecha):Fecha.bbddSoloDia(Fecha.inicioCurso))+"'"+
				" ORDER BY EVENTOS.DIA,EVENTOS.HORA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+tipo+","+Fecha.formaFecha(fecha)+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEventosTipoDesdeFecha(String tipo,Calendar fecha){
		Vector<Vector<String>> tabla; // si fecha=null => cualquier fecha y tipo=null => cualquier tipo 
		String sql="SELECT EVENTOS.ID,DAYOFMONTH(EVENTOS.DIA),MONTH(EVENTOS.DIA),YEAR(EVENTOS.DIA)," +
				" HOUR(EVENTOS.HORA),MINUTE(EVENTOS.HORA), EVENTOS.SUMARIO,EVENTOS.PREPARATIVOS," +
				" EVENTOS.CONCLUSION,TIPOS_EVENTO.ID,TIPOS_EVENTO.DESCRIPCION,TIPOS_EVENTO.COLOR " +
				" FROM EVENTOS, TIPOS_EVENTO" +
				" WHERE EVENTOS.TIPO=TIPOS_EVENTO.ID" +
				((tipo!=null)?((Integer.parseInt(tipo)!=0)?" AND EVENTOS.TIPO="+tipo:""):"")+
				" AND EVENTOS.DIA>='"+((fecha!=null)?Fecha.bbddSoloDia(fecha):Fecha.bbddSoloDia(Fecha.inicioCurso))+"'"+
				" ORDER BY EVENTOS.DIA,EVENTOS.HORA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+tipo+","+Fecha.formaFecha(fecha)+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarTiposEvento(){
		Vector<Vector<String>> tabla;
		String sql="SELECT ID,DESCRIPCION,COLOR,BBDD FROM TIPOS_EVENTO;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEstudiantesCitaTutores(String idCitaTutores){
		Vector<Vector<String>> tabla=null;
		if (idCitaTutores!=null) {
			String sql="SELECT ESTUDIANTES.ID,ESTUDIANTES.NOMBRE,ESTUDIANTES.APELLIDOS" +
					" FROM ESTUDIANTES_CITATUTORES,ESTUDIANTES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE=ESTUDIANTES.ID" +
					" AND ESTUDIANTES_CITATUTORES.EVENTO_CITATUTORES="+idCitaTutores +
					" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCitaTutores+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<String> consultarIdEstudiantesCitaTutores(String idCitaTutores) {
		Vector<String> registro=null;
		if (idCitaTutores!=null) {
			String sql="SELECT ESTUDIANTES.ID" +
					" FROM ESTUDIANTES_CITATUTORES,ESTUDIANTES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE=ESTUDIANTES.ID" +
					" AND ESTUDIANTES_CITATUTORES.EVENTO_CITATUTORES="+idCitaTutores +
					" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
			registro=this.sqlConsulta1Columna(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCitaTutores+") "+((registro!=null)?registro.size():"null")+" students");
		return registro;
	}
	/** 
	 * Get tipo events 
	 * @return <br>
	 * Evaluacion
	 * 0 - EVALUACIONES.ID<br>
	 * 1 - EVALUACIONES.DESCRIPCION<br>
	 * 2 - CLASES.ID<br>
	 * 3 - NIVELES.REFERENCIA<br>
	 * 4 - CLASES.REFERENCIA<br>
	 * 5 - TITULOS.REFERENCIA<br>
	 * 6 - PROFESORES.APELLIDOS<br>
	 * 7 - PROFESORES.NOMBRE<br>
	 * Claustro<br>
	 * 0 - ID<br>
	 * 1 - ORDINARIO<br>
	 * ConsejoEscolar<br>
	 * 0 - ID<br>
	 * 1 - ORDINARIO<br>
	 * CitaTutores<br>
	 * 0 - ID<br>
	 * 1 - RELACION<br>
	 * Eetcp<br>
	 * 0 - ID<br>
	 * Otro<br>
	 * 0 - ID<br>
	 * 1 - DESCRIPCION<br>
	 */
	public Vector<Vector<String>> consultarEventoTipoDatos(String id, int tipo){
		Vector<Vector<String>> tabla=null;
		String sql=null;
		//System.out.println("Evento tipo="+tipo+", id="+id);
		if (id!=null) if (!id.equals("") && tipo>=0) {
			if (tipo==BBDD.idTipoEvaluacion) {
				sql="SELECT EVALUACIONES.ID,EVALUACIONES.DESCRIPCION, CLASES.ID, NIVELES.REFERENCIA," +
						" CLASES.REFERENCIA,TITULOS.REFERENCIA,PROFESORES.APELLIDOS,PROFESORES.NOMBRE" +
						" FROM EVALUACIONES,CLASES,PROFESORES,NIVELES,TITULOS" +
						" WHERE EVALUACIONES.ID="+id+" AND CLASES.ID=EVALUACIONES.CLASE" +
						" AND CLASES.TUTOR=PROFESORES.ID AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID;";
			} else if (tipo==BBDD.idTipoClaustro) {
				sql="SELECT ID,ORDINARIO FROM CLAUSTROS WHERE ID="+id+";";
			} else if (tipo==BBDD.idTipoConsejoEscolar) {
				sql="SELECT ID,ORDINARIO FROM C_ESCOLARES WHERE ID="+id+";";
			} else if (tipo==BBDD.idTipoCitaConTutores) {
				sql="SELECT ID,RELACION FROM CITAS_TUTORES WHERE CITAS_TUTORES.ID="+id+";";
			} else if (tipo==BBDD.idTipoETCP) {
				sql="SELECT ID FROM ETCPS WHERE ID="+id+";";
			} else if (tipo==BBDD.idTipoOtroTipo) {
				sql="SELECT ID,DESCRIPCION FROM OTROS_EVENTOS WHERE ID="+id+";";
			}
			//System.out.println("sql="+sql);
			if (sql!=null)tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+","+tipo+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get events (can be null, get all events)
	 * @return <br>
	 * 0 - EVENTOS.ID<br>
	 * 1 - DAYOFMONTH(EVENTOS.DIA)<br>
	 * 2 - MONTH(EVENTOS.DIA)<br>
	 * 3 - YEAR(EVENTOS.DIA)<br>
	 * 4 - HOUR(EVENTOS.HORA)<br>
	 * 5 - MINUTE(EVENTOS.HORA)<br>
	 * 6 - EVENTOS.SUMARIO<br>
	 * 7 - EVENTOS.PREPARATIVOS<br>
	 * 8 - EVENTOS.CONCLUSION<br>
	 * 9 - TIPOS_EVENTO.ID<br>
	 * 10 - TIPOS_EVENTO.DESCRIPCION<br>
	 * 11 - TIPOS_EVENTO.COLOR<br>
	 */
	public Vector<Vector<String>> consultarEvento(String id){
		Vector<Vector<String>> tabla=null;
		String sql="SELECT EVENTOS.ID,DAYOFMONTH(EVENTOS.DIA),MONTH(EVENTOS.DIA),YEAR(EVENTOS.DIA)," +
				" HOUR(EVENTOS.HORA),MINUTE(EVENTOS.HORA), EVENTOS.SUMARIO,EVENTOS.PREPARATIVOS," +
				" EVENTOS.CONCLUSION,TIPOS_EVENTO.ID,TIPOS_EVENTO.DESCRIPCION,TIPOS_EVENTO.COLOR " +
				" FROM EVENTOS, TIPOS_EVENTO" +
				" WHERE EVENTOS.TIPO=TIPOS_EVENTO.ID" +
				((id!=null)?" AND EVENTOS.ID="+id:"")+
				" ORDER BY EVENTOS.DIA,EVENTOS.HORA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public boolean consultarUHvacio(String idMateriaClase){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		String sql="SELECT ID FROM UH_SEMANA" +
				((idMateriaClase!=null)?" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase:"")+
				" LIMIT 1;";
		tabla=this.sqlConsulta(sql);	
		if (tabla!=null) if (tabla.size()>0) resultado=false;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public Vector<Vector<String>> consultarUHOrdenadaMateriaClase(String idMateriaClase,int diaSemana){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null) if (!idMateriaClase.equals("") && diaSemana>=0) {
			String sql="SELECT UH_SEMANA.ID,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO),HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN) FROM UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+" AND UH_SEMANA.DIA_SEMANA="+diaSemana+" ORDER BY UH_SEMANA.INICIO;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+diaSemana+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
		
	}
	public Vector<Vector<String>> consultarUHMateriaClaseDiaDesdeHora(String idMateriaClase,Calendar fecha){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null && fecha!=null) if (!idMateriaClase.equals("")) {
			String sql="SELECT UH_SEMANA.ID,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)," +
					"HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN) FROM UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" AND UH_SEMANA.DIA_SEMANA="+fecha.get(Calendar.DAY_OF_WEEK)+
					" AND UH_SEMANA.FIN>'"+Fecha.bbddSoloHora(fecha)+"'"+
					" ORDER BY UH_SEMANA.INICIO;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+Fecha.formaFechaDiaHora(fecha)+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get uh for subject (can be null) and/or class (can be null)
	 * @return assessable identifier<br>
	 * 0 - UH_SEMANA.ID<br>
	 * 1 - UH_SEMANA.MATERIACLASE<br>
	 * 2 - HOUR(UH_SEMANA.INICIO)<br>
	 * 3 - MINUTE(UH_SEMANA.INICIO)<br>
	 * 4 - HOUR(UH_SEMANA.FIN)<br>
	 * 5 - MINUTE(UH_SEMANA.FIN)<br>
	 * 6 - UH_SEMANA.DIA_SEMANA<br>
	 */
	public Vector<Vector<String>> consultarUHMateriaClase(String idMateria, String idClase){
		Vector<Vector<String>> tabla=null;
		String sql="SELECT UH_SEMANA.ID,UH_SEMANA.MATERIACLASE,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)," +
				"HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN),UH_SEMANA.DIA_SEMANA FROM UH_SEMANA,MATERIA_CLASE" +
				" WHERE UH_SEMANA.MATERIACLASE=MATERIA_CLASE.ID"+
				((idMateria!=null)?" AND MATERIA_CLASE.MATERIA="+idMateria:"")+
				((idClase!=null)?" AND MATERIA_CLASE.CLASE="+idClase:"")+
				" ORDER BY UH_SEMANA.INICIO;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+idClase+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public void borrarUH(String idUH){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+")");
		int ejecucion=-1;
		Vector<String> evaluables=null;
		Vector<String> evalEvaluables=null;
		if (idUH!=null) if (!idUH.equals("")) {
			String sql="SELECT EVALUABLES.ID" +
				" FROM EVALUABLES"+
				" WHERE EVALUABLES.UH_FIN="+idUH+";";
			evaluables=this.sqlConsulta1Columna(sql);
			sql="";
			if (evaluables!=null) if (evaluables.size()>0) {
				for (int j=0;j<evaluables.size();j++){
					evalEvaluables=this.consultarEvaluacionesEvaluable(evaluables.get(j));
					if (evalEvaluables!=null) if (evalEvaluables.size()>0) {
						for (int i=0;i<evalEvaluables.size();i++){
							sql+="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+evalEvaluables.get(i)+";";
							sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+evalEvaluables.get(i)+";";
						}
					}
					sql+="DELETE FROM EVALUABLES_NODOS WHERE EVALUABLE="+evaluables.get(j)+";";
					sql+="DELETE FROM EVALUABLES WHERE ID="+evaluables.get(j)+";";
				}
			}
			sql+="DELETE FROM UH_SEMANA WHERE ID="+idUH+";";
			//System.out.println("sql:"+sql);
			ejecucion=this.sqlEjecuta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+") evaluables ("+((evaluables!=null)?evaluables.size():"null")+") evalEvaluables ("+((evalEvaluables!=null)?evalEvaluables.size():"null")+") deleted "+ejecucion);
	}
	public String insertarUHsiguiente(String idMateriaClase,String idDiaSemana){
		String id=null;
		Vector<Vector<String>> uhs=null;
		if (idMateriaClase!=null && idDiaSemana!=null) 
			if (!idMateriaClase.equals("") && !idDiaSemana.equals("")) {
				Calendar inicio=null;
				Calendar fin=null;
				String sql="SELECT HOUR(FIN),MINUTE(FIN) FROM UH_SEMANA WHERE UH_SEMANA.DIA_SEMANA="+idDiaSemana+
						" ORDER BY FIN ASC;";
				uhs=this.sqlConsulta(sql);
				if (uhs==null || uhs.size()==0) {
					inicio=Fecha.hora(Fecha.laboralPrimera, 0);
				} else {
					inicio=Fecha.hora(uhs.lastElement().get(0),uhs.lastElement().get(1));
				}
				fin=(Calendar)inicio.clone();
				fin.add(Calendar.HOUR_OF_DAY, +1);
				sql="INSERT INTO UH_SEMANA (MATERIACLASE,DIA_SEMANA,INICIO,FIN)"+
					" VALUES ("+idMateriaClase+","+idDiaSemana+",'"+Fecha.bbddSoloHora(inicio)+
					"','"+Fecha.bbddSoloHora(fin)+"');";
				id = this.sqlInsertarId("UH_SEMANA",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idDiaSemana+") uhs ("+uhs+") id inserted = "+id);
		return id;
	}
	public boolean consultarUHsolapada(String idUH,String idDiaSemana,String horaInicio,String minutoInicio,String horaFin,String minutoFin){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		if (idDiaSemana!=null && horaInicio!=null && minutoInicio!=null 
				&& horaFin!=null && minutoFin!=null) {
			if (!idDiaSemana.equals("") && !horaInicio.equals("") && !minutoInicio.equals("") 
					&& !horaFin.equals("") && !minutoFin.equals("")) {
				resultado=false;
				String inicio=Fecha.bbddSoloHora(Fecha.hora(horaInicio,minutoInicio));
				String fin=Fecha.bbddSoloHora(Fecha.hora(horaFin,minutoFin));
				String sql="SELECT UH_SEMANA.ID FROM UH_SEMANA" +
						" WHERE UH_SEMANA.ID<>"+idUH+" AND UH_SEMANA.DIA_SEMANA="+idDiaSemana+
						" AND ((INICIO<'"+inicio+"' AND FIN>'"+inicio+"')"+
						" OR (INICIO<'"+fin+"' AND FIN>'"+fin+"')" +
						" OR (INICIO='"+inicio+"' AND FIN='"+fin+"')"+
						" OR (INICIO>'"+inicio+"' AND FIN<'"+fin+"')"+
						" OR (INICIO<'"+inicio+"' AND FIN>'"+fin+"'));";
				tabla=this.sqlConsulta(sql);	
				//System.out.println("sql("+tabla.size()+"):"+sql);
				if (tabla!=null) if (tabla.size()>0) resultado=true;
			}
		}
		return resultado;
	}
	public void modificarUH(String idUH,String idDiaSemana,String idMateriaClase,String horaInicio,String minutoInicio,String horaFin,String minutoFin){
		int ejecucion=-1;
		if (idDiaSemana!=null && idMateriaClase!=null && horaInicio!=null && minutoInicio!=null 
				&& horaFin!=null && minutoFin!=null) {
			if (!idDiaSemana.equals("") && !idMateriaClase.equals("") && !horaInicio.equals("") && !minutoInicio.equals("") 
					&& !horaFin.equals("") && !minutoFin.equals("")) {
				String sql="UPDATE UH_SEMANA SET DIA_SEMANA="+idDiaSemana+",MATERIACLASE="+idMateriaClase+
					",INICIO='"+Fecha.bbddSoloHora(Fecha.hora(horaInicio,minutoInicio))+"'"+
					",FIN='"+Fecha.bbddSoloHora(Fecha.hora(horaFin,minutoFin))+"'"+
					" WHERE UH_SEMANA.ID="+idUH+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+","+idDiaSemana+","+idMateriaClase+","+horaInicio+","+minutoInicio+","+horaFin+","+minutoFin+") records modified "+ejecucion);		
	}
	/** 
	 * Get uh for subject-class (can't be null)
	 * @return assessable identifier<br>
	 * 0 - UH_SEMANA.ID<br>
	 * 1 - UH_SEMANA.DIA_SEMANA<br>
	 * 2 - HOUR(UH_SEMANA.INICIO)<br>
	 * 3 - MINUTE(UH_SEMANA.INICIO)<br>
	 * 4 - HOUR(UH_SEMANA.FIN)<br>
	 * 5 - MINUTE(UH_SEMANA.FIN)<br>
	 */
	public Vector<Vector<String>> consultarUHMateriaClase(String idMateriaClase){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null) if (!idMateriaClase.equals("")) {
			String sql="SELECT UH_SEMANA.ID,UH_SEMANA.DIA_SEMANA,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)," +
					"HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN) FROM UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" ORDER BY UH_SEMANA.INICIO;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
		
	}
	public String consultarIdUHoraria(Calendar fecha){
		Vector<Vector<String>> tabla=null;		// 8:30 a 9:30 es de 8:30 a 9:29
		String resultado=null;
		String sql="SELECT UH_SEMANA.ID" +
				" FROM UH_SEMANA" +
				" WHERE UH_SEMANA.DIA_SEMANA="+fecha.get(Calendar.DAY_OF_WEEK)+
				" AND UH_SEMANA.INICIO<='"+Fecha.bbddSoloHora(fecha)+"'"+
				" AND UH_SEMANA.FIN>'"+Fecha.bbddSoloHora(fecha)+"';";
		//System.out.println("consultarIdUHoraria: "+sql);
		tabla=this.sqlConsulta(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
						resultado=tabla.get(0).get(0);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public Vector<Vector<String>> consultarUH(String diaSemana,String idUH,boolean ordenDiaSemana){
		Vector<Vector<String>> tabla=null;
		String sql="SELECT UH_SEMANA.ID,UH_SEMANA.MATERIACLASE,UH_SEMANA.DIA_SEMANA,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)" +
			",HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN),MATERIAS.REFERENCIA,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA" +
			",ENSENANZAS.REFERENCIA FROM UH_SEMANA,MATERIA_CLASE,MATERIAS,CLASES,NIVELES,TITULOS,ENSENANZAS" +
			" WHERE UH_SEMANA.MATERIACLASE=MATERIA_CLASE.ID AND MATERIA_CLASE.MATERIA=MATERIAS.ID" +
			" AND MATERIA_CLASE.CLASE=CLASES.ID AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID" +
			" AND TITULOS.ENSENANZA=ENSENANZAS.ID"+
			((diaSemana!=null)?" AND UH_SEMANA.DIA_SEMANA="+diaSemana:"")+
			((idUH!=null)?" AND UH_SEMANA.ID="+idUH:"")+
			" ORDER BY "+((ordenDiaSemana)?"UH_SEMANA.DIA_SEMANA,":"")+"UH_SEMANA.INICIO,UH_SEMANA.FIN;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+diaSemana+","+idUH+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;	
	}
	public void borrarEstudianteGrupoEvalEvaluable(String idEstudiante, String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null) {
			if (idEvalEvaluable!="") {
				String sql="DELETE FROM EE_ESTUDIANTES WHERE" +
						" E_EVALUABLE="+idEvalEvaluable+" AND ESTUDIANTE="+idEstudiante+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEvalEvaluable+") deleted "+ejecucion);
	}
	public void borrarEvaluable(Vector<String> idEvaluables){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((idEvaluables!=null)?idEvaluables.size():"null")+" ids)");
		int ejecucion=-1;
		Vector<String> registro=null;
		if (idEvaluables!=null) if (idEvaluables.size()>0){
			String sql="";
			for (int i=0;i<idEvaluables.size();i++) {
				registro=this.consultarEvaluacionesEvaluable(idEvaluables.get(i));
				if (registro!=null) if (registro.size()>0) {
					for (int j=0;j<registro.size();j++){
						sql+="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+registro.get(j)+";";
						sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+registro.get(j)+";";
					}
				}
				sql+="DELETE FROM EVALUABLES_NODOS WHERE EVALUABLE="+idEvaluables.get(i)+";";
				sql+="DELETE FROM EVALUABLES WHERE ID="+idEvaluables.get(i)+";";
			}
			ejecucion=this.sqlEjecuta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((idEvaluables!=null)?idEvaluables.size():"null")+" ids) evalEvaluables ("+((registro!=null)?registro.size():"null")+") deleted "+ejecucion);
	}
	public void borrarEvalEvaluable(String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null) {
			if (idEvalEvaluable!="") {
				String sql="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+idEvalEvaluable+";";
				sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+idEvalEvaluable+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvalEvaluable+") deleted "+ejecucion);
	}
	public String insertarEvalEstudianteEvaluable(String idEstudiante,String idEvaluable,String nota){
		int ejecucion=-1;
		String id=null;
		if (idEstudiante!=null && idEvaluable!=null) {
			if (idEstudiante!="" && idEvaluable!="") {
				String sql="INSERT INTO EVALUACIONES_EVALUABLES (EVALUABLE,NOTA,OBSERVACIONES,GRUPO)"+
				" VALUES ("+idEvaluable+","+String.valueOf(Numero.numeroDecimal(nota))+",'','');";
				//System.out.println("Insertando EvalEvaluable para evaluable ("+idEvaluable+") para estudiante ("+idEstudiante+")");
				id = this.sqlInsertarId("EVALUACIONES_EVALUABLES",sql);
				if (id!=null) if (id!="") {
					sql="INSERT INTO EE_ESTUDIANTES (E_EVALUABLE,ESTUDIANTE)"+
					" VALUES ("+id+","+idEstudiante+");";
					ejecucion=this.sqlEjecuta(sql);
					//System.out.println("Insertando estudiante EvalEvaluable ("+id+")");
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEvaluable+","+nota+") idEvalEvaluable ("+id+") records inserted "+ejecucion);
		return id;
	}
	public void insertarEstudiantesMateriaClase(String idMateriaClase,Vector<String> idEstudiantes){
		int ejecucion=-1;
		if (idEstudiantes!=null && idMateriaClase!=null)
			if (idEstudiantes.size()>0 && !idMateriaClase.equals("")) {
				String sql="";
				for (int i=0;i<idEstudiantes.size();i++) {
					Vector<Vector<String>> estudiante=this.consultarEstudiantesMateriaClase(idMateriaClase, idEstudiantes.get(i));
					if (estudiante.size()==0) 
						sql+="INSERT INTO ESTUDIANTE_MATERIACLASE" +
							" (MATERIACLASE,ESTUDIANTE) VALUES ("+idMateriaClase+","+idEstudiantes.get(i)+");";
				}
				if (!sql.equals("")) {
					if (PCuaderno.noti) ejecucion += this.sqlEjecuta(sql);
					else this.sqlEjecuta(sql);
				}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idEstudiantes+") inserted = "+ejecucion);
	}
	public void borrarEstudiantesMateriaClase(String idMateriaClase,Vector<String> idEstudiantes){
		int ejecucion=-1;
		Vector<Vector<String>> evalEvaluable=null;
		if (idEstudiantes!=null && idMateriaClase!=null)
			if (idEstudiantes.size()>0 && !idMateriaClase.equals("")) {
				String sql="";
				for (int j=0;j<idEstudiantes.size();j++) {
					// debe borrar la evaluaciones 
					evalEvaluable = this.sqlConsulta("SELECT EVALUACIONES_EVALUABLES.ID,EVALUACIONES_EVALUABLES.GRUPO" +
							" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
							" WHERE EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
							" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiantes.get(j)+";");
					sql+="DELETE FROM EE_ESTUDIANTES WHERE EE_ESTUDIANTES.ESTUDIANTE="+idEstudiantes.get(j)+";";
					if (evalEvaluable!=null) if (evalEvaluable.size()>0) {
						for (int i=0;i<evalEvaluable.size();i++) {
							if (evalEvaluable.get(i).get(1).equals("")) 
								sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
							else {
								Vector<Vector<String>> estudiantesGrupo=this.consultarEstudiantesEvalEvaluable(evalEvaluable.get(i).get(0));
								if (estudiantesGrupo!=null) if (estudiantesGrupo.size()==1)
									sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
							}
						}
					}
					sql+="DELETE FROM ESTUDIANTE_MATERIACLASE" +
							" WHERE MATERIACLASE="+idMateriaClase+" AND ESTUDIANTE="+idEstudiantes.get(j)+";";
				}
				if (PCuaderno.noti) ejecucion += this.sqlEjecuta(sql);
				else this.sqlEjecuta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idEstudiantes+") inserted = "+ejecucion);
	}
	public void insertarEstudianteGrupoEvalEvaluable(String idEstudiante,String idEval){
		int ejecucion=-1;
		if (idEstudiante!=null && idEval!=null) {
			if (idEstudiante!="" && idEval!="") {
				String sql="INSERT INTO EE_ESTUDIANTES (E_EVALUABLE,ESTUDIANTE)"+
				" VALUES ("+idEval+","+idEstudiante+");";
				ejecucion=this.sqlEjecuta(sql);
				//System.out.println("Insertando EvalEvaluable ("+idEvalEvaluable+")");
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEval+") inserted "+ejecucion);
	}
	public Vector<Vector<String>> consultarEstudiantesEvalEvaluable(String idEvalEvaluable){
		Vector<Vector<String>> tabla;
		String sql="SELECT EE_ESTUDIANTES.ESTUDIANTE,ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE" +
				" FROM EE_ESTUDIANTES,ESTUDIANTES" +
				" WHERE EE_ESTUDIANTES.ESTUDIANTE=ESTUDIANTES.ID" +
				" AND EE_ESTUDIANTES.E_EVALUABLE="+idEvalEvaluable+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvalEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public String consultarIdMateriaClaseEvaluable(String idEvaluable){
		String resultado=null;
		Vector<Vector<String>> tabla;
		String sql="SELECT UH_SEMANA.MATERIACLASE" +
				" FROM EVALUABLES,UH_SEMANA" +
				" WHERE EVALUABLES.UH_FIN=UH_SEMANA.ID" +
				" AND EVALUABLES.ID="+idEvaluable+";";
		tabla=this.sqlConsulta(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public Vector<Vector<String>> consultarEstudiantesEvalSinGrupoOEvalEvaluable(String idEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvaluable!=null) if (!idEvaluable.equals("")) {
			tabla=this.consultarEstudiantesMateriaClase(this.consultarIdMateriaClaseEvaluable(idEvaluable),null);
			Vector<String> estudiantesMateriaConGrupo;
			String sql="SELECT ESTUDIANTES.ID" +
			" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES,ESTUDIANTES" +
			" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable+
			" AND EVALUACIONES_EVALUABLES.GRUPO<>''" +
			" AND EE_ESTUDIANTES.E_EVALUABLE=EVALUACIONES_EVALUABLES.ID" +
			" AND EE_ESTUDIANTES.ESTUDIANTE=ESTUDIANTES.ID" +
			" ORDER BY ESTUDIANTES.ID;";
			estudiantesMateriaConGrupo=this.sqlConsulta1Columna(sql);
			if (estudiantesMateriaConGrupo!=null) if (estudiantesMateriaConGrupo.size()>0) {
				for (int i=0;i<estudiantesMateriaConGrupo.size();i++){
					for (int j=0;j<tabla.size();j++){
						if (estudiantesMateriaConGrupo.get(i).equals(tabla.get(j).get(0))) {
							tabla.removeElementAt(j);
							//System.out.println("Borrando id "+estudiantesMateriaConGrupo.get(i)+" ("+j+") tamaño "+tabla.size());
							j=tabla.size();
						}
					}
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}/*
	public Vector<Vector<String>> consultarIdEstudiantesEvalEvaluable(String idEvalEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvalEvaluable!=null) if (!idEvalEvaluable.equals("")) {
			String sql="SELECT EE_ESTUDIANTES.ESTUDIANTE" +
					" FROM EE_ESTUDIANTES" +
					" WHERE EE_ESTUDIANTES.E_EVALUABLE="+idEvalEvaluable+";";
			tabla=this.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvalEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}*/
	public void modificarNotaEvalEvaluable(String nota,String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null) {
			if (!idEvalEvaluable.equals("")) {
				String sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.NOTA="+String.valueOf(Numero.numeroDecimal(nota))+
				" WHERE EVALUACIONES_EVALUABLES.ID="+idEvalEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nota+","+idEvalEvaluable+") records modified "+ejecucion);
	}
	public void modificarGrupoEvalEvaluable(String grupo,String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null && grupo!=null) {
			if (!idEvalEvaluable.equals("") && !grupo.equals("")) {
				String sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.GRUPO='"+grupo+"'"+
				" WHERE EVALUACIONES_EVALUABLES.ID="+idEvalEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+grupo+","+idEvalEvaluable+") records modified "+ejecucion);
	}
	public String consultarIdEvalEstudianteEvaluable(String idEstudiante,String idEvaluable){
		Vector<Vector<String>> tabla;
		String resultado=null;
		String sql="SELECT DISTINCT EVALUACIONES_EVALUABLES.ID" +
				" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
				" WHERE EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante+
				" AND EE_ESTUDIANTES.E_EVALUABLE=EVALUACIONES_EVALUABLES.ID" +
				" AND EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable+";";
		tabla=this.sqlConsulta(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
						//System.out.println("(estudiante="+idEstudiante+",evaluable="+idEvaluable+") idEval = "+resultado);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public String insertarTipoEvaluable(String idGrupoMateria, String referencia, String descripcion, Integer peso, String minimo){
		String id=null;
		if (referencia!=null && descripcion!=null && peso!=null && minimo!=null) {
			if (idGrupoMateria!=null) if (idGrupoMateria.equals(BBDD.idTodos)) idGrupoMateria=null;
			String sql="INSERT INTO TIPOS_EVALUABLE (MATERIA,REFERENCIA,DESCRIPCION,PESO,MINIMO)"+
			" VALUES ("+((idGrupoMateria==null)?"NULL":idGrupoMateria)+",'"+referencia+
			"','"+descripcion+"',"+peso+","+String.valueOf(Numero.numeroDecimal(minimo))+");";
			id = this.sqlInsertarId("EVENTOS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idGrupoMateria+","+referencia+","+descripcion+","+peso+","+minimo+") id inserted "+id);
		return id;
	}
	public void borrarTipoEvaluable(String idTipo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTipo+")");
		int ejecucion=-1;
		Vector<String> evaluables=null;
		Vector<String> evalEvaluables=null;
		if (idTipo!=null) if (!idTipo.equals("")) {
			String sql="SELECT EVALUABLES.ID" +
				" FROM EVALUABLES"+
				" WHERE EVALUABLES.TIPO="+idTipo+";";
			evaluables=this.sqlConsulta1Columna(sql);
			sql="";
			if (evaluables!=null) if (evaluables.size()>0) {
				for (int j=0;j<evaluables.size();j++){
					evalEvaluables=this.consultarEvaluacionesEvaluable(evaluables.get(j));
					if (evalEvaluables!=null) if (evalEvaluables.size()>0) {
						for (int i=0;i<evalEvaluables.size();i++){
							sql+="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+evalEvaluables.get(i)+";";
							sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+evalEvaluables.get(i)+";";
						}
					}
					sql+="DELETE FROM EVALUABLES WHERE ID="+evaluables.get(j)+";";
				}
			}
			sql+="DELETE FROM TIPOS_EVALUABLE WHERE ID="+idTipo+";";
			ejecucion=this.sqlEjecuta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTipo+") evaluables ("+((evaluables!=null)?evaluables.size():"null")+") evalEvaluables ("+((evalEvaluables!=null)?evalEvaluables.size():"null")+") deleted "+ejecucion);
	}
	public Vector<Vector<String>> consultarTiposEvaluableMateriaClaseParcial(String idMateriaClase,Integer parcial){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null && parcial!=null) if (!idMateriaClase.equals("") && parcial>0) {
			Calendar ip=((parcial>1)?Parcial.getFechaInicioParcial(parcial):Fecha.inicioCurso);
			Calendar fp=((parcial<Parcial.numeroParciales())?Fecha.diaLaboralOAnterior(Parcial.getFechaInicioParcial(parcial+1),true):Fecha.finCurso);
			String sql="SELECT DISTINCT TIPOS_EVALUABLE.ID, TIPOS_EVALUABLE.REFERENCIA," +
					" TIPOS_EVALUABLE.DESCRIPCION, TIPOS_EVALUABLE.PESO, TIPOS_EVALUABLE.MINIMO" +
					" FROM TIPOS_EVALUABLE,EVALUABLES,UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" AND EVALUABLES.UH_FIN=UH_SEMANA.ID" +
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(ip)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(fp)+"'"+
					" AND EVALUABLES.RECUPERA IS NULL" + // Si solo hay recuperaciones de otro parcial no mostrar tipo para el parcial
					" AND EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
					" ORDER BY TIPOS_EVALUABLE.PESO ASC;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+parcial+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get types of assessable for a subject (can be null, get for all subject).
	 * @return Types of assessable sort by TIPOS_EVALUABLE.REFERENCIA<br>
	 * 0 - TIPOS_EVALUABLE.ID<br>
	 * 1 - TIPOS_EVALUABLE.REFERENCIA<br>
	 * 2 - TIPOS_EVALUABLE.DESCRIPCION<br>
	 * 3 - TIPOS_EVALUABLE.PESO<br>
	 * 4 - TIPOS_EVALUABLE.MINIMO<br>
	 */
	public Vector<Vector<String>> consultarTiposEvaluablesParaMateria(String idMateria){
		Vector<Vector<String>> tabla=null;  // Si idGrupoMateria es nulo obtiene los tipos que afectan a todas las materias
		String sql="SELECT TIPOS_EVALUABLE.ID, TIPOS_EVALUABLE.REFERENCIA," +
				" TIPOS_EVALUABLE.DESCRIPCION, TIPOS_EVALUABLE.PESO, TIPOS_EVALUABLE.MINIMO" +
				" FROM TIPOS_EVALUABLE" +
				" WHERE TIPOS_EVALUABLE.MATERIA"+((idMateria==null)?" IS NULL":"="+idMateria)+
				" ORDER BY TIPOS_EVALUABLE.REFERENCIA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public boolean consultarTipoEvaluableVacio(String idMateria){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		String sql="SELECT TIPOS_EVALUABLE.ID FROM TIPOS_EVALUABLE" +
				" WHERE TIPOS_EVALUABLE.MATERIA IS NULL"+
				((idMateria!=null)?" OR TIPOS_EVALUABLE.MATERIA="+idMateria:"")+
				" LIMIT 1;";
		tabla=this.sqlConsulta(sql);	
		if (tabla!=null) if (tabla.size()>0) resultado=false;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public Vector<Vector<String>> consultarTiposEvaluables(String id){
		Vector<Vector<String>> tabla; // id = nulo para consultarlos todos
		String sql="SELECT TIPOS_EVALUABLE.ID, TIPOS_EVALUABLE.REFERENCIA," +
				" TIPOS_EVALUABLE.DESCRIPCION, TIPOS_EVALUABLE.PESO" +
				" TIPOS_EVALUABLE.MATERIA, TIPOS_EVALUABLE.MINIMO" +
				" FROM TIPOS_EVALUABLE" +
				((id!=null)?" WHERE TIPOS_EVALUABLE.ID="+id:"")+
				" ORDER BY TIPOS_EVALUABLE.REFERENCIA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<String> consultarEvaluacionesEvaluable(String idEvaluable){
		Vector<String> registro=null;
		if (idEvaluable!=null) if (!idEvaluable.equals("")) {
			String sql="SELECT EVALUACIONES_EVALUABLES.ID" +
					" FROM EVALUACIONES_EVALUABLES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +";";
			registro=this.sqlConsulta1Columna(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((registro!=null)?registro.size():"null")+" records");
		return registro;
	}
	public Vector<Vector<String>> consultarEvaluacionesEvaluableGrupos(String idEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvaluable!=null) if (!idEvaluable.equals("")) {
			String sql="SELECT EVALUACIONES_EVALUABLES.ID, EVALUACIONES_EVALUABLES.EVALUABLE," +
					" EVALUACIONES_EVALUABLES.NOTA,EVALUACIONES_EVALUABLES.OBSERVACIONES, EVALUACIONES_EVALUABLES.GRUPO" +
					" FROM EVALUACIONES_EVALUABLES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +
					" AND EVALUACIONES_EVALUABLES.GRUPO<>''" +
					" ORDER BY EVALUACIONES_EVALUABLES.GRUPO;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public String consultarEvalNota(String idEvaluable,String idEstudiante){
		Vector<Vector<String>> tabla=null;
		String resultado=null;
		if (idEvaluable!=null && idEstudiante!=null) if (!idEvaluable.equals("") && !idEstudiante.equals("")) {
			String sql="SELECT EVALUACIONES_EVALUABLES.NOTA" +
					" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +
					" AND EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
					" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante +
					" ORDER BY EVALUACIONES_EVALUABLES.GRUPO;";
			tabla=this.sqlConsulta(sql);
			if (tabla!=null)
				if (tabla.size()>0)
					if (tabla.get(0)!=null)
							resultado=tabla.get(0).get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public String consultarEvalObservaciones(String idEvaluable,String idEstudiante){
		Vector<Vector<String>> tabla=null;
		String resultado=null;
		if (idEvaluable!=null && idEstudiante!=null) if (!idEvaluable.equals("") && !idEstudiante.equals("")) {
			String sql="SELECT EVALUACIONES_EVALUABLES.OBSERVACIONES" +
					" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +
					" AND EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
					" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante +
					" ORDER BY EVALUACIONES_EVALUABLES.GRUPO;";
			tabla=this.sqlConsulta(sql);
			if (tabla!=null)
				if (tabla.size()>0)
					if (tabla.get(0)!=null)
							resultado=tabla.get(0).get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public Vector<Vector<String>> consultarEval(String idEval){
		Vector<Vector<String>> tabla=null;
		if (idEval!=null) if (!idEval.equals("")) {
			String sql="SELECT ID,EVALUABLE,NOTA,OBSERVACIONES,GRUPO" +
					" FROM EVALUACIONES_EVALUABLES" +
					" WHERE EVALUACIONES_EVALUABLES.ID="+idEval+";";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEval+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEvaluableRecuperacionesDe(String idEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvaluable!=null) if (!idEvaluable.equals("")) {
			String sql="SELECT EVALUABLES.ID" +
					",YEAR(EVALUABLES.DIA_FIN),MONTH(EVALUABLES.DIA_FIN),DAYOFMONTH(EVALUABLES.DIA_FIN),EVALUABLES.UH_FIN"+
			/*5*/	",EVALUABLES.REFERENCIA, EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID"+
					",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION, EVALUABLES.RECUPERA"+
					" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA"+
					" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID"+
					" AND UH_SEMANA.ID=EVALUABLES.UH_FIN"+
					" AND EVALUABLES.RECUPERA="+idEvaluable+
					" ORDER BY EVALUABLES.DIA_FIN ASC;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get one assessable from id field.
	 * @return assessable identifier<br>
	 * 0 - EVALUABLES.ID<br>
	 * 1 - YEAR(EVALUABLES.DIA_FIN)<br>
	 * 2 - MONTH(EVALUABLES.DIA_FIN)<br>
	 * 3 - DAYOFMONTH(EVALUABLES.DIA_FIN)<br>
	 * 4 - EVALUABLES.UH_FIN<br>
	 * 5 - EVALUABLES.REFERENCIA<br>
	 * 6 - EVALUABLES.ENUNCIADO<br>
	 * 7 - EVALUABLES.PESO<br>
	 * 8 - TIPOS_EVALUABLE.ID<br>
	 * 9 - TIPOS_EVALUABLE.REFERENCIA<br>
	 * 10 - TIPOS_EVALUABLE.DESCRIPCION<br>
	 * 11 - EVALUABLES.RECUPERA
	 */
	public Vector<Vector<String>> consultarEvaluable(String idEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvaluable!=null) if (!idEvaluable.equals("")) {
			String sql="SELECT EVALUABLES.ID" +
					",YEAR(EVALUABLES.DIA_FIN),MONTH(EVALUABLES.DIA_FIN),DAYOFMONTH(EVALUABLES.DIA_FIN),EVALUABLES.UH_FIN"+
			/*5*/	",EVALUABLES.REFERENCIA, EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID"+
			/*9*/	",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION, EVALUABLES.RECUPERA"+
					" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA"+
					" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID"+
					" AND UH_SEMANA.ID=EVALUABLES.UH_FIN"+
					" AND EVALUABLES.ID="+idEvaluable+
					" ORDER BY EVALUABLES.DIA_FIN ASC;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public boolean consultarEvaluableConUHvacio(String idUH){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		if (idUH!=null) if (!idUH.equals("")) { 
			String sql="SELECT ID FROM EVALUABLES" +
					" WHERE EVALUABLES.UH_FIN="+idUH+
					" LIMIT 1;";
			tabla=this.sqlConsulta(sql);	
			if (tabla!=null) if (tabla.size()>0) resultado=false;
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	/** 
	 * Get assessments from partial start or course (for fisrt partial), to partial start of the next partial or course en (for last partial).
	 * @return assessments<br>
	 * 0 - EVALUACIONES_EVALUABLES.ID<br>
	 * 1 - EVALUACIONES_EVALUABLES.EVALUABLE<br>
	 * 2 - EVALUACIONES_EVALUABLES.NOTA<br>
	 * 3 - EVALUACIONES_EVALUABLES.OBSERVACIONES<br>
	 * 4 - EVALUACIONES_EVALUABLES.GRUPO<br>
	 * 5 - EVALUABLES.RECUPERA
	 */
	public Vector<Vector<String>> consultarEvalEvaluablesPorEstudianteEnMateriaClaseParcial(String idEstudiante,String idMateriaClase,Integer parcial){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idMateriaClase+","+parcial+")");
		Vector<Vector<String>> tabla=null;
		if (idEstudiante!=null && idMateriaClase!=null && parcial!=null) if (!idEstudiante.equals("") && !idMateriaClase.equals("") && parcial>0) {
			Calendar ip=((parcial>1)?Parcial.getFechaInicioParcial(parcial):Fecha.inicioCurso);
			Calendar fp=((parcial<Parcial.numeroParciales())?Fecha.diaLaboralOAnterior(Parcial.getFechaInicioParcial(parcial+1),true):Fecha.finCurso);
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" from date "+Fecha.formaFechaDiaHora(ip)+", until "+Fecha.formaFechaDiaHora(fp));
			String sql="SELECT EVALUACIONES_EVALUABLES.ID, EVALUACIONES_EVALUABLES.EVALUABLE," +
					" EVALUACIONES_EVALUABLES.NOTA,EVALUACIONES_EVALUABLES.OBSERVACIONES," +
					" EVALUACIONES_EVALUABLES.GRUPO, EVALUABLES.RECUPERA" +
					" FROM UH_SEMANA,EVALUABLES,EVALUACIONES_EVALUABLES,EE_ESTUDIANTES,ESTUDIANTES" +
					" WHERE ESTUDIANTES.ID="+idEstudiante+
					" AND EE_ESTUDIANTES.ESTUDIANTE=ESTUDIANTES.ID" +
					" AND EE_ESTUDIANTES.E_EVALUABLE=EVALUACIONES_EVALUABLES.ID" +
					" AND EVALUACIONES_EVALUABLES.EVALUABLE=EVALUABLES.ID" +
					" AND EVALUABLES.UH_FIN=UH_SEMANA.ID" +
					" AND UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(ip)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(fp)+"'"+
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN ASC;";
			tabla=this.sqlConsulta(sql);
			if (PCuaderno.noti) for (int i=0;i<tabla.size();i++) PCuaderno.MiPCuaderno.notificar(i+": assessment "+tabla.get(i).get(0)+", assessable "+tabla.get(i).get(1)+" remedial of "+tabla.get(i).get(5));
			if (tabla!=null) if (tabla.size()>0) {
				// Remove assessable remedial assessment, of which assessable is out partial 
				Vector<String> idEvaluables = this.consultarIdEvaluablesMateriaClaseParcial(idMateriaClase, parcial);
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("item assessable = "+idEvaluables);
				Vector<Integer> idRecuperacionesOtroParcial = new Vector<Integer>(); 
				for (int i=0;i<tabla.size();i++)
					if (tabla.get(i).get(5)!=null) if (idEvaluables.indexOf(tabla.get(i).get(5))<0) idRecuperacionesOtroParcial.add(i);
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("assessable remedial assessments index, out partial = "+idRecuperacionesOtroParcial);
				for (int i=idRecuperacionesOtroParcial.size()-1;i>=0;i--) tabla.removeElementAt(idRecuperacionesOtroParcial.get(i));
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("size of assessments "+tabla.size());
				// Add assessable remedial assessment, of which remedial is out partial but assessable is in partial. 
				sql="SELECT EVALUACIONES_EVALUABLES.ID, EVALUACIONES_EVALUABLES.EVALUABLE," +
					" EVALUACIONES_EVALUABLES.NOTA,EVALUACIONES_EVALUABLES.OBSERVACIONES, EVALUACIONES_EVALUABLES.GRUPO, EVALUABLES.RECUPERA" +
					" FROM UH_SEMANA,EVALUABLES,EVALUACIONES_EVALUABLES,EE_ESTUDIANTES,ESTUDIANTES" +
					" WHERE ESTUDIANTES.ID="+idEstudiante+
					" AND EE_ESTUDIANTES.ESTUDIANTE=ESTUDIANTES.ID" +
					" AND EE_ESTUDIANTES.E_EVALUABLE=EVALUACIONES_EVALUABLES.ID" +
					" AND EVALUACIONES_EVALUABLES.EVALUABLE=EVALUABLES.ID" +
					" AND EVALUABLES.UH_FIN=UH_SEMANA.ID" +
					" AND UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" AND EVALUABLES.RECUPERA IS NOT NULL"+
					" AND EVALUABLES.DIA_FIN>'"+Fecha.bbddSoloDia(fp)+"'"+
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN ASC;";
				Vector<Vector<String>> recuperaciones=null;
				recuperaciones=this.sqlConsulta(sql);
				for (int i=0;i<recuperaciones.size();i++) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("if "+recuperaciones.get(i).get(5)+" is in then add "+recuperaciones.get(i));
					if (idEvaluables.indexOf(recuperaciones.get(i).get(5))>=0) tabla.add(recuperaciones.get(i));
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" END ("+idEstudiante+","+idMateriaClase+","+parcial+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get assessable identifier from partial start or course (for fisrt partial), to partial start of the next partial or course en (for last partial).
	 * @return ONLY assessable identifier
	 */
	private Vector<String> consultarIdEvaluablesMateriaClaseParcial(String idMateriaClase, Integer parcial) {
		Vector<String> registro=null;
		if (idMateriaClase!=null && parcial!=null) if (!idMateriaClase.equals("") && parcial>0) {
			Calendar ip=((parcial>1)?Parcial.getFechaInicioParcial(parcial):Fecha.inicioCurso);
			Calendar fp=((parcial<Parcial.numeroParciales())?Fecha.diaLaboralOAnterior(Parcial.getFechaInicioParcial(parcial+1),true):Fecha.finCurso);
			String sql="SELECT EVALUABLES.ID FROM EVALUABLES,UH_SEMANA" +
					" WHERE UH_SEMANA.ID=EVALUABLES.UH_FIN" + 
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(ip)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(fp)+"'"+
					" AND UH_SEMANA.MATERIACLASE="+idMateriaClase +
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN,EVALUABLES.REFERENCIA ASC;";
			registro=this.sqlConsulta1Columna(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+parcial+") "+((registro!=null)?registro.size():"null")+" records");
		return registro;
	}
	/** 
	 * Get assessable from partial start or course (for fisrt partial), to partial start of the next partial or course en (for last partial).
	 * @return assessable identifier <br>
	 * 0 - EVALUABLES.ID<br>
	 * 1 - YEAR(EVALUABLES.DIA_FIN)<br>
	 * 2 - MONTH(EVALUABLES.DIA_FIN)<br>
	 * 3 - DAYOFMONTH(EVALUABLES.DIA_FIN)<br>
	 * 4 - HOUR(UH_SEMANA.FIN)<br>
	 * 5 - MINUTE(UH_SEMANA.FIN)<br>
	 * 6 - EVALUABLES.REFERENCIA<br>
	 * 7 - EVALUABLES.ENUNCIADO<br>
	 * 8 - EVALUABLES.PESO<br>
	 * 9 - TIPOS_EVALUABLE.ID<br>
	 * 10 - TIPOS_EVALUABLE.REFERENCIA<br>
	 * 11 - TIPOS_EVALUABLE.DESCRIPCION<br>
	 * 12 - EVALUABLES.RECUPERA
	 */
	public Vector<Vector<String>> consultarEvaluablesMateriaClaseParcial(String idMateriaClase,Integer parcial){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+parcial+")");
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null && parcial!=null) if (!idMateriaClase.equals("") && parcial>0) {
			Calendar ip=((parcial>1)?Parcial.getFechaInicioParcial(parcial):Fecha.inicioCurso);
			Calendar fp=null;
			if (parcial<Parcial.numeroParciales()) {
				fp=Parcial.getFechaInicioParcial(parcial+1);
				fp.add(Calendar.DAY_OF_MONTH, -1);
				fp=Fecha.diaLaboralOAnterior(fp,true);
			} else fp=Fecha.finCurso;
			String sql="SELECT EVALUABLES.ID" +
					",YEAR(EVALUABLES.DIA_FIN), MONTH(EVALUABLES.DIA_FIN), DAYOFMONTH(EVALUABLES.DIA_FIN)" +
					",HOUR(UH_SEMANA.FIN), MINUTE(UH_SEMANA.FIN), EVALUABLES.REFERENCIA" +
					",EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID" +
					",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION, EVALUABLES.RECUPERA" +
					" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA" +
					" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
					" AND UH_SEMANA.ID=EVALUABLES.UH_FIN" +
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(ip)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(fp)+"'"+
					" AND UH_SEMANA.MATERIACLASE="+idMateriaClase +
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN,EVALUABLES.REFERENCIA ASC;";
			tabla=this.sqlConsulta(sql);
			//System.out.println("sql "+sql);
			//BBDD.imprimir(tabla);
			if (PCuaderno.noti) for (int i=0;i<tabla.size();i++) PCuaderno.MiPCuaderno.notificar(i+": assessable "+tabla.get(i).get(0)+" remedial of "+tabla.get(i).get(12));
			if (tabla!=null) if (tabla.size()>0) {
				// Remove assessable remedial assessment, of which assessable is out partial
				Vector<String> idEvaluables = new Vector<String>();
				for (int i=0;i<tabla.size();i++) idEvaluables.add(tabla.get(i).get(0));
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("item assessable = "+idEvaluables);
				Vector<Integer> idRecuperacionesOtroParcial = new Vector<Integer>(); 
				for (int i=0;i<tabla.size();i++) 
					if (tabla.get(i).get(12)!=null) if (idEvaluables.indexOf(tabla.get(i).get(12))<0) idRecuperacionesOtroParcial.add(i);
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("assessable remedial index, out partial = "+idRecuperacionesOtroParcial);
				for (int i=idRecuperacionesOtroParcial.size()-1;i>=0;i--) tabla.removeElementAt(idRecuperacionesOtroParcial.get(i));
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("size of assessable "+tabla.size());
				// Add assessable remedial assessment, of which remedial is out partial but assessable is in partial.
				sql="SELECT EVALUABLES.ID" +
						",YEAR(EVALUABLES.DIA_FIN), MONTH(EVALUABLES.DIA_FIN), DAYOFMONTH(EVALUABLES.DIA_FIN)" +
						",HOUR(UH_SEMANA.FIN), MINUTE(UH_SEMANA.FIN), EVALUABLES.REFERENCIA" +
						",EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID" +
						",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION, EVALUABLES.RECUPERA" +
						" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA" +
						" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
						" AND UH_SEMANA.ID=EVALUABLES.UH_FIN" +
						" AND EVALUABLES.RECUPERA IS NOT NULL"+
						" AND EVALUABLES.DIA_FIN>'"+Fecha.bbddSoloDia(fp)+"'"+
						" AND UH_SEMANA.MATERIACLASE="+idMateriaClase +
						" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN,EVALUABLES.REFERENCIA ASC;";
				Vector<Vector<String>> recuperaciones=null;
				recuperaciones=this.sqlConsulta(sql);
				for (int i=0;i<recuperaciones.size();i++) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("if "+recuperaciones.get(i).get(5)+" is in then add "+recuperaciones.get(i));
					if (idEvaluables.indexOf(recuperaciones.get(i).get(12))>=0) tabla.add(recuperaciones.get(i));
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" END ("+idMateriaClase+","+parcial+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get assessable betwen two dates, include this dates.
	 * @return assessable identifier <br>
	 * 0 - EVALUABLES.ID<br>
	 * 1 - YEAR(EVALUABLES.DIA_FIN)<br>
	 * 2 - MONTH(EVALUABLES.DIA_FIN)<br>
	 * 3 - DAYOFMONTH(EVALUABLES.DIA_FIN)<br>
	 * 4 - HOUR(UH_SEMANA.FIN)<br>
	 * 5 - MINUTE(UH_SEMANA.FIN)<br>
	 * 6 - EVALUABLES.REFERENCIA<br>
	 * 7 - EVALUABLES.ENUNCIADO<br>
	 * 8 - EVALUABLES.PESO<br>
	 * 9 - TIPOS_EVALUABLE.ID<br>
	 * 10 - TIPOS_EVALUABLE.REFERENCIA<br>
	 * 11 - TIPOS_EVALUABLE.DESCRIPCION<br>
	 * 12 - EVALUABLES.RECUPERA
	 */
	public Vector<Vector<String>> consultarEvaluablesMateriaClaseFecha(String idMateriaClase,Calendar fi,Calendar ff){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+fi+","+ff+")");
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null && fi!=null && ff!=null) if (!idMateriaClase.equals("")) {
			String sql="SELECT EVALUABLES.ID" +
					",YEAR(EVALUABLES.DIA_FIN), MONTH(EVALUABLES.DIA_FIN), DAYOFMONTH(EVALUABLES.DIA_FIN)" +
					",HOUR(UH_SEMANA.FIN), MINUTE(UH_SEMANA.FIN), EVALUABLES.REFERENCIA" +
					",EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID" +
					",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION, EVALUABLES.RECUPERA" +
					" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA" +
					" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
					" AND UH_SEMANA.ID=EVALUABLES.UH_FIN" +
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(fi)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(ff)+"'"+
					" AND UH_SEMANA.MATERIACLASE="+idMateriaClase +
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN,EVALUABLES.REFERENCIA ASC;";
			tabla=this.sqlConsulta(sql);
			if (PCuaderno.noti) for (int i=0;i<tabla.size();i++) PCuaderno.MiPCuaderno.notificar(i+": assessable "+tabla.get(i).get(0)+" remedial of "+tabla.get(i).get(12));
			if (tabla!=null) if (tabla.size()>0) {
				// Remove assessable remedial assessment, of which assessable is out partial
				Vector<String> idEvaluables = new Vector<String>();
				for (int i=0;i<tabla.size();i++) idEvaluables.add(tabla.get(i).get(0));
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("item assessable = "+idEvaluables);
				Vector<Integer> idRecuperacionesOtroParcial = new Vector<Integer>(); 
				for (int i=0;i<tabla.size();i++) 
					if (tabla.get(i).get(12)!=null) if (idEvaluables.indexOf(tabla.get(i).get(12))<0) idRecuperacionesOtroParcial.add(i);
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("assessable remedial index, out partial = "+idRecuperacionesOtroParcial);
				for (int i=idRecuperacionesOtroParcial.size()-1;i>=0;i--) tabla.removeElementAt(idRecuperacionesOtroParcial.get(i));
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("size of assessable "+tabla.size());
				// Add assessable remedial assessment, of which remedial is out partial but assessable is in partial.
				sql="SELECT EVALUABLES.ID" +
						",YEAR(EVALUABLES.DIA_FIN), MONTH(EVALUABLES.DIA_FIN), DAYOFMONTH(EVALUABLES.DIA_FIN)" +
						",HOUR(UH_SEMANA.FIN), MINUTE(UH_SEMANA.FIN), EVALUABLES.REFERENCIA" +
						",EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID" +
						",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION, EVALUABLES.RECUPERA" +
						" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA" +
						" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
						" AND UH_SEMANA.ID=EVALUABLES.UH_FIN" +
						" AND EVALUABLES.RECUPERA IS NOT NULL"+
						" AND EVALUABLES.DIA_FIN>'"+Fecha.bbddSoloDia(ff)+"'"+
						" AND UH_SEMANA.MATERIACLASE="+idMateriaClase +
						" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN,EVALUABLES.REFERENCIA ASC;";
				Vector<Vector<String>> recuperaciones=null;
				recuperaciones=this.sqlConsulta(sql);
				for (int i=0;i<recuperaciones.size();i++) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("if "+recuperaciones.get(i).get(5)+" is in then add "+recuperaciones.get(i));
					if (idEvaluables.indexOf(recuperaciones.get(i).get(12))>=0) tabla.add(recuperaciones.get(i));
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" END ("+idMateriaClase+","+fi+","+ff+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEvaluablesNodos(String idEvaluable,String idNodo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+","+idNodo+")");
		Vector<Vector<String>> tabla=null;
		String sql="SELECT * FROM EVALUABLES_NODOS" +
				((idEvaluable!=null || idEvaluable!=null)?" WHERE ":"") +
				((idEvaluable!=null)?" EVALUABLES_NODOS.EVALUABLE="+idEvaluable:"")+
				((idEvaluable!=null && idNodo!=null)?" AND":"")+
				((idNodo!=null)?" EVALUABLES_NODOS.NODO="+idNodo:"")+";";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" END ("+idEvaluable+","+idNodo+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEvaluablesParaNodoYPadres(String idNodo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")");
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null) if (!idNodo.equals("")) {
			tabla=this.consultarEvaluablesParaNodo(idNodo);
			String padre = idNodo;
			do {
				Vector<Vector<String>> info = this.consultarProgramacionNodo(padre);
				padre = info.get(0).get(1);
				if (padre!=null) {
					Vector<Vector<String>> mas = this.consultarEvaluablesParaNodo(padre);
					for (int i=0;i<mas.size();i++) tabla.add(mas.get(i));
				}
			} while (padre!=null);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" END ("+idNodo+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEvaluablesParaNodo(String idNodo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")");
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null) if (!idNodo.equals("")) {
			String sql="SELECT EVALUABLES.ID" +
					",YEAR(EVALUABLES.DIA_FIN), MONTH(EVALUABLES.DIA_FIN), DAYOFMONTH(EVALUABLES.DIA_FIN)" +
					",HOUR(UH_SEMANA.FIN), MINUTE(UH_SEMANA.FIN), EVALUABLES.REFERENCIA" +
					",EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID" +
					",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION, EVALUABLES.RECUPERA" +
					" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA,EVALUABLES_NODOS" +
					" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
					" AND EVALUABLES.ID=EVALUABLES_NODOS.EVALUABLE" +
					" AND UH_SEMANA.ID=EVALUABLES.UH_FIN" +
					" AND EVALUABLES_NODOS.NODO="+idNodo+
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN,EVALUABLES.REFERENCIA ASC;";
			tabla=this.sqlConsulta(sql);
			if (PCuaderno.noti) for (int i=0;i<tabla.size();i++) PCuaderno.MiPCuaderno.notificar(i+": assessable "+tabla.get(i).get(0)+" remedial of "+tabla.get(i).get(12));
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" END ("+idNodo+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEstudiantesMateriaClase(String idMateriaClase,String idEstudiante){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT DISTINCT ESTUDIANTES.ID,APELLIDOS,NOMBRE,DATOS" +
				" FROM ESTUDIANTES, ESTUDIANTE_MATERIACLASE" +
				" WHERE ESTUDIANTES.ID=ESTUDIANTE_MATERIACLASE.ESTUDIANTE" +
				((idMateriaClase!=null)?" AND ESTUDIANTE_MATERIACLASE.MATERIACLASE="+idMateriaClase:"")+
				((idEstudiante!=null)?" AND ESTUDIANTES.ID="+idEstudiante:"")+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idEstudiante+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEstudiantesClase(String idClase,String idEstudiante){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT ESTUDIANTES.ID,APELLIDOS,NOMBRE,DATOS" +
				" FROM ESTUDIANTES" +
				((idClase!=null || idEstudiante!=null)?" WHERE":"")+
				((idClase!=null)?" ESTUDIANTES.CLASE="+idClase:"")+
				((idClase!=null && idEstudiante!=null)?" AND":"")+
				((idEstudiante!=null)?" ESTUDIANTES.ID="+idEstudiante:"")+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+idEstudiante+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarEstudiantesMateria(String idMateria,String idEstudiante){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT DISTINCT ESTUDIANTES.ID,APELLIDOS,NOMBRE,DATOS" +
				" FROM ESTUDIANTES, ESTUDIANTE_MATERIACLASE, MATERIA_CLASE" +
				" WHERE ESTUDIANTES.ID=ESTUDIANTE_MATERIACLASE.ESTUDIANTE" +
				((idMateria!=null)?" AND ESTUDIANTE_MATERIACLASE.MATERIACLASE=MATERIA_CLASE.ID" +
				" AND MATERIA_CLASE.MATERIA="+idMateria:"")+
				((idEstudiante!=null)?" AND ESTUDIANTES.ID="+idEstudiante:"")+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+idEstudiante+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get subject (can be null, get all subject) 
	 * @return <br>
	 * 0 - MATERIAS.ID<br>
	 * 1 - MATERIAS.REFERENCIA<br>
	 * 2 - MATERIAS.DESCRIPCION<br>
	 * 3 - MATERIAS.NIVEL<br>
	 * 4 - NIVELES.REFERENCIA<br>
	 * 5 - TITULOS.REFERENCIA<br>
	 * 6 - ENSENANZAS.REFERENCIA<br>
	 */
	public Vector<Vector<String>> consultarMateria(String idMateria){
		Vector<Vector<String>> tabla;
		String sql="SELECT MATERIAS.ID,MATERIAS.REFERENCIA,MATERIAS.DESCRIPCION,MATERIAS.NIVEL"+
				",NIVELES.REFERENCIA,TITULOS.REFERENCIA,ENSENANZAS.REFERENCIA FROM MATERIAS,NIVELES,TITULOS,ENSENANZAS"+
				" WHERE MATERIAS.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID AND TITULOS.ENSENANZA=ENSENANZAS.ID"+
				((idMateria!=null)?" AND MATERIAS.ID="+idMateria:"")+
				" ORDER BY ENSENANZAS.REFERENCIA,TITULOS.REFERENCIA,NIVELES.REFERENCIA,MATERIAS.REFERENCIA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public String insertarMateria(String idNivel,String referencia,String titulo){
		String id=null;
		int ejecucion=0;
		Vector<Vector<String>> clases=null;
		if (idNivel!=null && referencia!=null && titulo!=null) 
			if (!idNivel.equals("") && !referencia.equals("")) {
				String sql="INSERT INTO MATERIAS (NIVEL,REFERENCIA,DESCRIPCION)" +
						" VALUES ("+idNivel+",'"+referencia+"','"+titulo+"');";
				id = this.sqlInsertarId("CLASES",sql);
				sql="";
				clases = this.consultarClase(idNivel, null);
				if (clases!=null) if (clases.size()>0) {
					for (int i=0;i<clases.size();i++) 
						sql+="INSERT INTO MATERIA_CLASE (MATERIA,CLASE) VALUES ("+id+","+clases.get(i).get(0)+");";
				}
				if (!sql.equals("")) ejecucion=this.sqlEjecuta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+referencia+","+titulo+") classes ("+((clases!=null)?clases.size():"null")+") id inserted = "+id+" records inserted "+ejecucion);
		return id;
	}
	/** 
	 * Get class (can be null, get all classes) 
	 * @return <br>
	 * 0 - CLASES.ID<br>
	 * 1 - NIVELES.REFERENCIA<br>
	 * 2 - CLASES.REFERENCIA<br>
	 * 3 - TITULOS.REFERENCIA<br>
	 * 4 - ENSENANZAS.REFERENCIA<br>
	 * 5 - PROFESORES.ID<br>
	 * 6 - PROFESORES.NOMBRE<br>
	 * 7 - PROFESORES.APELLIDOS<br>
	 */
	public Vector<Vector<String>> consultarClase(String idNivel,String idClase){
		Vector<Vector<String>> tabla;
		String sql="SELECT CLASES.ID,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA,ENSENANZAS.REFERENCIA" +
				",PROFESORES.ID,PROFESORES.NOMBRE,PROFESORES.APELLIDOS" +
				" FROM CLASES,NIVELES,TITULOS,ENSENANZAS,PROFESORES" +
				" WHERE CLASES.TUTOR=PROFESORES.ID AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID AND TITULOS.ENSENANZA=ENSENANZAS.ID"+
				((idNivel!=null)?" AND NIVELES.ID="+idNivel:"")+
				((idClase!=null)?" AND CLASES.ID="+idClase:"")+
				" ORDER BY TITULOS.REFERENCIA,NIVELES.REFERENCIA,CLASES.REFERENCIA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+idNivel+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get class for subject (can't be null) 
	 * @return assessable identifier<br>
	 * 0 - CLASES.ID<br>
	 * 1 - NIVELES.REFERENCIA<br>
	 * 2 - CLASES.REFERENCIA<br>
	 * 3 - TITULOS.REFERENCIA<br>
	 * 4 - ENSENANZAS.REFERENCIA<br>
	 * 5 - PROFESORES.ID<br>
	 * 6 - PROFESORES.NOMBRE<br>
	 * 7 - PROFESORES.APELLIDOS<br>
	 */
	public Vector<Vector<String>> consultarClasesParaMateria(String idMateria){
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null) {
			if (!idMateria.equals("")) {
			String sql="SELECT CLASES.ID,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA,ENSENANZAS.REFERENCIA" +
					",PROFESORES.ID,PROFESORES.NOMBRE,PROFESORES.APELLIDOS" +
					" FROM MATERIAS,MATERIA_CLASE,CLASES,PROFESORES,NIVELES,TITULOS,ENSENANZAS" +
					" WHERE MATERIA_CLASE.MATERIA=MATERIAS.ID AND MATERIA_CLASE.CLASE=CLASES.ID" +
					" AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID AND TITULOS.ENSENANZA=ENSENANZAS.ID" +
					" AND CLASES.TUTOR=PROFESORES.ID AND MATERIAS.ID="+idMateria+
					" ORDER BY MATERIAS.REFERENCIA;";
			tabla=this.sqlConsulta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get subject-class sort by TITULOS.REFERENCIA,NIVELES.REFERENCIA,CLASES.REFERENCIA,MATERIAS.REFERENCIA
	 * @return<br>
	 * 0 - MATERIA_CLASE.ID<br>
	 * 1 - NIVELES.REFERENCIA<br>
	 * 2 - CLASES.REFERENCIA<br>
	 * 3 - TITULOS.REFERENCIA<br>
	 * 4 - MATERIAS.REFERENCIA<br>
	 * 5 - MATERIAS.DESCRIPCION<br>
	 */
	public Vector<Vector<String>> consultarMateriaClase(){
		Vector<Vector<String>> tabla;
		String sql="SELECT MATERIA_CLASE.ID,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA,MATERIAS.REFERENCIA," +
				" MATERIAS.DESCRIPCION FROM MATERIAS,MATERIA_CLASE,CLASES,NIVELES,TITULOS" +
				" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND MATERIA_CLASE.CLASE=CLASES.ID" +
				" AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID"+
				" ORDER BY TITULOS.REFERENCIA,NIVELES.REFERENCIA,CLASES.REFERENCIA,MATERIAS.REFERENCIA;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Integer> consultarDiasSemanaMateriaClase(String idMateriaClase){
		Vector<Integer> dias = new Vector<Integer>();
		Vector<Vector<String>> tabla;
		String sql="SELECT DISTINCT DIA_SEMANA FROM UH_SEMANA WHERE MATERIACLASE="+idMateriaClase+";";
		tabla=this.sqlConsulta(sql);
		if (tabla!=null) if (tabla.size()>0){
			for (int i=0;i<tabla.size();i++) {
				dias.add(Integer.parseInt(tabla.get(i).get(0)));
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+") "+((dias!=null)?dias.size():"null")+" days");
		return dias;
	}
	public Calendar consultarHoraSiguienteParaFechaMateriaClase(Calendar fecha,String idMateriaClase){
		Calendar resultado = null;
		Vector<Vector<String>> tabla=null;
		if (fecha!=null) {
			String sql="SELECT HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO) FROM UH_SEMANA" +
			" WHERE UH_SEMANA.DIA_SEMANA="+ fecha.get(Calendar.DAY_OF_WEEK) +
			" AND UH_SEMANA.INICIO>'"+Fecha.bbddSoloHora(fecha)+"'"+
			((idMateriaClase==null)?"":" AND UH_SEMANA.MATERIACLASE="+idMateriaClase) +
			" ORDER BY UH_SEMANA.INICIO;";
			tabla=this.sqlConsulta(sql);
			if (tabla!=null)
				if (tabla.size()>0)
					if (tabla.get(0)!=null)
						if (tabla.get(0).size()>0) {
							resultado = (Calendar) fecha.clone();
							resultado.set(Calendar.HOUR_OF_DAY, Integer.parseInt(tabla.get(0).get(0)));
							resultado.set(Calendar.MINUTE, Integer.parseInt(tabla.get(0).get(1)));
						}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+","+idMateriaClase+") "+((tabla!=null)?tabla.size():"null")+" records => "+((resultado==null)?"null":Fecha.formaFechaDiaHora(resultado)));
		return resultado;
	}
	/** 
	 * Get class for subject (can't be null) 
	 * @return assessable identifier<br>
	 * 0 - MATERIA_CLASE.ID<br>
	 * 1 - MATERIA_CLASE.MATERIA<br>
	 * 2 - MATERIA_CLASE.CLASE<br>
	 * 3 - NIVELES.ID<br>
	 * 4 - TITULOS.ID<br>
	 * 5 - ENSENANZAS.ID<br>
	 */
	public Vector<String> consultarIdMateriaClase(String idMateria,String idClase){
		String sql=null;
		Vector<String> registro=null;
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null && idClase!=null) {
			sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
				" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
				" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
				" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
				" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
				" AND MATERIA_CLASE.MATERIA="+idMateria+
				" AND MATERIA_CLASE.CLASE="+idClase+";";
			tabla=this.sqlConsulta(sql);
			if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records => "+((registro!=null)?registro.size():"null")+" datas");
		return registro;
	}
	/** 
	 * Get subject-class for subject and/or class (can't be null) 
	 * @return subject-class identifier<br>
	 * 0 - MATERIA_CLASE.ID<br>
	 * 1 - MATERIA_CLASE.MATERIA<br>
	 * 2 - MATERIA_CLASE.CLASE<br>
	 * 3 - NIVELES.ID<br>
	 * 4 - TITULOS.ID<br>
	 * 5 - ENSENANZAS.ID<br>
	 */
	public Vector<Vector<String>> consultarIdMateriaClases(String idMateria,String idClase){
		String sql=null;
		Vector<Vector<String>> tabla=null;
		sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
			" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
			" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
			" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
			" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
			((idMateria!=null)?" AND MATERIA_CLASE.MATERIA="+idMateria:"")+
			((idClase!=null)?" AND MATERIA_CLASE.CLASE="+idClase:"")+";";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records => "+((tabla!=null)?tabla.size():"null")+" datas");
		return tabla;
	}
	public Vector<String> consultarIdMateriaClaseParaMateria(String idMateria,String idClase){
		String sql=null;
		Vector<String> registro=null;
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null){
			if (idClase!=null) {
				sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
						" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
						" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
						" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
						" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
						" AND MATERIA_CLASE.MATERIA="+idMateria+
						" AND MATERIA_CLASE.CLASE="+idClase+";";
				tabla=this.sqlConsulta(sql);
				if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
				if (registro==null) {
					sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
							" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
							" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
							" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
							" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
							" AND MATERIA_CLASE.MATERIA="+idMateria+";";
					tabla=this.sqlConsulta(sql);
					if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
					if (registro==null) {
						sql="SELECT NULL,MATERIAS.ID,NULL" +
								" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID" +
								" FROM MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
								" WHERE NIVELES.ID=MATERIAS.NIVEL" +
								" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
								" AND MATERIAS.ID="+idMateria+";";
						tabla=this.sqlConsulta(sql);
						if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
					}
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records => "+((registro!=null)?registro.size():"null")+" datas");
		return registro;
	}
	public Vector<String> consultarIdMateriaClaseParaClase(String idMateria,String idClase){
		String sql=null;
		Vector<String> registro=null;
		Vector<Vector<String>> tabla=null;
		if (idClase!=null) {
			if (idMateria!=null) {
				sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
					" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
					" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
					" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
					" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
					" AND MATERIA_CLASE.MATERIA="+idMateria+
					" AND MATERIA_CLASE.CLASE="+idClase+";";
				tabla=this.sqlConsulta(sql);
				if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
				if (registro==null) {
					sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
							" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
							" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
							" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
							" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
							" AND CLASE.ID="+idClase+";";
					tabla=this.sqlConsulta(sql);
					if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
					if (registro==null) {
						sql="SELECT NULL,NULL,CLASES.ID" +
							" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
							" FROM CLASES,NIVELES,TITULOS,ENSENANZAS" +
							" WHERE NIVELES.ID=CLASES.NIVEL" +
							" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
							" AND CLASES.ID="+idClase+";";
					}
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records => "+((registro!=null)?registro.size():"null")+" datas");
		return registro;
	}
	public Vector<String> consultarIdMateriaClase(String idMateriaClase){
		Vector<String> registro=null;
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null) {
			String sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
					" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
					" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
					" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
					" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
					" AND MATERIA_CLASE.ID="+idMateriaClase+";";
			tabla=this.sqlConsulta(sql);
			if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records => "+((registro!=null)?registro.size():"null")+" datas");
		return registro;
	}
	public String consultarIdMateriaClasePrimera(){
		String resultado=null;
		Vector<Vector<String>> tabla;
		String sql="SELECT ID FROM MATERIA_CLASE LIMIT 1;";
		tabla=this.sqlConsulta(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public String consultarIdMateriaClaseParaFechaHora(Calendar c){
		String resultado=null;
		Vector<Vector<String>> tabla;
		String sql="SELECT MATERIACLASE FROM UH_SEMANA" +
		" WHERE UH_SEMANA.DIA_SEMANA="+ c.get(Calendar.DAY_OF_WEEK) + " AND"+
		" UH_SEMANA.INICIO<='"+String.format("%02d",c.get(Calendar.HOUR_OF_DAY))+":"+String.format("%02d",c.get(Calendar.MINUTE))+":00' AND"+
		" UH_SEMANA.FIN>'"+String.format("%02d",c.get(Calendar.HOUR_OF_DAY))+":"+String.format("%02d",c.get(Calendar.MINUTE))+":00';";
		tabla=this.sqlConsulta(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(c)+") "+((tabla!=null)?tabla.size():"null")+" records => "+resultado);
		return resultado;
	}
	public Vector<String> consultarDiaNoLectivo(String idNoLectivo){
		Vector<String> dia=null;
		Vector<Vector<String>> tabla=null;
		if (idNoLectivo!=null) if (!idNoLectivo.equals("")) {
			String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA), MONTH(FECHA), YEAR(FECHA), DESCRIPCION FROM DIASNOLECTIVOS WHERE ID='"+idNoLectivo+"';";
			tabla=this.sqlConsulta(sql);
			if (tabla!=null) if (tabla.size()>0) if (tabla.get(0)!=null) if (tabla.get(0).size()>0) dia=tabla.get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+") "+((dia!=null)?Fecha.formaFecha(dia.get(2), dia.get(3), dia.get(4))+": "+dia.get(5):"null"));
		return dia;
	}
	public Vector<Vector<String>> consultarDiaNoLectivo(Calendar fecha){
		Vector<Vector<String>> tabla=null;
		if (fecha!=null) {
			String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA), MONTH(FECHA), YEAR(FECHA),DESCRIPCION FROM DIASNOLECTIVOS "+
				" WHERE DAYOFMONTH(FECHA)="+fecha.get(Calendar.DAY_OF_MONTH)+" AND"+
				" MONTH(FECHA)="+(fecha.get(Calendar.MONTH)+1)+" AND"+
				" YEAR(FECHA)="+fecha.get(Calendar.YEAR)+
				" ORDER BY CLASE;";
			tabla=this.sqlConsulta(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public void modificarDiaNoLectivoFecha(String idNoLectivo, Calendar fecha){
		int ejecucion=-1;
		if (idNoLectivo!=null && fecha!=null) {
			if (!idNoLectivo.equals("")) {
				String sql="UPDATE DIASNOLECTIVOS SET DIASNOLECTIVOS.FECHA='"+Fecha.bbddSoloDia(fecha)+"'"+
				" WHERE DIASNOLECTIVOS.ID="+idNoLectivo+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+","+Fecha.formaFechaDiaHora(fecha)+") records modified "+ejecucion);
	}
	public void modificarDiaNoLectivoClase(String idNoLectivo, String clase){
		int ejecucion=-1;
		if (idNoLectivo!=null) {
			if (!idNoLectivo.equals("")) {
				String sql="UPDATE DIASNOLECTIVOS SET DIASNOLECTIVOS.CLASE="+((clase==null)?"NULL":clase)+
				" WHERE DIASNOLECTIVOS.ID="+idNoLectivo+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+","+clase+") records modified "+ejecucion);
	}
	public void borrarDiaNoLectivo(String idNoLectivo){
		int ejecucion=-1;
		if (idNoLectivo!=null) if (!idNoLectivo.equals("")) {
				String sql="DELETE FROM DIASNOLECTIVOS WHERE ID="+idNoLectivo+";";
				ejecucion=this.sqlEjecuta(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+") deleted "+ejecucion);
	}
	public String insertarDiaNoLectivo(Calendar fecha,String idClase,String descripcion){
		String id=null;
		if (fecha!=null && descripcion!=null) {
			String sql="INSERT INTO DIASNOLECTIVOS (CLASE,FECHA,DESCRIPCION)"+
			" VALUES ("+((idClase==null)?"NULL":idClase)+",'"+Fecha.bbddSoloDia(fecha)+"','"+descripcion+"');";
			id = this.sqlInsertarId("DIASNOLECTIVOS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+","+idClase+","+descripcion+") id inserted "+id);
		return id;
	}
	public Vector<Vector<String>> consultarDiasNoLectivosDiaCompletoYClase(String idClase){
		Vector<Vector<String>> tabla; 
		String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA)," +
				" MONTH(FECHA), YEAR(FECHA)," +
				" DESCRIPCION FROM DIASNOLECTIVOS" +
				" WHERE CLASE IS NULL"+((idClase!=null)?" OR CLASE="+idClase:"")+
				" ORDER BY FECHA ASC;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public Vector<Vector<String>> consultarDiasNoLectivosClase(String idClase){
		Vector<Vector<String>> tabla; 
		String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA)," +
				" MONTH(FECHA), YEAR(FECHA)," +
				" DESCRIPCION FROM DIASNOLECTIVOS" +
				" WHERE "+((idClase!=null)?"CLASE="+idClase:"CLASE IS NOT NULL")+
				" ORDER BY FECHA ASC;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	/** 
	 * Get no school days  
	 * @return <br>
	 * 0 - ID<br>
	 * 1 - CLASE<br>
	 * 2 - DAYOFMONTH(FECHA)<br>
	 * 3 - MONTH(FECHA)<br>
	 * 4 - YEAR(FECHA)<br>
	 * 5 - DESCRIPCION<br>
	 */
	public Vector<Vector<String>> consultarDiasNoLectivos(){
		Vector<Vector<String>> tabla; 
		String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA)," +
				" MONTH(FECHA), YEAR(FECHA)," +
				" DESCRIPCION FROM DIASNOLECTIVOS" +
				" ORDER BY FECHA ASC;";
		tabla=this.sqlConsulta(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" records");
		return tabla;
	}
	public static void imprimir(Vector<Vector<String>> datos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((datos!=null)?datos.size():null)+")");
		String s=null;
		if (datos!=null) {
			if (datos.size()>0) {
				for (int i=0;i<datos.size();i++){
					s=i+":[ ";
					for (int j=0;j<datos.get(i).size();j++){
						s=s+"("+j+")=\""+datos.get(i).get(j)+((j<datos.get(i).size()-1)?"\" | ":" ");
					}
					PCuaderno.MiPCuaderno.notificar(s+"]");
				}
			} else PCuaderno.MiPCuaderno.notificar("Vacio");
		} else PCuaderno.MiPCuaderno.notificar("Nulo");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("fin");
	}
	public static void imprimirRegistro(Vector<String> datos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((datos!=null)?datos.size():null)+")");
		String s=null;
		if (datos!=null) {
			if (datos.size()>0) {
				s="[ ";
				for (int j=0;j<datos.size();j++)
					s=s+"("+j+")=\""+datos.get(j)+((j<datos.size()-1)?"\" | ":"\" ");
				PCuaderno.MiPCuaderno.notificar(s+"]");
			} else PCuaderno.MiPCuaderno.notificar("Vacio");
		} else PCuaderno.MiPCuaderno.notificar("Nulo");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("fin");
	}
	public static void imprimirRegistroNumero(Vector<Integer> datos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((datos!=null)?datos.size():null)+")");
		String s=null;
		if (datos!=null) {
			if (datos.size()>0) {
				s="[ ";
				for (int j=0;j<datos.size();j++)
					s=s+"("+j+")="+datos.get(j)+((j<datos.size()-1)?" | ":" ");
				PCuaderno.MiPCuaderno.notificar(s+"]");
			} else PCuaderno.MiPCuaderno.notificar("Vacio");
		} else PCuaderno.MiPCuaderno.notificar("Nulo");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("fin");
	}
	public Vector<Vector<String>> sqlConsulta(String sql){
		PCuaderno.MiPCuaderno.resetTimerCloseBBDD();
		Statement sentencia = null;ResultSet resultado = null;
		Vector<Vector<String>> salida = new Vector<Vector<String>>();
		Vector<String> fila=null;
		if (this.conexion==null) this.conexionAbrir();
		if (this.conexion!=null) {
			//System.out.println("ConsultaSQL = "+sql);
			try {
				sentencia = this.conexion.createStatement();
				this.conexion.setAutoCommit(true);
				resultado = sentencia.executeQuery(sql);
				while (resultado.next()) {
					//System.out.print("   ");
					fila = new Vector<String>();
					for(int i=1;i<=resultado.getMetaData().getColumnCount();i++){
						fila.addElement(resultado.getString(i));
						//System.out.print("|"+resultado.getString(i));
					}
					salida.add(fila);
					fila=null;
					//System.out.println("|");
				}
			} catch (SQLException e) {System.out.println("this.consultaSQL Error sql: "+sql);e.printStackTrace();}
		}
		return salida;
	}
	public Vector<String> sqlConsulta1Columna(String sql){
		PCuaderno.MiPCuaderno.resetTimerCloseBBDD();
		Statement sentencia = null;ResultSet resultado = null;
		Vector<String> salida = new Vector<String>();
		if (this.conexion==null) this.conexionAbrir();
		if (this.conexion!=null) {
			//System.out.println("ConsultaSQL = "+sql);
			try {
				sentencia = this.conexion.createStatement();
				this.conexion.setAutoCommit(true);
				resultado = sentencia.executeQuery(sql);
				while (resultado.next()) {
					salida.addElement(resultado.getString(1));
					//System.out.println("|"+resultado.getString(1));
				}
			} catch (SQLException e) {System.out.println("this.consulta1ColumnaSQL Error sql: "+sql);e.printStackTrace();}
		}
		return salida;
	}
	public boolean conexionAbrir(){
		boolean resultado=false;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("openning");
		this.conexion = null;
		try {this.conexion = DriverManager.getConnection(this.cadenaConexion);resultado=true;
		} catch (SQLException e) {e.printStackTrace();}
		return resultado;
	}
	public boolean conexionCerrar(){
		boolean resultado=false;
		if (this.conexion!=null) {
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("closing");
			try {
				this.conexion.setAutoCommit(true);
				Statement sentencia = this.conexion.createStatement();
				sentencia.execute("SHUTDOWN;");
			} catch (SQLException e) {System.out.println("this.cerrarConexion Error stop database.");e.printStackTrace();}
			try {this.conexion.close();this.conexion=null;resultado=true;} catch (SQLException e) {
				System.out.println("this.cerrarConexion - Can't close database.");
				e.printStackTrace();
			}
		}
		return resultado;
	}
	public String sqlInsertarId(String tabla,String sql){
		PCuaderno.MiPCuaderno.resetTimerCloseBBDD();
		String idUltimo = null;
		if (this.conexion==null) this.conexionAbrir();
		if (this.conexion!=null) {
			Statement sentencia = null;
			ResultSet resultado = null;
			try {
				this.conexion.setAutoCommit(false);
				sentencia = this.conexion.createStatement();
				sentencia.execute("SET AUTOCOMMIT FALSE;");
				sentencia.execute("LOCK TABLE "+tabla+" WRITE;");
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" sql = "+sql);
				sentencia.execute(sql);
				resultado = sentencia.executeQuery("CALL IDENTITY();");
				if (resultado.next()) {
					idUltimo=resultado.getString(1);
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Last id = "+idUltimo);
				}
				sentencia.execute("COMMIT;SET AUTOCOMMIT TRUE;");
				this.conexion.commit();
				this.conexion.setAutoCommit(true);
			} catch (SQLException e) {
				try {sentencia.execute("ROLLBACK;");this.conexion.setAutoCommit(true);} catch (SQLException e1) {e1.printStackTrace();}
				System.out.println("this.insertarIdSQL Error SQL:"+sql);e.printStackTrace();}
		}
		return idUltimo;
	}
	public int sqlEjecuta(String sql){
		PCuaderno.MiPCuaderno.resetTimerCloseBBDD();
		int cambios=0;
		if (sql!=null) if (!sql.equals("")) {
			if (this.conexion==null) this.conexionAbrir();
			if (this.conexion!=null) {
				Statement sentencia = null;
				try {
					sentencia = this.conexion.createStatement();
					sentencia.execute("SET AUTOCOMMIT FALSE;"); //como una transacción si hay varias sentencias
					sentencia.execute("SET WRITE_DELAY 0;");
					
					String[] sentencias=sql.split(";");
					cambios=sentencias.length;
					for (int i=0;i<cambios;i++) {
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("sql ("+i+"/"+sentencias.length+"):["+sentencias[i]+";] => ");
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("sql ("+i+"/"+sentencias.length+"): "+sentencia.executeUpdate(sentencias[i]+";")+" records");
						else sentencia.executeUpdate(sentencias[i]+";");
					}
					//if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("sql: "+sql);
					//cambios=sentencia.executeUpdate(sql);
					
					this.conexion.commit();
					sentencia.execute("COMMIT;");
				} catch (SQLException e) {e.printStackTrace();}
			}
		} else System.out.println("this.ejecutaSQL - Sentence sql empty.");
		return cambios;
	}
	public int sqlEjecutaVarias(Vector<String> sql){
		int insertados=0;
		String sqlSentencia="";
		if (this.conexion==null) this.conexionAbrir();
		if (this.conexion!=null) {
			Statement sentencia = null;
			try {
				sentencia = this.conexion.createStatement();
				for (int i=0;i<sql.size();i++) {
					PCuaderno.MiPCuaderno.resetTimerCloseBBDD();
					//System.out.println("ejecutaVariasSQL sql("+i+"):"+sql.get(i));
					sqlSentencia=sql.get(i);
					if (sentencia.execute(sqlSentencia)) insertados++;
					//sentencia.execute("COMMIT;");
				}
				this.conexion.commit();
				sentencia.execute("COMMIT;");
			} catch (SQLException e) {System.out.println("this.ejecutaVariasSQL Error sql:"+sqlSentencia);e.printStackTrace();}
		}
		return insertados;
	}
	public void bbddCrear(){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Making tables...");
		Vector<String> sentencias = new Vector<String>();
		
		// SUSTITUIR POR bbdd.java
		sentencias.addElement("CREATE TABLE PROFESORES(ID INT NOT NULL IDENTITY, NOMBRE VARCHAR(256) NOT NULL, APELLIDOS VARCHAR(256) NOT NULL, DATOS VARCHAR(256));");
		sentencias.addElement("CREATE TABLE USUARIO(PROFESOR INT NOT NULL);");
		sentencias.addElement("CREATE TABLE ENSENANZAS(ID INT NOT NULL IDENTITY, REFERENCIA VARCHAR(5) NOT NULL, DESCRIPCION VARCHAR(1024) NOT NULL);");
		sentencias.addElement("CREATE TABLE TITULOS(ID INT NOT NULL IDENTITY, ENSENANZA INT NOT NULL, REFERENCIA VARCHAR(5) NOT NULL, DESCRIPCION VARCHAR(1024) NOT NULL);");
		sentencias.addElement("CREATE TABLE NIVELES(ID INT NOT NULL IDENTITY, TITULO INT NOT NULL, REFERENCIA VARCHAR(3) NOT NULL);");
		sentencias.addElement("CREATE TABLE MATERIAS(ID INT NOT NULL IDENTITY, REFERENCIA VARCHAR(5) NOT NULL, DESCRIPCION VARCHAR(256) NOT NULL, NIVEL INT);");
		sentencias.addElement("CREATE TABLE CLASES(ID INT NOT NULL IDENTITY, NIVEL INT, REFERENCIA VARCHAR(5) NOT NULL, TUTOR INT NOT NULL);");
		sentencias.addElement("CREATE TABLE MATERIA_CLASE(ID INT NOT NULL IDENTITY, MATERIA INT NOT NULL, CLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE UH_SEMANA(ID INT NOT NULL IDENTITY, MATERIACLASE INT NOT NULL, FIN TIME NOT NULL, INICIO TIME NOT NULL, DIA_SEMANA INT NOT NULL);");
		sentencias.addElement("CREATE TABLE ESTUDIANTES(ID INT NOT NULL IDENTITY, NOMBRE VARCHAR(256) NOT NULL, APELLIDOS VARCHAR(256) NOT NULL, DATOS VARCHAR(1024), CLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE ESTUDIANTE_MATERIACLASE(ESTUDIANTE INT NOT NULL, MATERIACLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE TIPOS_EVALUABLE(ID INT NOT NULL IDENTITY, REFERENCIA VARCHAR(3) NOT NULL, DESCRIPCION VARCHAR(256), PESO INT NOT NULL, MATERIA INT, MINIMO DECIMAL(3,1) DEFAULT 0 NOT NULL);");
		sentencias.addElement("CREATE TABLE EVALUABLES(ID INT NOT NULL IDENTITY, DIA_FIN DATE NOT NULL, UH_FIN INT NOT NULL, REFERENCIA VARCHAR(8) NOT NULL, ENUNCIADO VARCHAR(2048) NOT NULL, PESO INT NOT NULL, TIPO INT NOT NULL, RECUPERA INT DEFAULT NULL);");
		sentencias.addElement("CREATE TABLE EVALUACIONES_EVALUABLES(ID INT NOT NULL IDENTITY, EVALUABLE INT NOT NULL, NOTA DECIMAL(3,1) DEFAULT 0 NOT NULL, OBSERVACIONES VARCHAR(2048), GRUPO VARCHAR(10));");
		sentencias.addElement("CREATE TABLE EE_ESTUDIANTES(E_EVALUABLE INT NOT NULL, ESTUDIANTE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE TIPOS_EVENTO(ID INT NOT NULL, DESCRIPCION VARCHAR(64) NOT NULL, COLOR INT NOT NULL, BBDD VARCHAR(255));");
		sentencias.addElement("CREATE TABLE EVENTOS(ID INT NOT NULL IDENTITY, DIA DATE NOT NULL, HORA TIME NOT NULL, SUMARIO VARCHAR(2048), PREPARATIVOS VARCHAR(2048), CONCLUSION VARCHAR(2048), TIPO INT NOT NULL);");
		sentencias.addElement("CREATE TABLE CITAS_TUTORES(ID INT NOT NULL, RELACION VARCHAR(256));");
		sentencias.addElement("CREATE TABLE ESTUDIANTES_CITATUTORES(ESTUDIANTE INT NOT NULL, EVENTO_CITATUTORES INT NOT NULL);");
		sentencias.addElement("CREATE TABLE EVALUACIONES(ID INT NOT NULL, DESCRIPCION VARCHAR(256) NOT NULL, CLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE CLAUSTROS(ID INT NOT NULL, ORDINARIO BOOLEAN);");
		sentencias.addElement("CREATE TABLE ETCPS(ID INT NOT NULL);");
		sentencias.addElement("CREATE TABLE C_ESCOLARES(ID INT NOT NULL, ORDINARIO BOOLEAN);");
		sentencias.addElement("CREATE TABLE OTROS_EVENTOS(ID INT NOT NULL, DESCRIPCION VARCHAR(256));");
		sentencias.addElement("CREATE TABLE DIASNOLECTIVOS(ID INT NOT NULL IDENTITY, CLASE INT, FECHA DATE NOT NULL, DESCRIPCION VARCHAR(256));");
		sentencias.addElement("CREATE TABLE PROGRAMACION_NODO(ID INT NOT NULL IDENTITY, PADRE INT, POSICION INT NOT NULL, MATERIA INT, ENUNCIADO VARCHAR(256) NOT NULL, SUMARIO VARCHAR(10240));");
		sentencias.addElement("CREATE TABLE PROGRAMACION_TEMPORAL(NODO INT NOT NULL, CLASE INT NOT NULL, DURACION INTEGER NOT NULL);");
		sentencias.addElement("CREATE TABLE PARCIALES(ID INT NOT NULL IDENTITY, MATERIA INT NOT NULL, INICIO DATE NOT NULL, FIN DATE NOT NULL);");
		sentencias.addElement("CREATE TABLE CONFIGURACION(ID INT IDENTITY, MESINICIOCURSO INT, MESESCURSO INT, AGNOINICIOCURSO INT);");
		sentencias.addElement("CREATE TABLE EVALUABLES_NODOS(EVALUABLE INT NOT NULL, NODO INT NOT NULL);");
		sentencias.addElement("ALTER TABLE USUARIO ADD CONSTRAINT IDX_USUARIO_PK PRIMARY KEY (PROFESOR);");
		sentencias.addElement("ALTER TABLE ESTUDIANTE_MATERIACLASE ADD CONSTRAINT IDX_ESTUDIANTE_MATERIACLASE_PK PRIMARY KEY (ESTUDIANTE, MATERIACLASE);");
		sentencias.addElement("ALTER TABLE EE_ESTUDIANTES ADD CONSTRAINT IDX_EE_ESTUDIANTES_PK PRIMARY KEY (E_EVALUABLE, ESTUDIANTE);");
		sentencias.addElement("ALTER TABLE TIPOS_EVENTO ADD CONSTRAINT IDX_TIPOS_EVENTO_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE CITAS_TUTORES ADD CONSTRAINT IDX_CITAS_TUTORES_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES_CITATUTORES ADD CONSTRAINT IDX_ESTUDIANTES_CITATUTORES_PK PRIMARY KEY (ESTUDIANTE, EVENTO_CITATUTORES);");
		sentencias.addElement("ALTER TABLE CLAUSTROS ADD CONSTRAINT IDX_CLAUSTROS_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE ETCPS ADD CONSTRAINT IDX_ETCPS_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE C_ESCOLARES ADD CONSTRAINT IDX_C_ESCOLARES_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE OTROS_EVENTOS ADD CONSTRAINT IDX_OTROS_EVENTOS_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_TEMPORAL ADD CONSTRAINT IDX_PROGRAMACION_TEMPORAL_PK PRIMARY KEY (NODO, CLASE);");
		sentencias.addElement("ALTER TABLE EVALUABLES_NODOS ADD CONSTRAINT IDX_EVALUABLES_NODOS_PK PRIMARY KEY (EVALUABLE, NODO);");
		sentencias.addElement("ALTER TABLE USUARIO ADD CONSTRAINT IDX_USUARIO_FK0 FOREIGN KEY (PROFESOR) REFERENCES PROFESORES (ID);");
		sentencias.addElement("ALTER TABLE TITULOS ADD CONSTRAINT IDX_TITULOS_FK0 FOREIGN KEY (ENSENANZA) REFERENCES ENSENANZAS (ID);");
		sentencias.addElement("ALTER TABLE NIVELES ADD CONSTRAINT IDX_NIVELES_FK0 FOREIGN KEY (TITULO) REFERENCES TITULOS (ID);");
		sentencias.addElement("ALTER TABLE MATERIAS ADD CONSTRAINT IDX_MATERIAS_FK0 FOREIGN KEY (NIVEL) REFERENCES NIVELES (ID);");
		sentencias.addElement("ALTER TABLE CLASES ADD CONSTRAINT IDX_CLASES_FK0 FOREIGN KEY (TUTOR) REFERENCES PROFESORES (ID);");
		sentencias.addElement("ALTER TABLE CLASES ADD CONSTRAINT IDX_CLASES_FK1 FOREIGN KEY (NIVEL) REFERENCES NIVELES (ID);");
		sentencias.addElement("ALTER TABLE MATERIA_CLASE ADD CONSTRAINT IDX_MATERIA_CLASE_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		sentencias.addElement("ALTER TABLE MATERIA_CLASE ADD CONSTRAINT IDX_MATERIA_CLASE_FK1 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE UH_SEMANA ADD CONSTRAINT IDX_UH_SEMANA_FK0 FOREIGN KEY (MATERIACLASE) REFERENCES MATERIA_CLASE (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES ADD CONSTRAINT IDX_ESTUDIANTES_FK0 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTE_MATERIACLASE ADD CONSTRAINT IDX_ESTUDIANTE_MATERIACLASE_FK0 FOREIGN KEY (ESTUDIANTE) REFERENCES ESTUDIANTES (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTE_MATERIACLASE ADD CONSTRAINT IDX_ESTUDIANTE_MATERIACLASE_FK1 FOREIGN KEY (MATERIACLASE) REFERENCES MATERIA_CLASE (ID);");
		sentencias.addElement("ALTER TABLE TIPOS_EVALUABLE ADD CONSTRAINT IDX_TIPOS_EVALUABLE_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		sentencias.addElement("ALTER TABLE EVALUABLES ADD CONSTRAINT IDX_EVALUABLES_FK0 FOREIGN KEY (TIPO) REFERENCES TIPOS_EVALUABLE (ID);");
		sentencias.addElement("ALTER TABLE EVALUABLES ADD CONSTRAINT IDX_EVALUABLES_FK1 FOREIGN KEY (UH_FIN) REFERENCES UH_SEMANA (ID);");
		sentencias.addElement("ALTER TABLE EVALUACIONES_EVALUABLES ADD CONSTRAINT IDX_EVALUACIONES_EVALUABLES_FK0 FOREIGN KEY (EVALUABLE) REFERENCES EVALUABLES (ID);");
		sentencias.addElement("ALTER TABLE EE_ESTUDIANTES ADD CONSTRAINT IDX_EE_ESTUDIANTES_FK0 FOREIGN KEY (E_EVALUABLE) REFERENCES EVALUACIONES_EVALUABLES (ID);");
		sentencias.addElement("ALTER TABLE EE_ESTUDIANTES ADD CONSTRAINT IDX_EE_ESTUDIANTES_FK1 FOREIGN KEY (ESTUDIANTE) REFERENCES ESTUDIANTES (ID);");
		sentencias.addElement("ALTER TABLE EVENTOS ADD CONSTRAINT IDX_EVENTOS_FK0 FOREIGN KEY (TIPO) REFERENCES TIPOS_EVENTO (ID);");
		sentencias.addElement("ALTER TABLE CITAS_TUTORES ADD CONSTRAINT IDX_CITAS_TUTORES_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES_CITATUTORES ADD CONSTRAINT IDX_ESTUDIANTES_CITATUTORES_FK0 FOREIGN KEY (ESTUDIANTE) REFERENCES ESTUDIANTES (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES_CITATUTORES ADD CONSTRAINT IDX_ESTUDIANTES_CITATUTORES_FK1 FOREIGN KEY (EVENTO_CITATUTORES) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE EVALUACIONES ADD CONSTRAINT IDX_EVALUACIONES_FK0 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE EVALUACIONES ADD CONSTRAINT IDX_EVALUACIONES_FK1 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE CLAUSTROS ADD CONSTRAINT IDX_CLAUSTROS_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE ETCPS ADD CONSTRAINT IDX_ETCPS_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE C_ESCOLARES ADD CONSTRAINT IDX_C_ESCOLARES_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE OTROS_EVENTOS ADD CONSTRAINT IDX_OTROS_EVENTOS_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE DIASNOLECTIVOS ADD CONSTRAINT IDX_DIASNOLECTIVOS_FK0 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_NODO ADD CONSTRAINT IDX_PROGRAMACION_NODO_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_TEMPORAL ADD CONSTRAINT IDX_PROGRAMACION_TEMPORAL_FK0 FOREIGN KEY (NODO) REFERENCES PROGRAMACION_NODO (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_TEMPORAL ADD CONSTRAINT IDX_PROGRAMACION_TEMPORAL_FK1 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE PARCIALES ADD CONSTRAINT IDX_PARCIALES_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		sentencias.addElement("ALTER TABLE EVALUABLES_NODOS ADD CONSTRAINT IDX_EVALUABLES_NODOS_FK0 FOREIGN KEY (EVALUABLE) REFERENCES EVALUABLES (ID);");
		sentencias.addElement("ALTER TABLE EVALUABLES_NODOS ADD CONSTRAINT IDX_EVALUABLES_NODOS_FK1 FOREIGN KEY (NODO) REFERENCES PROGRAMACION_NODO (ID);");
		//HASTA AQUÍ
		
		System.out.println("Making database ... "+sentencias.size()+" sentences. ");
		sqlEjecutaVarias(sentencias);
		
		sentencias.clear();
		System.out.print("Insert variable values... ");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoCitaConTutores+",'CITAS_TUTORES','"+PCuaderno.MiPCuaderno.mensajes.getString("AppoimentTutors")+"', "+SWT.COLOR_RED+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoEvaluacion+",'EVALUACIONES','"+PCuaderno.MiPCuaderno.mensajes.getString("Evaluation")+"', "+SWT.COLOR_GREEN+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoClaustro+",'CLAUSTROS','"+PCuaderno.MiPCuaderno.mensajes.getString("MeetingTeachers")+"', "+SWT.COLOR_MAGENTA+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoETCP+",'ETCPS','"+PCuaderno.MiPCuaderno.mensajes.getString("MeetingCoordPed")+"', "+SWT.COLOR_DARK_GREEN+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoConsejoEscolar+",'C_ESCOLARES','"+PCuaderno.MiPCuaderno.mensajes.getString("SchoolCouncil")+"', "+SWT.COLOR_DARK_CYAN+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoOtroTipo+",'OTROS_EVENTOS','"+PCuaderno.MiPCuaderno.mensajes.getString("OtherTypeEvent")+"', "+SWT.COLOR_CYAN+");");

		System.out.println(sqlEjecutaVarias(sentencias)+"/"+sentencias.size());
	}
	public void bbddValoresIniciales(){
		Vector<String> sentencias = new Vector<String>();
		System.out.print("Insert initial values... ");
		// Solo crear el usuario ...
		sentencias.addElement("INSERT INTO PROFESORES (NOMBRE,APELLIDOS) VALUES ('"+PCuaderno.MiPCuaderno.mensajes.getString("Teacher")+"','...');");
		sentencias.addElement("INSERT INTO USUARIO (PROFESOR) VALUES (0);");
		System.out.println(sqlEjecutaVarias(sentencias)+"/"+sentencias.size());
	}
	public void configuracionValores(){
		System.out.print("Insert configuration values... ");
		Configuracion configuracion = new Configuracion(PCuaderno.MiPCuaderno.shellPrincipal);
		configuracion.open();
		String sentencia="INSERT INTO CONFIGURACION (MESINICIOCURSO,MESESCURSO,AGNOINICIOCURSO) VALUES ("+Fecha.MesInicioCurso+","+Fecha.MesesLectivosCurso+","+Fecha.AgnoInicioCurso+");";
		System.out.println(sqlEjecuta(sentencia));
	}

	public boolean bbddVersion_0_1(){
		String sql="SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='CONFIGURACION';";
		Vector<Vector<String>> res = this.sqlConsulta(sql);
		if (PCuaderno.noti) BBDD.imprimir(res);
		return (res.size()==0);
	}
	public boolean bbddVersion_0_2(){
		//String sql="SELECT TABLE_NAME,COLUMN_NAME FROM INFORMATION_SCHEMA.SYSTEM_COLUMNS WHERE TABLE_NAME='EVALUABLES' AND COLUMN_NAME LIKE 'DIA_INICIO';";
		String sql="SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='EVALUABLES_NODOS';";
		Vector<Vector<String>> res = this.sqlConsulta(sql);
		if (PCuaderno.noti) BBDD.imprimir(res);
		//this.imprimir(this.consultaSQL("SELECT TABLE_NAME,COLUMN_NAME FROM INFORMATION_SCHEMA.SYSTEM_COLUMNS WHERE TABLE_NAME='EVALUABLES';"));
		return (res.size()==0);
	}
	public boolean bbddVersion_0_4(){
		String sql="SELECT CHARACTER_MAXIMUM_LENGTH FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME='EVALUABLES' AND COLUMN_NAME='ENUNCIADO';";
		Vector<Vector<String>> res = this.sqlConsulta(sql);
		if (PCuaderno.noti) BBDD.imprimir(res);
		return (res.get(0).get(0).equals("512"));
	}
	public void configuracionActualizar(){
		System.out.println("Update configuration values... ");
		String sql=null;
		if (this.bbddVersion_0_1()) {
			sql="CREATE TABLE CONFIGURACION(ID INT IDENTITY, MESINICIOCURSO INT, MESESCURSO INT, AGNOINICIOCURSO INT);" +
				"ALTER TABLE TIPOS_EVALUABLE ADD MINIMO DECIMAL(3,1) DEFAULT 0 NOT NULL;"+
				"ALTER TABLE EVALUABLES ADD RECUPERA INT DEFAULT NULL;"+
				"ALTER TABLE EVALUABLES DROP COLUMN DIA_INICIO;" +
				"ALTER TABLE EVALUABLES DROP CONSTRAINT IDX_EVALUABLES_FK0;" +
				"ALTER TABLE EVALUABLES DROP COLUMN UH_INICIO;" +
				"CREATE TABLE EVALUABLES_NODOS(EVALUABLE INT NOT NULL, NODO INT NOT NULL);" +
				"ALTER TABLE EVALUABLES_NODOS ADD CONSTRAINT IDX_EVALUABLES_NODOS_FK0 FOREIGN KEY (EVALUABLE) REFERENCES EVALUABLES (ID);" +
				"ALTER TABLE EVALUABLES_NODOS ADD CONSTRAINT IDX_EVALUABLES_NODOS_FK1 FOREIGN KEY (NODO) REFERENCES PROGRAMACION_NODO (ID);"+
				"ALTER TABLE EVALUABLES ALTER COLUMN ENUNCIADO VARCHAR(2048);" +
				"ALTER TABLE EVALUACIONES_EVALUABLES ALTER COLUMN OBSERVACIONES VARCHAR(2048);"+
				"ALTER TABLE PROGRAMACION_NODO ALTER COLUMN SUMARIO VARCHAR(10240);"+
				"ALTER TABLE EVENTOS ALTER COLUMN PREPARATIVOS VARCHAR(2048);"+
				"ALTER TABLE EVENTOS ALTER COLUMN SUMARIO VARCHAR(2048);"+
				"ALTER TABLE EVENTOS ALTER COLUMN CONCLUSION VARCHAR(2048);";
			System.out.println("from version 0.1 ... "+sqlEjecuta(sql));
			this.configuracionValores();
		} else if (this.bbddVersion_0_2()) {
			sql="ALTER TABLE EVALUABLES DROP COLUMN DIA_INICIO;" +
				"ALTER TABLE EVALUABLES DROP CONSTRAINT IDX_EVALUABLES_FK0;" +
				"ALTER TABLE EVALUABLES DROP COLUMN UH_INICIO;" +
				"CREATE TABLE EVALUABLES_NODOS (EVALUABLE INT NOT NULL, NODO INT NOT NULL);" +
				"ALTER TABLE EVALUABLES_NODOS ADD CONSTRAINT IDX_EVALUABLES_NODOS_FK0 FOREIGN KEY (EVALUABLE) REFERENCES EVALUABLES (ID);" +
				"ALTER TABLE EVALUABLES_NODOS ADD CONSTRAINT IDX_EVALUABLES_NODOS_FK1 FOREIGN KEY (NODO) REFERENCES PROGRAMACION_NODO (ID);" +
				"ALTER TABLE EVALUABLES ALTER COLUMN ENUNCIADO VARCHAR(2048);" +
				"ALTER TABLE EVALUACIONES_EVALUABLES ALTER COLUMN OBSERVACIONES VARCHAR(2048);"+
				"ALTER TABLE PROGRAMACION_NODO ALTER COLUMN SUMARIO VARCHAR(10240);"+
				"ALTER TABLE EVENTOS ALTER COLUMN PREPARATIVOS VARCHAR(2048);"+
				"ALTER TABLE EVENTOS ALTER COLUMN SUMARIO VARCHAR(2048);"+
				"ALTER TABLE EVENTOS ALTER COLUMN CONCLUSION VARCHAR(2048);";
			System.out.println("from version 0.2 ... "+sqlEjecuta(sql));
		} else if (this.bbddVersion_0_4()) {
			sql="ALTER TABLE EVALUABLES ALTER COLUMN ENUNCIADO VARCHAR(2048);" +
				"ALTER TABLE EVALUACIONES_EVALUABLES ALTER COLUMN OBSERVACIONES VARCHAR(2048);"+
				"ALTER TABLE PROGRAMACION_NODO ALTER COLUMN SUMARIO VARCHAR(10240);"+
				"ALTER TABLE EVENTOS ALTER COLUMN PREPARATIVOS VARCHAR(2048);"+
				"ALTER TABLE EVENTOS ALTER COLUMN SUMARIO VARCHAR(2048);"+
				"ALTER TABLE EVENTOS ALTER COLUMN CONCLUSION VARCHAR(2048);";
			System.out.println("from version 0.3 o 0.4 ... "+sqlEjecuta(sql));
		}

		sql="SELECT * FROM CONFIGURACION;";
		Vector<Vector<String>> confi = this.sqlConsulta(sql);
		if (confi.size()>0) {
			//System.out.print(" mesInicio="+confi.get(0).get(1)+", meses="+confi.get(0).get(2)+", año="+confi.get(0).get(3)+" ");
			Fecha.MesInicioCurso=Integer.valueOf(confi.get(0).get(1));
			Fecha.MesesLectivosCurso=Integer.valueOf(confi.get(0).get(2));
			Fecha.AgnoInicioCurso=Integer.valueOf(confi.get(0).get(3));
		} else {
			sql="INSERT INTO CONFIGURACION (MESINICIOCURSO,MESESCURSO,AGNOINICIOCURSO) VALUES ("+Fecha.MesInicioCurso+","+Fecha.MesesLectivosCurso+","+Fecha.AgnoInicioCurso+");";
			System.out.println("Configuration empty, add default..."+sqlEjecuta(sql));
		}
	}
}
